# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


from __future__ import absolute_import

# python 2 and python 3 compatibility library
import six

from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.configuration import Configuration


class CustomerApi(object):

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def create_customer(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_customer_request, x_opay_headers, **kwargs):  # noqa: E501
        """Create Customer  # noqa: E501

        The Create Customer API is used to register a customer with a biller in EBPP. It is also mandatory to create a customer account for the customer. So, at least one customer account should be provided in order to register a customer. There is no limit on the number of accounts that can be associated with the customer. Either first_name or last_name is required to create a customer.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.create_customer(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_customer_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param CreateCustomerRequest create_customer_request: The JSON that contains all the attributes of customer to be created. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: CustomerResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_customer_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.create_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_customer_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def create_customer_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_customer_request, x_opay_headers, **kwargs):  # noqa: E501
        """Create Customer  # noqa: E501

        The Create Customer API is used to register a customer with a biller in EBPP. It is also mandatory to create a customer account for the customer. So, at least one customer account should be provided in order to register a customer. There is no limit on the number of accounts that can be associated with the customer. Either first_name or last_name is required to create a customer.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.create_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_customer_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param CreateCustomerRequest create_customer_request: The JSON that contains all the attributes of customer to be created. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: CustomerResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'create_customer_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_customer" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'create_customer_request' in params:
            body_params = params['create_customer_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CustomerResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_customer(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, x_opay_headers, **kwargs):  # noqa: E501
        """Get Customer  # noqa: E501

        The Get Customer API is used to retrieve the details of the customer based on the id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.get_customer(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: CustomerResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.get_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def get_customer_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, x_opay_headers, **kwargs):  # noqa: E501
        """Get Customer  # noqa: E501

        The Get Customer API is used to retrieve the details of the customer based on the id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.get_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: CustomerResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_customer', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_customer" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_customer' in params:
            path_params['ID_CUSTOMER'] = params['id_customer']  # noqa: E501
        if path_params['ID_CUSTOMER'] is None or path_params['ID_CUSTOMER'] == "":
            raise ApiException(status="0", reason="ID_CUSTOMER missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/{ID_CUSTOMER}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CustomerResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def retrieve_customers(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, x_opay_headers, **kwargs):  # noqa: E501
        """Retrieve Customers  # noqa: E501

        This API is used to retrieve/lookup customers from EBPP. Customers can be retrieved matching criteria that includes the customer_reference, the ssn, the email or the customer account number. A unique customer can be retrieved by using search criteria as per the customer uniqueness parameters opted by the client/biller. If the requestor_type is customer, the Retrieve Customers API will return an error if it cannot retrieve a unique customer.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.retrieve_customers(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :param str page_size: The maximum number of objects returned in the query.
        :param str customer_reference: Unique ID assigned by the biller/client system for a given customer.
        :param str ssn: The SSN of the customer if the account holder is an individual or the tax ID, if the customer is a business.
        :param str email: This field contains the customer’s email address within EBPP.
        :param str account_number: Customer Account Number of the customer to get details of.
        :return: CustomersResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.retrieve_customers_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.retrieve_customers_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def retrieve_customers_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, x_opay_headers, **kwargs):  # noqa: E501
        """Retrieve Customers  # noqa: E501

        This API is used to retrieve/lookup customers from EBPP. Customers can be retrieved matching criteria that includes the customer_reference, the ssn, the email or the customer account number. A unique customer can be retrieved by using search criteria as per the customer uniqueness parameters opted by the client/biller. If the requestor_type is customer, the Retrieve Customers API will return an error if it cannot retrieve a unique customer.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.retrieve_customers_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :param str page_size: The maximum number of objects returned in the query.
        :param str customer_reference: Unique ID assigned by the biller/client system for a given customer.
        :param str ssn: The SSN of the customer if the account holder is an individual or the tax ID, if the customer is a business.
        :param str email: This field contains the customer’s email address within EBPP.
        :param str account_number: Customer Account Number of the customer to get details of.
        :return: CustomersResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'requestor', 'x_opay_headers', 'trace_id', 'page_size', 'customer_reference', 'ssn', 'email', 'account_number']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_customers" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        query_params = []
        if 'page_size' in params:
            query_params.append(('page_size', params['page_size']))  # noqa: E501

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}
        if 'customer_reference' in params:
            form_params.append(('customer_reference', params['customer_reference']))  # noqa: E501
        if 'ssn' in params:
            form_params.append(('ssn', params['ssn']))  # noqa: E501
        if 'email' in params:
            form_params.append(('email', params['email']))  # noqa: E501
        if 'account_number' in params:
            form_params.append(('account_number', params['account_number']))  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/x-www-form-urlencoded'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/lists', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CustomersResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def retrieve_customers_page(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, query_id, x_opay_headers, **kwargs):  # noqa: E501
        """Retrieve Customers Pagination  # noqa: E501

        This API is used to paginate through the list of customers returned in the Retrieve Customers API.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.retrieve_customers_page(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, query_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str query_id: query id of the Retrieve/Search Customers lookup. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :param str page_size: The maximum number of objects returned in the query.
        :param str from_index: To fetch the next set of objects that start after this object
        :param str to_index: To fetch the previous set of objects that end at this object
        :return: CustomersResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.retrieve_customers_page_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, query_id, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.retrieve_customers_page_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, query_id, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def retrieve_customers_page_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, query_id, x_opay_headers, **kwargs):  # noqa: E501
        """Retrieve Customers Pagination  # noqa: E501

        This API is used to paginate through the list of customers returned in the Retrieve Customers API.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.retrieve_customers_page_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, query_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str query_id: query id of the Retrieve/Search Customers lookup. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :param str page_size: The maximum number of objects returned in the query.
        :param str from_index: To fetch the next set of objects that start after this object
        :param str to_index: To fetch the previous set of objects that end at this object
        :return: CustomersResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'query_id', 'requestor', 'x_opay_headers', 'trace_id', 'page_size', 'from_index', 'to_index']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_customers_page" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        query_params = []
        if 'page_size' in params:
            query_params.append(('page_size', params['page_size']))  # noqa: E501
        if 'query_id' in params:
            query_params.append(('query_id', params['query_id']))  # noqa: E501
        if 'from_index' in params:
            query_params.append(('from_index', params['from_index']))  # noqa: E501
        if 'to_index' in params:
            query_params.append(('to_index', params['to_index']))  # noqa: E501

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/lists', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CustomersResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_customer(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, update_customer_request, x_opay_headers, **kwargs):  # noqa: E501
        """Update Customer  # noqa: E501

        The Update Customer API is used to edit the customer''s personal details, accounts or the customer''s status. Only the information being updated needs to be sent in the request, along with the id of the customer being updated. A customer can also be associated with new customer accounts using the API. <br><br> Along with Customer details ,  Customer and Customer Account statuses also can be updated using the Update Customer API. Below are the possible customer state transformations.  | from_status | to_status | allowed requestor_type | | ------------ | ------------ | ------------ | | active | suspended| <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> | | active | revoked | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | suspended | active | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | suspended | revoked | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | revoked | active | <ul> <li>external_user</li> <li> client_agent</li></ul> </ul> |  Below are the possible customer account state transformation.  | from_status | to_status | allowed requestor_type | | ------------ | ------------ | ------------ | | active | inactive| <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> | | inactive | active | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | active | incollections | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | incollections | active | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | inactive | incollections | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | incollections | inactive | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | active | inbankruptcy  | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | incollections | inbankruptcy  | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | inbankruptcy | active  | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | inbankruptcy | closed  | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> |   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.update_customer(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, update_customer_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param UpdateCustomerRequest update_customer_request: JSON containing all the attributes of the customer to be updated. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: CustomerResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, update_customer_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.update_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, update_customer_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def update_customer_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, update_customer_request, x_opay_headers, **kwargs):  # noqa: E501
        """Update Customer  # noqa: E501

        The Update Customer API is used to edit the customer''s personal details, accounts or the customer''s status. Only the information being updated needs to be sent in the request, along with the id of the customer being updated. A customer can also be associated with new customer accounts using the API. <br><br> Along with Customer details ,  Customer and Customer Account statuses also can be updated using the Update Customer API. Below are the possible customer state transformations.  | from_status | to_status | allowed requestor_type | | ------------ | ------------ | ------------ | | active | suspended| <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> | | active | revoked | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | suspended | active | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | suspended | revoked | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | revoked | active | <ul> <li>external_user</li> <li> client_agent</li></ul> </ul> |  Below are the possible customer account state transformation.  | from_status | to_status | allowed requestor_type | | ------------ | ------------ | ------------ | | active | inactive| <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> | | inactive | active | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | active | incollections | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | incollections | active | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | inactive | incollections | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | incollections | inactive | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | active | inbankruptcy  | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | incollections | inbankruptcy  | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | inbankruptcy | active  | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> | | inbankruptcy | closed  | <ul> <li>external_user</li> <li> client_agent</li> <li> system</li></ul> </ul> |   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = CustomerApi(api_client=api_client)
        >>> thread = api.update_customer_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, update_customer_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param UpdateCustomerRequest update_customer_request: JSON containing all the attributes of the customer to be updated. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: CustomerResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_customer', 'update_customer_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_customer" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_customer' in params:
            path_params['ID_CUSTOMER'] = params['id_customer']  # noqa: E501
        if path_params['ID_CUSTOMER'] is None or path_params['ID_CUSTOMER'] == "":
            raise ApiException(status="0", reason="ID_CUSTOMER missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'update_customer_request' in params:
            body_params = params['update_customer_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/{ID_CUSTOMER}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='CustomerResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
