# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


from __future__ import absolute_import

# python 2 and python 3 compatibility library
import six

from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.configuration import Configuration


class PaymentApi(object):

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def create_payment(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, create_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Create Payment  # noqa: E501

        The API is used to make payments to service a customer account. Client systems can provide the unique identifier for the payment in their system as payment_reference.<p>Fee amount and fee type are conditional and should be provided if the details of the fee to be charged are maintained in the client system. The fee types that are supported in EBPP are add_to_principal and bill_separately. Alternatively, the fee information can be setup in EBPP, in which case, these fields are ignored. If fee information is setup in the system, the details of the fee are returned in the response.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.create_payment(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, create_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param CreatePaymentRequest create_payment_request: JSON containing all the fields required to make a payment. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, create_payment_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.create_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, create_payment_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def create_payment_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, create_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Create Payment  # noqa: E501

        The API is used to make payments to service a customer account. Client systems can provide the unique identifier for the payment in their system as payment_reference.<p>Fee amount and fee type are conditional and should be provided if the details of the fee to be charged are maintained in the client system. The fee types that are supported in EBPP are add_to_principal and bill_separately. Alternatively, the fee information can be setup in EBPP, in which case, these fields are ignored. If fee information is setup in the system, the details of the fee are returned in the response.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.create_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, create_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param CreatePaymentRequest create_payment_request: JSON containing all the fields required to make a payment. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_customer', 'create_payment_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_payment" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_customer' in params:
            path_params['ID_CUSTOMER'] = params['id_customer']  # noqa: E501
        if path_params['ID_CUSTOMER'] is None or path_params['ID_CUSTOMER'] == "":
            raise ApiException(status="0", reason="ID_CUSTOMER missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'create_payment_request' in params:
            body_params = params['create_payment_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/{ID_CUSTOMER}/payments', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PaymentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_payment(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, delete_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Delete Payment  # noqa: E501

        The API is used to delete or cancel a payment. Payments that are under processing or have already been processed cannot be  deleted.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.delete_payment(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, delete_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_payment: The unique identifier assigned by EBPP to the payment. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param DeletePaymentRequest delete_payment_request: Details of payment to be deleted/cancelled (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: DeleteResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, delete_payment_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, delete_payment_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def delete_payment_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, delete_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Delete Payment  # noqa: E501

        The API is used to delete or cancel a payment. Payments that are under processing or have already been processed cannot be  deleted.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.delete_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, delete_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_payment: The unique identifier assigned by EBPP to the payment. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param DeletePaymentRequest delete_payment_request: Details of payment to be deleted/cancelled (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: DeleteResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_payment', 'id_customer', 'delete_payment_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_payment" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_payment' in params:
            path_params['ID_PAYMENT'] = params['id_payment']  # noqa: E501
        if path_params['ID_PAYMENT'] is None or path_params['ID_PAYMENT'] == "":
            raise ApiException(status="0", reason="ID_PAYMENT missing")

        if 'id_customer' in params:
            path_params['ID_CUSTOMER'] = params['id_customer']  # noqa: E501
        if path_params['ID_CUSTOMER'] is None or path_params['ID_CUSTOMER'] == "":
            raise ApiException(status="0", reason="ID_CUSTOMER missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'delete_payment_request' in params:
            body_params = params['delete_payment_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/{ID_CUSTOMER}/payments/{ID_PAYMENT}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='DeleteResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_record_payment(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, delete_recorded_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Delete Recorded Payment  # noqa: E501

        The API is used to delete the recorded payment. It returns 204 http status code with empty body in successful case  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.delete_record_payment(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, delete_recorded_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_payment: The unique identifier assigned by EBPP to the payment. (required)
        :param DeleteRecordedPaymentRequest delete_recorded_payment_request: Details of payment to be deleted/cancelled (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_record_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, delete_recorded_payment_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_record_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, delete_recorded_payment_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def delete_record_payment_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, delete_recorded_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Delete Recorded Payment  # noqa: E501

        The API is used to delete the recorded payment. It returns 204 http status code with empty body in successful case  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.delete_record_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, delete_recorded_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_payment: The unique identifier assigned by EBPP to the payment. (required)
        :param DeleteRecordedPaymentRequest delete_recorded_payment_request: Details of payment to be deleted/cancelled (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_payment', 'delete_recorded_payment_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method delete_record_payment" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_payment' in params:
            path_params['ID_PAYMENT'] = params['id_payment']  # noqa: E501
        if path_params['ID_PAYMENT'] is None or path_params['ID_PAYMENT'] == "":
            raise ApiException(status="0", reason="ID_PAYMENT missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'delete_recorded_payment_request' in params:
            body_params = params['delete_recorded_payment_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/recordedpayments/{ID_PAYMENT}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_payment(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, x_opay_headers, **kwargs):  # noqa: E501
        """Get Payment  # noqa: E501

        The API is used to retrieve the details of a payment or a recorded payment by id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.get_payment(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_payment: The unique identifier assigned by EBPP to the payment. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.get_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def get_payment_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, x_opay_headers, **kwargs):  # noqa: E501
        """Get Payment  # noqa: E501

        The API is used to retrieve the details of a payment or a recorded payment by id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.get_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_payment: The unique identifier assigned by EBPP to the payment. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_payment', 'id_customer', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_payment" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_payment' in params:
            path_params['ID_PAYMENT'] = params['id_payment']  # noqa: E501
        if path_params['ID_PAYMENT'] is None or path_params['ID_PAYMENT'] == "":
            raise ApiException(status="0", reason="ID_PAYMENT missing")

        if 'id_customer' in params:
            path_params['ID_CUSTOMER'] = params['id_customer']  # noqa: E501
        if path_params['ID_CUSTOMER'] is None or path_params['ID_CUSTOMER'] == "":
            raise ApiException(status="0", reason="ID_CUSTOMER missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/{ID_CUSTOMER}/payments/{ID_PAYMENT}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PaymentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def record_payment(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, record_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Record Payment  # noqa: E501

        The API is used to record payments in EBPP those are made and fulfilled outside EBPP.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.record_payment(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, record_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param RecordPaymentRequest record_payment_request: JSON containing all the fields required to record a payment. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: RecordedPaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.record_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, record_payment_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.record_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, record_payment_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def record_payment_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, record_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Record Payment  # noqa: E501

        The API is used to record payments in EBPP those are made and fulfilled outside EBPP.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.record_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, record_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param RecordPaymentRequest record_payment_request: JSON containing all the fields required to record a payment. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: RecordedPaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'record_payment_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method record_payment" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'record_payment_request' in params:
            body_params = params['record_payment_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/recordedpayments', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='RecordedPaymentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def retrieve_fee_payment(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_fee, id_customer, x_opay_headers, **kwargs):  # noqa: E501
        """Get Fee Payment  # noqa: E501

        The API is used to retrieve the details of a fee payment by id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.retrieve_fee_payment(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_fee, id_customer, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by OrbiPay to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_fee: The unique identifier assigned by OrbiPay to the FeePayment. (required)
        :param str id_customer: The unique identifier assigned by OrbiPay to the customer. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.retrieve_fee_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_fee, id_customer, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.retrieve_fee_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_fee, id_customer, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def retrieve_fee_payment_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_fee, id_customer, x_opay_headers, **kwargs):  # noqa: E501
        """Get Fee Payment  # noqa: E501

        The API is used to retrieve the details of a fee payment by id.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.retrieve_fee_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_fee, id_customer, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by OrbiPay to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_fee: The unique identifier assigned by OrbiPay to the FeePayment. (required)
        :param str id_customer: The unique identifier assigned by OrbiPay to the customer. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_fee', 'id_customer', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_fee_payment" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_fee' in params:
            path_params['ID_FEE'] = params['id_fee']  # noqa: E501
        if path_params['ID_FEE'] is None or path_params['ID_FEE'] == "":
            raise ApiException(status="0", reason="ID_FEE missing")

        if 'id_customer' in params:
            path_params['ID_CUSTOMER'] = params['id_customer']  # noqa: E501
        if path_params['ID_CUSTOMER'] is None or path_params['ID_CUSTOMER'] == "":
            raise ApiException(status="0", reason="ID_CUSTOMER missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/{ID_CUSTOMER}/fees/{ID_FEE}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PaymentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def retrieve_payments(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, x_opay_headers, **kwargs):  # noqa: E501
        """Retrieve Payments  # noqa: E501

        The API is used to retrieve the payments and recorded payments made by a customer. Payments can be filtered using the confirmation number, customer account, funding account, status and payment schedule type, payment method, by date range and others mentioned below. If no dates are provided for retrieval, payments in the last six months and all the payments scheduled for a future date are returned. In case a payment has failed due to ACH returns, then the payment return date and the return code will be available in the response. The results are sorted in descending order of the payment_date followed by the last_modified_time.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.retrieve_payments(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :param str id_customer: The unique identifier assigned by EBPP to the customer.
        :param str id_funding_account: The unique identifier assigned by EBPP to the funding account
        :param str id_customer_account: The unique identifier assigned by EBPP to the customer account
        :param str confirmation_number: The confirmation number or reference provided to the customer for the successful payment.
        :param list[str] status: The status of the payment. This can take multiple values in the format key=value1&key=value2....
        :param list[str] payment_schedule_type: The schedule type for the payment setup. This can take multiple values in the format key=value1&key=value2....
        :param list[str] payment_method: The fulfillment method for the payment.
        :param str from_date: The date from which payments, matching the criteria specified, need to be retrieved.
        :param str to_date: The date upto which payments, matching the criteria specified, need to be retrieved.
        :param str page_size: The maximum number of objects returned in the query.
        :param str query_id: query id of the Retrieve/Search Customers lookup.
        :param str from_index: To fetch the next set of objects that start after this object
        :param str to_index: To fetch the previous set of objects that end at this object
        :return: PaymentsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.retrieve_payments_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.retrieve_payments_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def retrieve_payments_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, x_opay_headers, **kwargs):  # noqa: E501
        """Retrieve Payments  # noqa: E501

        The API is used to retrieve the payments and recorded payments made by a customer. Payments can be filtered using the confirmation number, customer account, funding account, status and payment schedule type, payment method, by date range and others mentioned below. If no dates are provided for retrieval, payments in the last six months and all the payments scheduled for a future date are returned. In case a payment has failed due to ACH returns, then the payment return date and the return code will be available in the response. The results are sorted in descending order of the payment_date followed by the last_modified_time.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.retrieve_payments_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :param str id_customer: The unique identifier assigned by EBPP to the customer.
        :param str id_funding_account: The unique identifier assigned by EBPP to the funding account
        :param str id_customer_account: The unique identifier assigned by EBPP to the customer account
        :param str confirmation_number: The confirmation number or reference provided to the customer for the successful payment.
        :param list[str] status: The status of the payment. This can take multiple values in the format key=value1&key=value2....
        :param list[str] payment_schedule_type: The schedule type for the payment setup. This can take multiple values in the format key=value1&key=value2....
        :param list[str] payment_method: The fulfillment method for the payment.
        :param str from_date: The date from which payments, matching the criteria specified, need to be retrieved.
        :param str to_date: The date upto which payments, matching the criteria specified, need to be retrieved.
        :param str page_size: The maximum number of objects returned in the query.
        :param str query_id: query id of the Retrieve/Search Customers lookup.
        :param str from_index: To fetch the next set of objects that start after this object
        :param str to_index: To fetch the previous set of objects that end at this object
        :return: PaymentsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'requestor', 'x_opay_headers', 'trace_id', 'id_customer', 'id_funding_account', 'id_customer_account', 'confirmation_number', 'status', 'payment_schedule_type', 'payment_method', 'from_date', 'to_date', 'page_size', 'query_id', 'from_index', 'to_index']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method retrieve_payments" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        query_params = []
        if 'id_customer' in params:
            query_params.append(('id_customer', params['id_customer']))  # noqa: E501
        if 'id_funding_account' in params:
            query_params.append(('id_funding_account', params['id_funding_account']))  # noqa: E501
        if 'id_customer_account' in params:
            query_params.append(('id_customer_account', params['id_customer_account']))  # noqa: E501
        if 'confirmation_number' in params:
            query_params.append(('confirmation_number', params['confirmation_number']))  # noqa: E501
        if 'status' in params:
            query_params.append(('status', params['status']))  # noqa: E501
            collection_formats['status'] = 'multi'  # noqa: E501
        if 'payment_schedule_type' in params:
            query_params.append(('payment_schedule_type', params['payment_schedule_type']))  # noqa: E501
            collection_formats['payment_schedule_type'] = 'multi'  # noqa: E501
        if 'payment_method' in params:
            query_params.append(('payment_method', params['payment_method']))  # noqa: E501
            collection_formats['payment_method'] = 'multi'  # noqa: E501
        if 'from_date' in params:
            query_params.append(('from_date', params['from_date']))  # noqa: E501
        if 'to_date' in params:
            query_params.append(('to_date', params['to_date']))  # noqa: E501
        if 'page_size' in params:
            query_params.append(('page_size', params['page_size']))  # noqa: E501
        if 'query_id' in params:
            query_params.append(('query_id', params['query_id']))  # noqa: E501
        if 'from_index' in params:
            query_params.append(('from_index', params['from_index']))  # noqa: E501
        if 'to_index' in params:
            query_params.append(('to_index', params['to_index']))  # noqa: E501

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/payments', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PaymentsResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def simple_payment(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, simple_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Simple Payment  # noqa: E501

        The API is used to make payments to service a customer account. Client systems can provide the unique identifier for the payment in their system as payment_reference.  Fee amount and fee type are conditional and should be provided if the details of the fee to be charged are maintained in the client system. The fee types that are supported in EBPP are add_to_principal and bill_separately. Alternatively, the fee information can be setup in EBPP, in which case, these fields are ignored. If fee information is setup in the system, the details of the fee are returned in the response.  This API updates the details of the customer, customer account and the funding account if those exist in EBPP or creates them otherwise.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.simple_payment(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, simple_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param SimplePaymentRequest simple_payment_request: JSON containing all the fields required to make a payment. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :param str save_funding_account: query param
        :return: SimplePaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.simple_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, simple_payment_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.simple_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, simple_payment_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def simple_payment_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, simple_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Simple Payment  # noqa: E501

        The API is used to make payments to service a customer account. Client systems can provide the unique identifier for the payment in their system as payment_reference.  Fee amount and fee type are conditional and should be provided if the details of the fee to be charged are maintained in the client system. The fee types that are supported in EBPP are add_to_principal and bill_separately. Alternatively, the fee information can be setup in EBPP, in which case, these fields are ignored. If fee information is setup in the system, the details of the fee are returned in the response.  This API updates the details of the customer, customer account and the funding account if those exist in EBPP or creates them otherwise.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.simple_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, simple_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param SimplePaymentRequest simple_payment_request: JSON containing all the fields required to make a payment. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :param str save_funding_account: query param
        :return: SimplePaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'simple_payment_request', 'requestor', 'x_opay_headers', 'trace_id', 'save_funding_account']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method simple_payment" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        query_params = []
        if 'save_funding_account' in params:
            query_params.append(('save_funding_account', params['save_funding_account']))  # noqa: E501

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'simple_payment_request' in params:
            body_params = params['simple_payment_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/payments', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='SimplePaymentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_payment(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, update_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Update Payment  # noqa: E501

        The API is used to edit a payment. Only payments in scheduled status can be edited.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.update_payment(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, update_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_payment: The unique identifier assigned by EBPP to the payment. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param UpdatePaymentRequest update_payment_request: Details of the Payment to be updated. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, update_payment_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.update_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, update_payment_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def update_payment_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, update_payment_request, x_opay_headers, **kwargs):  # noqa: E501
        """Update Payment  # noqa: E501

        The API is used to edit a payment. Only payments in scheduled status can be edited.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentApi(api_client=api_client)
        >>> thread = api.update_payment_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_payment, id_customer, update_payment_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_payment: The unique identifier assigned by EBPP to the payment. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param UpdatePaymentRequest update_payment_request: Details of the Payment to be updated. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_payment', 'id_customer', 'update_payment_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_payment" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_payment' in params:
            path_params['ID_PAYMENT'] = params['id_payment']  # noqa: E501
        if path_params['ID_PAYMENT'] is None or path_params['ID_PAYMENT'] == "":
            raise ApiException(status="0", reason="ID_PAYMENT missing")

        if 'id_customer' in params:
            path_params['ID_CUSTOMER'] = params['id_customer']  # noqa: E501
        if path_params['ID_CUSTOMER'] is None or path_params['ID_CUSTOMER'] == "":
            raise ApiException(status="0", reason="ID_CUSTOMER missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'update_payment_request' in params:
            body_params = params['update_payment_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/{ID_CUSTOMER}/payments/{ID_PAYMENT}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PaymentResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
