# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


from __future__ import absolute_import

# python 2 and python 3 compatibility library
import six

from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.configuration import Configuration


class PaymentAuthorizationApi(object):

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def cancel_payment_authorization(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, id_authorization, cancel_payment_authorization_request, x_opay_headers, **kwargs):  # noqa: E501
        """Cancel Payment Authorization  # noqa: E501

        The API is used to cancel a payment authorization. Payment Authorizations that are already been processed into Payment cannot be  cancelled. It returns 204 http status code with empty body in successful case.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentAuthorizationApi(api_client=api_client)
        >>> thread = api.cancel_payment_authorization(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, id_authorization, cancel_payment_authorization_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param str id_authorization: The unique identifier assigned by EBPP to the payment authorization. (required)
        :param CancelPaymentAuthorizationRequest cancel_payment_authorization_request: Details of payment authorization to be cancelled (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.cancel_payment_authorization_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, id_authorization, cancel_payment_authorization_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.cancel_payment_authorization_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, id_authorization, cancel_payment_authorization_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def cancel_payment_authorization_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, id_authorization, cancel_payment_authorization_request, x_opay_headers, **kwargs):  # noqa: E501
        """Cancel Payment Authorization  # noqa: E501

        The API is used to cancel a payment authorization. Payment Authorizations that are already been processed into Payment cannot be  cancelled. It returns 204 http status code with empty body in successful case.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentAuthorizationApi(api_client=api_client)
        >>> thread = api.cancel_payment_authorization_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, id_customer, id_authorization, cancel_payment_authorization_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param str id_customer: The unique identifier assigned by EBPP to the customer. (required)
        :param str id_authorization: The unique identifier assigned by EBPP to the payment authorization. (required)
        :param CancelPaymentAuthorizationRequest cancel_payment_authorization_request: Details of payment authorization to be cancelled (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'id_customer', 'id_authorization', 'cancel_payment_authorization_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method cancel_payment_authorization" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        if 'id_customer' in params:
            path_params['ID_CUSTOMER'] = params['id_customer']  # noqa: E501
        if path_params['ID_CUSTOMER'] is None or path_params['ID_CUSTOMER'] == "":
            raise ApiException(status="0", reason="ID_CUSTOMER missing")

        if 'id_authorization' in params:
            path_params['ID_AUTHORIZATION'] = params['id_authorization']  # noqa: E501
        if path_params['ID_AUTHORIZATION'] is None or path_params['ID_AUTHORIZATION'] == "":
            raise ApiException(status="0", reason="ID_AUTHORIZATION missing")

        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'cancel_payment_authorization_request' in params:
            body_params = params['cancel_payment_authorization_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/customers/{ID_CUSTOMER}/payments/auth/{ID_AUTHORIZATION}/cancel', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def create_payment_authorization(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_payment_authorization_request, x_opay_headers, **kwargs):  # noqa: E501
        """Create Payment Authorization  # noqa: E501

        The API is used to create a payment authorization with certain amount on a given card account. Client systems can provide the unique identifier for the payment authorization in their system as payment_auth_reference.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentAuthorizationApi(api_client=api_client)
        >>> thread = api.create_payment_authorization(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_payment_authorization_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param CreatePaymentAuthorizationRequest create_payment_authorization_request: JSON containing all the fields required to make a payment. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentAuthorizationResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.create_payment_authorization_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_payment_authorization_request, x_opay_headers, **kwargs)  # noqa: E501
        else:
            (data) = self.create_payment_authorization_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_payment_authorization_request, x_opay_headers, **kwargs)  # noqa: E501
            return data

    def create_payment_authorization_with_http_info(self, channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_payment_authorization_request, x_opay_headers, **kwargs):  # noqa: E501
        """Create Payment Authorization  # noqa: E501

        The API is used to create a payment authorization with certain amount on a given card account. Client systems can provide the unique identifier for the payment authorization in their system as payment_auth_reference.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> configuration = Configuration()
        >>> # configuration.secret = secret
        >>> # configuration.host = host
        >>> api_client = ApiClient(configuration=configuration)
        >>> api = PaymentAuthorizationApi(api_client=api_client)
        >>> thread = api.create_payment_authorization_with_http_info(channel, client_key, product, timestamp, idempotent_request_key, requestor_type, create_payment_authorization_request, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str channel: The channel through which the API is invoked. (required)
        :param str client_key: The unique identifier assigned by EBPP to the client. (required)
        :param str product: The product identifier corresponding to the API. (required)
        :param str timestamp: The timestamp for the moment when the API request is created. (required)
        :param str idempotent_request_key: The unique token that clients can generate and maintain in order to identify an API request. (required)
        :param str requestor_type: Type of the requestor of the API. (required)
        :param CreatePaymentAuthorizationRequest create_payment_authorization_request: JSON containing all the fields required to make a payment. (required)
        :param str requestor: The identifier for the requestor of the API.
        :param str x_opay_headers: Intended for the future use.
        :param str trace_id: The unique reference that can be used for tracing and debugging an API call.
        :return: PaymentAuthorizationResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['channel', 'client_key', 'product', 'timestamp', 'idempotent_request_key', 'requestor_type', 'create_payment_authorization_request', 'requestor', 'x_opay_headers', 'trace_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_payment_authorization" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}
        
        query_params = []

        header_params = {}
        if 'channel' in params:
            header_params['channel'] = params['channel']  # noqa: E501
        if 'client_key' in params:
            header_params['client_key'] = params['client_key']  # noqa: E501
        if 'product' in params:
            header_params['product'] = params['product']  # noqa: E501
        if 'timestamp' in params:
            header_params['timestamp'] = params['timestamp']  # noqa: E501
        if 'idempotent_request_key' in params:
            header_params['idempotent_request_key'] = params['idempotent_request_key']  # noqa: E501
        if 'requestor_type' in params:
            header_params['requestor_type'] = params['requestor_type']  # noqa: E501
        if 'requestor' in params:
            header_params['requestor'] = params['requestor']  # noqa: E501
        if 'x_opay_headers' in params:
            header_params['X-OPAY-Headers'] = params['x_opay_headers']  # noqa: E501
        if 'trace_id' in params:
            header_params['trace_id'] = params['trace_id']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        if 'create_payment_authorization_request' in params:
            body_params = params['create_payment_authorization_request']
        # HTTP header `Accept`
        header_params['Accept'] = ApiClient.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = ApiClient.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/payments/auth', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PaymentAuthorizationResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
