# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""

from __future__ import absolute_import
# 50
import datetime
import mimetypes
import os
import re
import tempfile
import base64
import hashlib
import hmac
import json
from multiprocessing.pool import ThreadPool


# python 2 and python 3 compatibility library
import six
from six.moves.urllib.parse import quote
from six import string_types
from six import iterkeys

from orbipay_paymentsapi_client.configuration import Configuration
import orbipay_paymentsapi_client.models
from orbipay_paymentsapi_client import rest, logger_util
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.common_util import CommonUtil

DEFAULT_CONNECT_TIMEOUT_IN_SECONDS = 10
DEFAULT_READ_TIMEOUT_IN_SECONDS = 50

API_CLIENT_CONNECT_TIMEOUT = "ORBIPAY_PAYMENTS_API_CONNECT_TIMEOUT_SECONDS"
API_CLIENT_READ_TIMEOUT = "ORBIPAY_PAYMENTS_API_READ_TIMEOUT_SECONDS"


class ApiClient(object):
    """
    :param configuration: .Configuration object for this client
    :param header_name: a header to pass when making calls to the API.
    :param header_value: a header value to pass when making calls to
        the API.
    :param cookie: a cookie to include in the header when making calls
        to the API
    """

    PRIMITIVE_TYPES = (float, bool, bytes, six.text_type) + six.integer_types
    
    NATIVE_TYPES_MAPPING = {
        'int': int,
        'long': int if six.PY3 else long,  # noqa: F821
        'float': float,
        'str': str,
        'bool': bool,
        'date': datetime.date,
        'datetime': datetime.datetime,
        'object': object,
    }
    
    VALID_REQUEST_HEADERS = \
        {
            'product',
            'trace_id',
            'idempotent_request_key',
            'channel',
            'requestor',
            'client_key',
            'X-OPAY-Headers',
            'requestor_type',
            'timestamp',
        }

    ENCODE_TYPE = 'utf-8'
    
    @staticmethod
    def is_string_or_primitive(thing):
        return (type(thing) in ApiClient.PRIMITIVE_TYPES) or isinstance(thing, string_types)

    def __init__(self, configuration=None, header_name=None, header_value=None,
                 cookie=None):
        if configuration is None:
            configuration = Configuration()
        self.configuration = configuration

        self.pool = ThreadPool()
        self.rest_client = rest.RESTClientObject(configuration)
        self.default_headers = {}
        if header_name is not None:
            self.default_headers[header_name] = header_value
        self.cookie = cookie
        # Set default User-Agent.
        self.user_agent = 'orbipay-paymentsapi-client/1.13.0/python)'

    def __del__(self):
        self.pool.close()
        self.pool.join()

    @property
    def user_agent(self):
        """User agent for this API client"""
        return self.default_headers['User-Agent']

    @user_agent.setter
    def user_agent(self, value):
        self.default_headers['User-Agent'] = value

    def set_default_header(self, header_name, header_value):
        self.default_headers[header_name] = header_value

    def __call_api(
            self, resource_path, method, path_params=None,
            query_params=None, header_params=None, body=None, post_params=None,
            files=None, response_type=None, auth_settings=None,
            _return_http_data_only=None, collection_formats=None,
            _preload_content=True, _request_timeout=None):

        config = self.configuration

        # header parameters
        header_params = header_params or {}
        header_params.update(self.default_headers)
        if self.cookie:
            header_params['Cookie'] = self.cookie
        if header_params:
            header_params = self.sanitize_for_serialization(header_params)
            header_params = dict(ApiClient.parameters_to_tuples(header_params, collection_formats))

        # path parameters
        if path_params:
            path_params = self.sanitize_for_serialization(path_params)
            path_params = ApiClient.parameters_to_tuples(path_params, collection_formats)
            for k, v in path_params:
                # specified safe chars, encode everything
                resource_path = resource_path.replace(
                    '{%s}' % k,
                    quote(str(v), safe=config.safe_chars_for_path_param)
                )

        # query parameters
        if query_params:
            query_params = self.sanitize_for_serialization(query_params)
            query_params = ApiClient.parameters_to_tuples(query_params, collection_formats)

        # post parameters
        if post_params or files:
            post_params = ApiClient.prepare_post_parameters(post_params, files)
            post_params = self.sanitize_for_serialization(post_params)
            post_params = ApiClient.parameters_to_tuples(post_params, collection_formats)

        # body
        if body:
            body = self.sanitize_for_serialization(body)

        # add Authentication Params into header parameters
        client_key = header_params["client_key"]
        client_api_key = self.configuration.client_api_key
        secret = self.configuration.secret
        algorithm = self.configuration.algorithm
        
        masked_authorization_string = ""
        algorithm1 = "OPAY1-HMAC-SHA256"
        algorithm2 = "OPAY2-HMAC-SHA256"
        algorithm3 = "OPAY3-HMAC-SHA256"

        if client_api_key is None or client_api_key == "":
            client_api_key = client_key

        if algorithm is None or algorithm == "" or algorithm == algorithm1:
            auth_list = self.compute_opay1_hmac_sha256_hash(resource_path, method, query_params, header_params, post_params, body)
            client_hash = auth_list[0]
            masked_authorization_string = auth_list[1]
            header_params["Authorization"] = algorithm1 + " Credential=" + client_api_key + ",Signature=" + client_hash.strip()	
        elif algorithm == algorithm2:
            client_hash = self.compute_opay2_hmac_sha256_hash(client_key, client_api_key, secret, header_params["timestamp"])
            header_params["Authorization"] = algorithm2 + " Credential=" + client_api_key + ",Signature=" + client_hash.strip()	
        elif algorithm == algorithm3:
            header_params["Authorization"] = algorithm3 + " Credential=" + client_api_key 
        else:
            raise ApiException(
                    status=0,
                    reason="Invalid auth_scheme received"
                )
        # auth setting
        self.update_params_for_auth(header_params, query_params, auth_settings)

        # request url
        url = self.configuration.host + resource_path

        if logger_util.is_debug_enabled():
            logger_util.log_debug("HTTP URL :" + str(url))
            logger_util.log_debug("HTTP Method :" + str(method))
            logger_util.log_debug("Request Headers :"+str(header_params))
            logger_util.log_debug("Query Params :"+str(query_params))
            logger_util.log_debug("Form Params :"+str(CommonUtil.get_masked_properties(post_params)))
            logger_util.log_debug("Request Body :"+str(CommonUtil.get_masked_properties(body)))
            logger_util.log_debug("Request Timeout values connection_timeout, read_timeout:" + str(_request_timeout))
        # perform request and return response
        response_data = self.request(
            method, url, query_params=query_params, headers=header_params,
            post_params=post_params, body=body,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout, masked_authorization_string=masked_authorization_string)

        self.last_response = response_data

        return_data = response_data
        if _preload_content:
            # deserialize response data...
            if response_type:
                return_data = self.deserialize(response_data, response_type)
            else:
                return_data = None

        response_dict = dict()
        response_dict['data'] = return_data
        response_dict['status'] = response_data.status
        response_dict['headers'] = response_data.getheaders()

        return response_dict

    def sanitize_for_serialization(self, obj):
        """Builds a JSON POST object.

        If obj is None, return None.
        If obj is str, int, long, float, bool, return directly.
        If obj is datetime.datetime, datetime.date
            convert to string in iso8601 format.
        If obj is list, sanitize each element in the list.
        If obj is dict, return the dict.
        If obj is swagger model, return the properties dict.

        :param obj: The data to serialize.
        :return: The serialized form of data.
        """
        if obj is None:
            return None
        elif isinstance(obj, self.PRIMITIVE_TYPES):
            return obj
        elif isinstance(obj, list):
            return [self.sanitize_for_serialization(sub_obj)
                    for sub_obj in obj]
        elif isinstance(obj, tuple):
            return tuple(self.sanitize_for_serialization(sub_obj)
                         for sub_obj in obj)
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()

        if isinstance(obj, dict):
            obj_dict = obj
        else:
            # Convert model obj to dict except
            # attributes `swagger_types`, `attribute_map`
            # and attributes which value is not None.
            # Convert attribute name to json key in
            # model definition for request.
            obj_dict = {obj.attribute_map[attr]: getattr(obj, attr)
                        for attr, _ in six.iteritems(obj.swagger_types)
                        if getattr(obj, attr) is not None}

        return {key: self.sanitize_for_serialization(val)
                for key, val in six.iteritems(obj_dict)}

    def compute_opay1_hmac_sha256_hash(self, path, method, query_params, header_params, form_params, body):
        try:
            path = "/payments/v1" + path
            separator = ":"
            params = list()
            params.append(method.upper().strip(' '))
            params.append(path.strip(' '))
            params.append(ApiClient.get_query_params_string(query_params))
            params.append(ApiClient.get_header_params_string(header_params))
            body_text = json.dumps(body)
            masked_body_text = json.dumps(CommonUtil.get_masked_properties(body))
            if form_params:
                masked_params = params[:]
                masked_params.append(ApiClient.get_form_params_string(CommonUtil.get_masked_properties(form_params)))
                masked_param_string = separator.join(masked_params)
                params.append(ApiClient.get_form_params_string(form_params))
                param_string = separator.join(params)
            elif body_text and "null" != body_text:
                masked_params = params[:]
                masked_params.append(masked_body_text)
                masked_param_string = separator.join(masked_params)
                params.append(body_text)
                param_string = separator.join(params)
            else:
                param_string = separator.join(params) + separator
                masked_param_string = separator.join(params) + separator
            if self.configuration.secret is None or self.configuration.secret == "":
                raise ApiException(
                    status=0,
                    reason="Missing secret while generating signature"
                )
            hmac_sha_256_hash = ApiClient.decrypt_encoded_data(secret=self.configuration.secret, message=param_string)
            return [hmac_sha_256_hash, masked_param_string]
        except BaseException as e:
            raise ApiException(
                status=0,
                reason="Failed to compute HMACSHA256 hash {}".format(e)
            )

    def compute_opay2_hmac_sha256_hash(self, client_key, client_api_key, secret, timestamp ):
        try:
            separator = ":"
            params = list()
            if client_key is None or client_key == "":
                raise ApiException(
                    status=0,
                    reason="Missing client_key while generating signature"
                )
            if secret is None or secret == "":
                raise ApiException(
                    status=0,
                    reason="Missing secret while generating signature"
                )
            if client_api_key is None or client_api_key == "":
                raise ApiException(
                    status=0,
                    reason="Missing api_key while generating signature"
                )
            params.append(client_key.strip(' '))
            params.append(client_api_key.strip(' '))
            params.append(secret.strip(' '))
            params.append(timestamp.strip(' '))
            param_string = separator.join(params)
            hmac_sha_256_hash = ApiClient.decrypt_encoded_data(secret=secret, message=param_string)
            return hmac_sha_256_hash
        except BaseException as e:
            raise ApiException(
                status=0,
                reason="Failed to compute HMACSHA256 hash {}".format(e)
            )

    @staticmethod
    def get_header_params_string(params):
        param_string = ""
        try:
            filtered_params = {}
            for key in params:
                stripped_key = key.strip()
                if ApiClient.is_valid_header(stripped_key):  # and params[key] ( remove me )
                    value = params[key]
                    if isinstance(value, string_types):
                        stripped_value = value.strip()
                        if stripped_value != "":
                            filtered_params[stripped_key] = stripped_value
                    elif value is not None:
                        filtered_params[stripped_key] = value

            for key in sorted(iterkeys(filtered_params)):
                value = filtered_params[key]
                param_string += key + "=" + str(value) + "&"

            if param_string.endswith("&"):
                param_string = param_string[:-1]

            return param_string
        except BaseException as e:
            raise ApiException(
                status=0,
                reason="Failed to get_header_params_string {}".format(e)
            )

    @staticmethod
    def get_query_params_string(params):
        param_string = ""
        try:
            filtered_params = {}
            for param in params:
                if param is not None and len(param) == 2:
                    k = param[0]
                    v = param[1]
                    if isinstance(k, string_types):
                        stripped_key = k.strip()
                        if stripped_key != "":
                            if isinstance(v, string_types):
                                stripped_value = v.strip()
                                if stripped_value != "":
                                    if stripped_key not in filtered_params.keys():
                                        filtered_params[stripped_key] = []
                                    filtered_params[stripped_key].append(stripped_value)
                            elif v is not None:
                                if stripped_key not in filtered_params.keys():
                                    filtered_params[stripped_key] = []
                                filtered_params[stripped_key].append(v)

            for key in sorted(iterkeys(filtered_params)):
                value = filtered_params[key]
                for element in sorted(value):
                    param_string += key + "=" + str(element) + "&"

            if param_string.endswith("&"):
                param_string = param_string[:-1]

            return param_string
        except BaseException as e:
            raise ApiException(
                status=0,
                reason="Failed to get_query_params_string {}".format(e)
            )

    @staticmethod
    def get_form_params_string(form_params):
        form_string = ""
        try:
            for param in form_params:
                if param is not None and len(param) == 2:
                    k = param[0]
                    v = param[1]
                    if v is None or ApiClient.is_string_or_primitive(v):
                        form_string += k + "=" + str(v) + "&"

            if form_string.endswith("&"):
                form_string = form_string[:-1]

            return form_string
        except BaseException as e:
            raise ApiException(
                status=0,
                reason="Failed to get_form_params_string {}".format(e)
            )

    @staticmethod
    def decrypt_encoded_data(secret, message):
        try:
            dig = hmac.new(secret.encode(ApiClient.ENCODE_TYPE),
                           msg=message.encode(ApiClient.ENCODE_TYPE), digestmod=hashlib.sha256).digest()
            base64_encoded_data = base64.b64encode(dig).decode()
            return base64_encoded_data
        except BaseException as e:
            raise ApiException(
                status=0,
                reason="Failed to decrypt_encoded_data {}".format(e)
            )

    @staticmethod
    def is_valid_header(header):
        if header in ApiClient.VALID_REQUEST_HEADERS:
            return True
        return False

    def deserialize(self, response, response_type):
        """Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: class literal for
            deserialized object, or string of class name.

        :return: deserialized object.
        """
        # handle file downloading
        # save response body into a tmp file and return the instance
        if response_type == "file":
            return self.__deserialize_file(response)

        # fetch data from response object
        try:
            data = json.loads(response.data)
        except ValueError:
            data = response.data

        return self.__deserialize(data, response_type)

    def __deserialize(self, data, klass):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.

        :return: object.
        """
        if data is None:
            return None

        if type(klass) == str:
            if klass.startswith('list['):
                sub_kls = re.match(r'list\[(.*)\]', klass).group(1)
                return [self.__deserialize(sub_data, sub_kls)
                        for sub_data in data]

            if klass.startswith('dict('):
                sub_kls = re.match(r'dict\(([^,]*), (.*)\)', klass).group(2)
                return {k: self.__deserialize(v, sub_kls)
                        for k, v in six.iteritems(data)}

            # convert str to class
            if klass in self.NATIVE_TYPES_MAPPING:
                klass = self.NATIVE_TYPES_MAPPING[klass]
            else:
                klass = getattr(orbipay_paymentsapi_client.models, klass)

        if klass in self.PRIMITIVE_TYPES:
            return ApiClient.__deserialize_primitive(data, klass)
        elif klass == object:
            return ApiClient.__deserialize_object(data)
        elif klass == datetime.date:
            return ApiClient.__deserialize_date(data)
        elif klass == datetime.datetime:
            return ApiClient.__deserialize_datatime(data)
        else:
            return self.__deserialize_model(data, klass)

    def call_api(self, resource_path, method,
                 path_params=None, query_params=None, header_params=None,
                 body=None, post_params=None, files=None,
                 response_type=None, auth_settings=None, async_req=None,
                 _return_http_data_only=None, collection_formats=None,
                 _preload_content=True, _request_timeout=None):
        """Makes the HTTP request (synchronous) and returns deserialized data.

        To make an async request, set the async_req parameter.

        :param resource_path: Path to method endpoint.
        :param method: Method to call.
        :param path_params: Path parameters in the url.
        :param query_params: Query parameters in the url.
        :param header_params: Header parameters to be
            placed in the request header.
        :param body: Request body.
        :param post_params: dict Request post form parameters,
            for `application/x-www-form-urlencoded`, `multipart/form-data`.
        :param files: dict key -> filename, value -> filepath,
            for `multipart/form-data`.
        :param response_type: Response data type.     
        :param auth_settings: list Auth Settings names for the request.
        :param async_req: bool execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param collection_formats: dict of collection formats for path, query,
            header, and post parameters.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return:
            If async_req parameter is True,
            the request will be called asynchronously.
            The method will return the request thread.
            If parameter async_req is False or missing,
            then the method will return the response directly.
        """
        
        _request_timeout = (ApiClient.get_connection_timeout_value(), ApiClient.get_read_timeout_value())
        
        if not async_req:
            return self.__call_api(resource_path, method,
                                   path_params, query_params, header_params,
                                   body, post_params, files,
                                   response_type, auth_settings,
                                   _return_http_data_only, collection_formats,
                                   _preload_content, _request_timeout)
        else:
            thread = self.pool.apply_async(self.__call_api, (resource_path,
                                           method, path_params, query_params,
                                           header_params, body,
                                           post_params, files,
                                           response_type, auth_settings,
                                           _return_http_data_only,
                                           collection_formats,
                                           _preload_content, _request_timeout))
        return thread

    def request(self, method, url, query_params=None, headers=None,
                post_params=None, body=None, _preload_content=True,
                _request_timeout=None, masked_authorization_string=None):
        """Makes the HTTP request using RESTClient."""
        if method == "GET":
            return self.rest_client.get(url,
                                        query_params=query_params,
                                        _preload_content=_preload_content,
                                        _request_timeout=_request_timeout,
                                        headers=headers, masked_authorization_string=masked_authorization_string)
        elif method == "HEAD":
            return self.rest_client.head(url,
                                         query_params=query_params,
                                         _preload_content=_preload_content,
                                         _request_timeout=_request_timeout,
                                         headers=headers, masked_authorization_string=masked_authorization_string)
        elif method == "OPTIONS":
            return self.rest_client.options(url,
                                            query_params=query_params,
                                            headers=headers,
                                            post_params=post_params,
                                            _preload_content=_preload_content,
                                            _request_timeout=_request_timeout,
                                            body=body, masked_authorization_string=masked_authorization_string)
        elif method == "POST":
            return self.rest_client.post(url,
                                         query_params=query_params,
                                         headers=headers,
                                         post_params=post_params,
                                         _preload_content=_preload_content,
                                         _request_timeout=_request_timeout,
                                         body=body, masked_authorization_string=masked_authorization_string)
        elif method == "PUT":
            return self.rest_client.put(url,
                                        query_params=query_params,
                                        headers=headers,
                                        post_params=post_params,
                                        _preload_content=_preload_content,
                                        _request_timeout=_request_timeout,
                                        body=body, masked_authorization_string=masked_authorization_string)
        elif method == "PATCH":
            return self.rest_client.patch(url,
                                          query_params=query_params,
                                          headers=headers,
                                          post_params=post_params,
                                          _preload_content=_preload_content,
                                          _request_timeout=_request_timeout,
                                          body=body, masked_authorization_string=masked_authorization_string)
        elif method == "DELETE":
            return self.rest_client.delete(url,
                                           query_params=query_params,
                                           headers=headers,
                                           _preload_content=_preload_content,
                                           _request_timeout=_request_timeout,
                                           body=body, masked_authorization_string=masked_authorization_string)
        else:
            raise ValueError(
                "http method must be `GET`, `HEAD`, `OPTIONS`,"
                " `POST`, `PATCH`, `PUT` or `DELETE`."
            )

    @staticmethod
    def parameters_to_tuples(params, collection_formats):
        """Get parameters as list of tuples, formatting collections.

        :param params: Parameters as dict or list of two-tuples
        :param dict collection_formats: Parameter collection formats
        :return: Parameters as list of tuples, collections formatted
        """
        new_params = []
        if collection_formats is None:
            collection_formats = {}
        for k, v in six.iteritems(params) if isinstance(params, dict) else params:  # noqa: E501
            if k in collection_formats:
                collection_format = collection_formats[k]
                if v is not None:
                    if collection_format == 'multi':
                        new_params.extend((k, value) for value in v)
                    else:
                        if collection_format == 'ssv':
                            delimiter = ' '
                        elif collection_format == 'tsv':
                            delimiter = '\t'
                        elif collection_format == 'pipes':
                            delimiter = '|'
                        else:   # csv is the default
                            delimiter = ','
                        new_params.append((k, delimiter.join(str(value) for value in v)))
            else:
                if v is not None:
                    new_params.append((k, v))
        return new_params

    @staticmethod
    def prepare_post_parameters(post_params=None, files=None):
        """Builds form parameters.

        :param post_params: Normal form parameters.
        :param files: File parameters.
        :return: Form parameters with files.
        """
        params = []

        if post_params:
            params = post_params

        if files:
            for k, v in six.iteritems(files):
                if not v:
                    continue
                file_names = v if type(v) is list else [v]
                for n in file_names:
                    with open(n, 'rb') as f:
                        filename = os.path.basename(f.name)
                        filedata = f.read()
                        mimetype = (mimetypes.guess_type(filename)[0] or
                                    'application/octet-stream')
                        params.append(
                            tuple([k, tuple([filename, filedata, mimetype])]))

        return params

    @staticmethod
    def select_header_accept(accepts):
        """Returns `Accept` based on an array of accepts provided.

        :param accepts: List of headers.
        :return: Accept (e.g. application/json).
        """
        if not accepts:
            return

        accepts = [x.lower() for x in accepts]

        if 'application/json' in accepts:
            return 'application/json'
        else:
            return ', '.join(accepts)

    @staticmethod
    def select_header_content_type(content_types):
        """Returns `Content-Type` based on an array of content_types provided.

        :param content_types: List of content-types.
        :return: Content-Type (e.g. application/json).
        """
        if not content_types:
            return 'application/json'

        content_types = [x.lower() for x in content_types]

        if 'application/json' in content_types or '*/*' in content_types:
            return 'application/json'
        else:
            return content_types[0]

    def update_params_for_auth(self, headers, querys, auth_settings):
        """Updates header and query params based on authentication setting.

        :param headers: Header parameters dict to be updated.
        :param querys: Query parameters tuple list to be updated.
        :param auth_settings: Authentication setting identifiers list.
        """
        if not auth_settings:
            return

        for auth in auth_settings:
            auth_setting = self.configuration.auth_settings().get(auth)
            if auth_setting:
                if not auth_setting['value']:
                    continue
                elif auth_setting['in'] == 'header':
                    headers[auth_setting['key']] = auth_setting['value']
                elif auth_setting['in'] == 'query':
                    querys.append((auth_setting['key'], auth_setting['value']))
                else:
                    raise ValueError(
                        'Authentication token must be in `query` or `header`'
                    )

    def __deserialize_file(self, response):
        """Deserializes body to file

        Saves response body into a file in a temporary folder,
        using the filename from the `Content-Disposition` header if provided.

        :param response:  RESTResponse.
        :return: file path.
        """
        fd, path = tempfile.mkstemp(dir=self.configuration.temp_folder_path)
        os.close(fd)
        os.remove(path)

        content_disposition = response.getheader("Content-Disposition")
        if content_disposition:
            filename = re.search(r'filename=[\'"]?([^\'"\s]+)[\'"]?',
                                 content_disposition).group(1)
            path = os.path.join(os.path.dirname(path), filename)

        with open(path, "wb") as f:
            f.write(response.data)

        return path

    @staticmethod
    def __deserialize_primitive(data, klass):
        """Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.

        :return: int, long, float, str, bool.
        """
        try:
            return klass(data)
        except UnicodeEncodeError:
            return six.text_type(data)
        except TypeError:
            return data

    @staticmethod
    def __deserialize_object(value):
        """Return a original value.

        :return: object.
        """
        return value

    @staticmethod
    def __deserialize_date(string):
        """Deserializes string to date.

        :param string: str.
        :return: date.
        """
        try:
            from dateutil.parser import parse
            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0,
                reason="Failed to parse `{0}` as date object".format(string)
            )

    @staticmethod
    def __deserialize_datatime(string):
        """Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        try:
            from dateutil.parser import parse
            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0,
                reason=(
                    "Failed to parse `{0}` as datetime object"
                    .format(string)
                )
            )

    def __deserialize_model(self, data, klass):
        """Deserializes list or dict to model.

        :param data: dict, list.
        :param klass: class literal.
        :return: model object.
        """

        if not klass.swagger_types and not hasattr(klass,
                                                   'get_real_child_model'):
            return data

        kwargs = {}
        if klass.swagger_types is not None:
            for attr, attr_type in six.iteritems(klass.swagger_types):
                if (data is not None and
                        klass.attribute_map[attr] in data and
                        isinstance(data, (list, dict))):
                    value = data[klass.attribute_map[attr]]
                    kwargs[attr] = self.__deserialize(value, attr_type)

        instance = klass(**kwargs)

        if (isinstance(instance, dict) and
                klass.swagger_types is not None and
                isinstance(data, dict)):
            for key, value in data.items():
                if key not in klass.swagger_types:
                    instance[key] = value
        if hasattr(instance, 'get_real_child_model'):
            klass_name = instance.get_real_child_model(data)
            if klass_name:
                instance = self.__deserialize(data, klass_name)
        return instance      

    @staticmethod        
    def get_connection_timeout_value():
        connection_timeout = DEFAULT_CONNECT_TIMEOUT_IN_SECONDS
        try:
            if API_CLIENT_CONNECT_TIMEOUT in os.environ and os.environ[API_CLIENT_CONNECT_TIMEOUT] is not None:
                connection_time = float(os.environ[API_CLIENT_CONNECT_TIMEOUT])
                if connection_time <= 0:
                    logger_util.log_warn("connectionTimeout passed in -ve set to"
                                          " default value connectionTimeout :: " + str(connection_time))
                else:
                    connection_timeout = connection_time
        except Exception as e:
            logger_util.log_warn("can't parse the given system variable value so set to"
                                  " default value connectionTimeout :: " + str(connection_timeout))
        return connection_timeout
    
    @staticmethod
    def get_read_timeout_value():
        read_timeout = DEFAULT_READ_TIMEOUT_IN_SECONDS
        try:
            if API_CLIENT_READ_TIMEOUT in os.environ and os.environ[API_CLIENT_READ_TIMEOUT] is not None:
                read_time = float(os.environ[API_CLIENT_READ_TIMEOUT])
                if read_time <= 0:
                    logger_util.log_warn("connectionTimeout passed in -ve set to"
                                          " default value connectionTimeout :: " + str(read_time))
                else:
                    read_timeout = read_time
        except Exception as e:
            logger_util.log_warn("can't parse the given system variable value so set to"
                                  " default value connectionTimeout :: " + str(read_timeout))
        return read_timeout
