# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import datetime
import tempfile
import os
import re
import json
import six
from six import string_types
from orbipay_paymentsapi_client import rest
from time import gmtime, strftime
import orbipay_paymentsapi_client.models
import orbipay_paymentsapi_client.wrappers


class CommonUtil:

    def __init__(self):
        pass

    PRIMITIVE_TYPES = (float, bool, bytes, six.text_type) + six.integer_types
    NATIVE_TYPES_MAPPING = {
        'int': int,
        'long': int if six.PY3 else long,  # noqa: F821
        'float': float,
        'str': str,
        'bool': bool,
        'date': datetime.date,
        'datetime': datetime.datetime,
        'object': object,
    }

    @staticmethod
    def is_json(json_string):
        try:
            json.loads(json_string)
        except ValueError as e:
            return False
        return True
    
    @staticmethod
    def sanitize_for_serialization(obj):
        """Builds a JSON POST object.

        If obj is None, return None.
        If obj is str, int, long, float, bool, return directly.
        If obj is datetime.datetime, datetime.date
            convert to string in iso8601 format.
        If obj is list, sanitize each element in the list.
        If obj is dict, return the dict.
        If obj is swagger model, return the properties dict.

        :param obj: The data to serialize.
        :return: The serialized form of data.
        """
        if obj is None:
            return None
        elif isinstance(obj, CommonUtil.PRIMITIVE_TYPES):
            return obj
        elif isinstance(obj, list):
            return [CommonUtil.sanitize_for_serialization(sub_obj)
                    for sub_obj in obj]
        elif isinstance(obj, tuple):
            return tuple(CommonUtil.sanitize_for_serialization(sub_obj)
                         for sub_obj in obj)
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()

        if isinstance(obj, dict):
            obj_dict = obj
        else:
            # Convert model obj to dict except
            # attributes `swagger_types`, `attribute_map`
            # and attributes which value is not None.
            # Convert attribute name to json key in
            # model definition for request.
            obj_dict = {obj.attribute_map[attr]: getattr(obj, attr)
                        for attr, _ in six.iteritems(obj.swagger_types)
                        if getattr(obj, attr) is not None}

        return {key: CommonUtil.sanitize_for_serialization(val)
                for key, val in six.iteritems(obj_dict)}

    @staticmethod
    def __deserialize_file(configuration, response):
        """Deserializes body to file

        Saves response body into a file in a temporary folder,
        using the filename from the `Content-Disposition` header if provided.

        :param response:  RESTResponse.
        :return: file path.
        """
        fd, path = tempfile.mkstemp(dir=configuration.temp_folder_path)
        os.close(fd)
        os.remove(path)

        content_disposition = response.getheader("Content-Disposition")
        if content_disposition:
            filename = re.search(r'filename=[\'"]?([^\'"\s]+)[\'"]?',
                                 content_disposition).group(1)
            path = os.path.join(os.path.dirname(path), filename)

        with open(path, "wb") as f:
            f.write(response.data)

        return path

    @staticmethod
    def __deserialize_primitive(data, klass):
        """Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.

        :return: int, long, float, str, bool.
        """
        try:
            return klass(data)
        except UnicodeEncodeError:
            return six.text_type(data)
        except TypeError:
            return data

    @staticmethod
    def __deserialize_object(value):
        """Return a original value.

        :return: object.
        """
        return value

    @staticmethod
    def __deserialize_date(string):
        """Deserializes string to date.

        :param string: str.
        :return: date.
        """
        try:
            from dateutil.parser import parse
            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0,
                reason="Failed to parse `{0}` as date object".format(string)
            )

    @staticmethod
    def __deserialize_datatime(string):
        """Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        try:
            from dateutil.parser import parse
            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0,
                reason=(
                    "Failed to parse `{0}` as datetime object".format(string)
                )
            )

    @staticmethod
    def __deserialize_model(data, klass, input_type, class_name=None):
        """Deserializes list or dict to model.

        :param data: dict, list.
        :param klass: class literal.
        :return: model object.
        """
        if not klass.swagger_types and not hasattr(klass,
                                                   'get_real_child_model'):
            return data
        kwargs = {}
        if klass.swagger_types is not None:
            for attr, attr_type in six.iteritems(klass.swagger_types):
                if data is not None and klass.attribute_map[attr] in data and isinstance(data, (list, dict)):
                    value = data[klass.attribute_map[attr]]
                    kwargs[attr] = CommonUtil.__deserialize(value, attr_type, input_type)
        instance = None
        if input_type == 'response' or input_type == 'error_response':
            if class_name is not None:
                method = getattr(klass, '_' + class_name + '__init_object')
                if method is not None:
                    instance = method(None, **kwargs)
            else:
                instance = klass(**kwargs)
        else:
            instance = klass(**kwargs)
        if isinstance(instance, dict) and klass.swagger_types is not None and isinstance(data, dict):
            for key, value in data.items():
                if key not in klass.swagger_types:
                    instance[key] = value
        if hasattr(instance, 'get_real_child_model'):
            klass_name = instance.get_real_child_model(data)
            if klass_name:
                instance = CommonUtil.__deserialize(data, klass_name, input_type)
        return instance

    @staticmethod
    def __deserialize(data, klass, input_type):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.

        :return: object.
        """

        if data is None:
            return None
            
        class_name = klass

        if type(klass) == str:
            if klass.startswith('list['):
                sub_kls = re.match(r'list\[(.*)\]', klass).group(1)
                return [CommonUtil.__deserialize(sub_data, sub_kls, input_type)
                        for sub_data in data]

            if klass.startswith('dict('):
                sub_kls = re.match(r'dict\(([^,]*), (.*)\)', klass).group(2)
                return {k: CommonUtil.__deserialize(v, sub_kls, input_type)
                        for k, v in six.iteritems(data)}

            # convert str to class
            if klass in CommonUtil.NATIVE_TYPES_MAPPING:
                klass = CommonUtil.NATIVE_TYPES_MAPPING[klass]
            elif input_type == 'response' or input_type == 'error_response':
                klass = getattr(orbipay_paymentsapi_client.wrappers, klass)
            else:
                klass = getattr(orbipay_paymentsapi_client.models, klass)

        if klass in CommonUtil.PRIMITIVE_TYPES:
            return CommonUtil.__deserialize_primitive(data, klass)
        elif klass == object:
            return CommonUtil.__deserialize_object(data)
        elif klass == datetime.date:
            return CommonUtil.__deserialize_date(data)
        elif klass == datetime.datetime:
            return CommonUtil.__deserialize_datatime(data)
        else:
            return CommonUtil.__deserialize_model(data, klass, input_type, class_name) 		

    @staticmethod
    def deserialize(data, klass, input_type):
        return CommonUtil.__deserialize(data, klass, input_type)

    @staticmethod
    def time_stamp():
        time_stamp = datetime.datetime.now()
        time_zone = strftime("%z", gmtime())
        time_stamp = time_stamp.strftime('%Y-%m-%d %H:%M:%S.') + ('%02d' % (time_stamp.microsecond / 1000)) + (str(time_zone))
        return time_stamp

    @staticmethod
    def invocation_context(headers):
        from orbipay_paymentsapi_client.wrappers.invocation_context import InvocationContext
        invocation_context = InvocationContext()
        kwargs = {}
        if headers is not None:
            if headers.get("product") is not None and len(headers.get("product")) > 0:
                kwargs["product"] = headers.get("product")
            if headers.get("idempotent_request_key") is not None and len(headers.get("idempotent_request_key")) > 0:
                kwargs["idempotent_request_key"] = headers.get("idempotent_request_key")
            if headers.get("request_uuid") is not None and len(headers.get("request_uuid")) > 0:
                kwargs["request_uuid"] = headers.get("request_uuid")
            if headers.get("channel") is not None and len(headers.get("channel")) > 0:
                kwargs["channel"] = headers.get("channel")
            if headers.get("requestor") is not None and len(headers.get("requestor")) > 0:
                kwargs["requestor"] = headers.get("requestor")
            if headers.get("client_key") is not None and len(headers.get("client_key")) > 0:
                kwargs["client_key"] = headers.get("client_key")
            if headers.get("x_opay_headers") is not None and len(headers.get("x_opay_headers")) > 0:
                kwargs["x_opay_headers"] = headers.get("x_opay_headers")
            if headers.get("requestor_type") is not None and len(headers.get("requestor_type")) > 0:
                kwargs["requestor_type"] = headers.get("requestor_type")
            if headers.get("response_codes") is not None and len(headers.get("response_codes")) > 0:
                kwargs["response_codes"] = headers.get("response_codes")
            if headers.get("response_text") is not None and len(headers.get("response_text")) > 0:
                kwargs["response_text"] = headers.get("response_text")
            if headers.get("timestamp") is not None and len(headers.get("timestamp")) > 0:
                kwargs["timestamp"] = headers.get("timestamp")
            
            method = getattr(InvocationContext, '_' + "InvocationContext" + '__init_object')
            if method is not None:
                invocation_context = method(invocation_context, **kwargs)			    
        return invocation_context

    @staticmethod
    def mask_string(data, visible_length):
        data = str(data)
        if visible_length >= len(data):
            return "**********"
        else:
            return "**********"+data[len(data)-visible_length:]

    @staticmethod
    def get_sensitive_info():
        return {
            'account_number': 3,
            'ssn': 0,
            'card_cvv_number': 0,
            'secret': 0,
            'expiry_date': 0,
            'custom_fields': 2,
            'customer_reference': 3
        }

    @staticmethod
    def get_masked_properties(data):
        sanitized_data = CommonUtil.sanitize_for_serialization(data)
        sensitive_info = CommonUtil.get_sensitive_info()
        if isinstance(sanitized_data, dict):
            sanitized_data = CommonUtil.mask_dict_data(sanitized_data, sensitive_info)
        elif isinstance(sanitized_data, list):
            sanitized_data = CommonUtil.mask_list_data(sanitized_data, sensitive_info)
        elif isinstance(sanitized_data, tuple):
            sanitized_data = CommonUtil.mask_tuple_data(sanitized_data, sensitive_info)
        return sanitized_data

    @staticmethod
    def mask_tuple_data(data, sensitive_info):
        if isinstance(data[0], (int, long if six.PY2 else int, float, complex, string_types)):
            for s_data in sensitive_info:
                if s_data == data[0]:
                    data = data[:1]+(CommonUtil.mask_string(data[1], sensitive_info[s_data]),)
        return data

    @staticmethod
    def mask_list_data(data, sensitive_info):
        for i in range(len(data)):
            if isinstance(data[i], (int, long if six.PY2 else int, float, complex, string_types)):
                for j in sensitive_info:
                    if i == j:
                        data[i] = CommonUtil.mask_string(data[i], sensitive_info[i])
            elif isinstance(data[i], tuple):
                data[i] = CommonUtil.mask_tuple_data(data[i], sensitive_info)
            elif isinstance(data[i], list):
                data[i] = CommonUtil.mask_list_data(data[i], sensitive_info)
            elif isinstance(data[i], dict):
                data[i] = CommonUtil.mask_dict_data(data[i], sensitive_info)
        return data

    @staticmethod
    def mask_dict_data(data, sensitive_info):
        for i in data:
            if isinstance(data[i], (int, long if six.PY2 else int, float, complex, string_types)):
                for j in sensitive_info:
                    if i == j:
                        data[i] = CommonUtil.mask_string(data[i], sensitive_info[i])
            elif isinstance(data[i], dict):
                for j in sensitive_info:
                    if i == j:
                        data[i] = CommonUtil.mask_object(data[i], sensitive_info[i])
            elif isinstance(data[i], list):
                data[i] = CommonUtil.mask_list_data(data[i], sensitive_info)
            elif isinstance(data[i], tuple):
                data[i] = CommonUtil.mask_tuple_data(data[i], sensitive_info)
        return data

    @staticmethod
    def mask_object(data, visible_length):
        for i in data:
            if isinstance(data[i], (int, long if six.PY2 else int, float, complex, string_types)):
                data[i] = CommonUtil.mask_string(data[i], visible_length)
        return data
