# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.payment_authorization_api import PaymentAuthorizationApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class CancelPaymentAuthorizationHandler(Base):

	def process(self, payment_authorization):

		configuration = Configuration()
		configuration.secret = payment_authorization.invocation_context.secret
		configuration.client_api_key = payment_authorization.invocation_context.api_key
		configuration.algorithm = payment_authorization.invocation_context.auth_scheme
		configuration.host = payment_authorization.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = PaymentAuthorizationApi(api_client=api_client)
	
			channel = payment_authorization.invocation_context.channel
			client_key = payment_authorization.invocation_context.client_key
			product = payment_authorization.invocation_context.product
			if payment_authorization.invocation_context.timestamp is not None: 
				timestamp = payment_authorization.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = payment_authorization.invocation_context.idempotent_request_key
			requestor_type = payment_authorization.invocation_context.requestor_type
			id_customer = payment_authorization.id_customer
			id_authorization = payment_authorization.id_authorization
			cancel_payment_authorization_request = self.populate_body(payment_authorization)
			requestor = payment_authorization.invocation_context.requestor
			x_opay_headers = payment_authorization.invocation_context.x_opay_headers
			trace_id = payment_authorization.invocation_context.trace_id
	
			result = api_instance.cancel_payment_authorization(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_customer=id_customer, id_authorization=id_authorization, cancel_payment_authorization_request=cancel_payment_authorization_request, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id)
			response = None
			from orbipay_paymentsapi_client.wrappers.payment_authorization import PaymentAuthorization 
			if response is None:
				response = PaymentAuthorization()
			method = getattr(PaymentAuthorization, '_' + 'PaymentAuthorization' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def populate_body(payment_authorization):
		sanitized_payment_authorization = CommonUtil.sanitize_for_serialization(payment_authorization)
		cancel_payment_authorization_request = CommonUtil.deserialize(sanitized_payment_authorization, 'CancelPaymentAuthorizationRequest', 'request')
		return cancel_payment_authorization_request

