# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.funding_account_api import FundingAccountApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class DeleteFundingAccountHandler(Base):

	def process(self, funding_account):

		configuration = Configuration()
		configuration.secret = funding_account.invocation_context.secret
		configuration.client_api_key = funding_account.invocation_context.api_key
		configuration.algorithm = funding_account.invocation_context.auth_scheme
		configuration.host = funding_account.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = FundingAccountApi(api_client=api_client)
	
			channel = funding_account.invocation_context.channel
			client_key = funding_account.invocation_context.client_key
			product = funding_account.invocation_context.product
			if funding_account.invocation_context.timestamp is not None: 
				timestamp = funding_account.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = funding_account.invocation_context.idempotent_request_key
			requestor_type = funding_account.invocation_context.requestor_type
			id_funding_account = funding_account.id_funding_account
			id_customer = funding_account.id_customer
			delete_funding_account_request = self.populate_body(funding_account)
			requestor = funding_account.invocation_context.requestor
			x_opay_headers = funding_account.invocation_context.x_opay_headers
			trace_id = funding_account.invocation_context.trace_id
	
			result = api_instance.delete_funding_account(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_funding_account=id_funding_account, id_customer=id_customer, delete_funding_account_request=delete_funding_account_request, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.funding_account import FundingAccount 
			if response is None:
				response = FundingAccount()
			method = getattr(FundingAccount, '_' + 'FundingAccount' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def populate_body(funding_account):
		sanitized_funding_account = CommonUtil.sanitize_for_serialization(funding_account)
		delete_funding_account_request = CommonUtil.deserialize(sanitized_funding_account, 'DeleteFundingAccountRequest', 'request')
		return delete_funding_account_request

	@staticmethod
	def wrap_response(delete_response):
		sanitized_delete_response = CommonUtil.sanitize_for_serialization(delete_response)
		funding_account = CommonUtil.deserialize(sanitized_delete_response, 'FundingAccount', 'response')
		return funding_account
