# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.payment_api import PaymentApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class DeletePaymentHandler(Base):

	def process(self, payment):

		configuration = Configuration()
		configuration.secret = payment.invocation_context.secret
		configuration.client_api_key = payment.invocation_context.api_key
		configuration.algorithm = payment.invocation_context.auth_scheme
		configuration.host = payment.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = PaymentApi(api_client=api_client)
	
			channel = payment.invocation_context.channel
			client_key = payment.invocation_context.client_key
			product = payment.invocation_context.product
			if payment.invocation_context.timestamp is not None: 
				timestamp = payment.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = payment.invocation_context.idempotent_request_key
			requestor_type = payment.invocation_context.requestor_type
			id_payment = payment.id_payment
			id_customer = payment.id_customer
			delete_payment_request = self.populate_body(payment)
			requestor = payment.invocation_context.requestor
			x_opay_headers = payment.invocation_context.x_opay_headers
			trace_id = payment.invocation_context.trace_id
	
			result = api_instance.delete_payment(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_payment=id_payment, id_customer=id_customer, delete_payment_request=delete_payment_request, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.payment import Payment 
			if response is None:
				response = Payment()
			method = getattr(Payment, '_' + 'Payment' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def populate_body(payment):
		sanitized_payment = CommonUtil.sanitize_for_serialization(payment)
		delete_payment_request = CommonUtil.deserialize(sanitized_payment, 'DeletePaymentRequest', 'request')
		return delete_payment_request

	@staticmethod
	def wrap_response(delete_response):
		sanitized_delete_response = CommonUtil.sanitize_for_serialization(delete_response)
		payment = CommonUtil.deserialize(sanitized_delete_response, 'Payment', 'response')
		return payment
