# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.payment_setup_api import PaymentSetupApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class DeletePaymentSetupHandler(Base):

	def process(self, payment_setup):

		configuration = Configuration()
		configuration.secret = payment_setup.invocation_context.secret
		configuration.client_api_key = payment_setup.invocation_context.api_key
		configuration.algorithm = payment_setup.invocation_context.auth_scheme
		configuration.host = payment_setup.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = PaymentSetupApi(api_client=api_client)
	
			channel = payment_setup.invocation_context.channel
			client_key = payment_setup.invocation_context.client_key
			product = payment_setup.invocation_context.product
			if payment_setup.invocation_context.timestamp is not None: 
				timestamp = payment_setup.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = payment_setup.invocation_context.idempotent_request_key
			requestor_type = payment_setup.invocation_context.requestor_type
			id_payment_setup = payment_setup.id_payment_setup
			id_customer = payment_setup.id_customer
			delete_payment_setup_request = self.populate_body(payment_setup)
			requestor = payment_setup.invocation_context.requestor
			x_opay_headers = payment_setup.invocation_context.x_opay_headers
			trace_id = payment_setup.invocation_context.trace_id
	
			result = api_instance.delete_payment_setup(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_payment_setup=id_payment_setup, id_customer=id_customer, delete_payment_setup_request=delete_payment_setup_request, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.payment_setup import PaymentSetup 
			if response is None:
				response = PaymentSetup()
			method = getattr(PaymentSetup, '_' + 'PaymentSetup' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def populate_body(payment_setup):
		sanitized_payment_setup = CommonUtil.sanitize_for_serialization(payment_setup)
		delete_payment_setup_request = CommonUtil.deserialize(sanitized_payment_setup, 'DeletePaymentSetupRequest', 'request')
		return delete_payment_setup_request

	@staticmethod
	def wrap_response(delete_response):
		sanitized_delete_response = CommonUtil.sanitize_for_serialization(delete_response)
		payment_setup = CommonUtil.deserialize(sanitized_delete_response, 'PaymentSetup', 'response')
		return payment_setup
