# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.payment_api import PaymentApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class DeleteRecordPaymentHandler(Base):

	def process(self, recorded_payment):

		configuration = Configuration()
		configuration.secret = recorded_payment.invocation_context.secret
		configuration.client_api_key = recorded_payment.invocation_context.api_key
		configuration.algorithm = recorded_payment.invocation_context.auth_scheme
		configuration.host = recorded_payment.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = PaymentApi(api_client=api_client)
	
			channel = recorded_payment.invocation_context.channel
			client_key = recorded_payment.invocation_context.client_key
			product = recorded_payment.invocation_context.product
			if recorded_payment.invocation_context.timestamp is not None: 
				timestamp = recorded_payment.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = recorded_payment.invocation_context.idempotent_request_key
			requestor_type = recorded_payment.invocation_context.requestor_type
			id_payment = recorded_payment.id_payment
			delete_recorded_payment_request = self.populate_body(recorded_payment)
			requestor = recorded_payment.invocation_context.requestor
			x_opay_headers = recorded_payment.invocation_context.x_opay_headers
			trace_id = recorded_payment.invocation_context.trace_id
	
			result = api_instance.delete_record_payment(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_payment=id_payment, delete_recorded_payment_request=delete_recorded_payment_request, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id)
			response = None
			from orbipay_paymentsapi_client.wrappers.recorded_payment import RecordedPayment 
			if response is None:
				response = RecordedPayment()
			method = getattr(RecordedPayment, '_' + 'RecordedPayment' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def populate_body(recorded_payment):
		sanitized_recorded_payment = CommonUtil.sanitize_for_serialization(recorded_payment)
		delete_recorded_payment_request = CommonUtil.deserialize(sanitized_recorded_payment, 'DeleteRecordedPaymentRequest', 'request')
		return delete_recorded_payment_request

