# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.customer_account_api import CustomerAccountApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class GetCustomerAccountHandler(Base):

	def process(self, customer_account):

		configuration = Configuration()
		configuration.secret = customer_account.invocation_context.secret
		configuration.client_api_key = customer_account.invocation_context.api_key
		configuration.algorithm = customer_account.invocation_context.auth_scheme
		configuration.host = customer_account.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = CustomerAccountApi(api_client=api_client)
	
			channel = customer_account.invocation_context.channel
			client_key = customer_account.invocation_context.client_key
			product = customer_account.invocation_context.product
			if customer_account.invocation_context.timestamp is not None: 
				timestamp = customer_account.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = customer_account.invocation_context.idempotent_request_key
			requestor_type = customer_account.invocation_context.requestor_type
			id_customer_account = customer_account.id_customer_account
			id_customer = customer_account.id_customer
			requestor = customer_account.invocation_context.requestor
			x_opay_headers = customer_account.invocation_context.x_opay_headers
			trace_id = customer_account.invocation_context.trace_id
	
			result = api_instance.get_customer_account(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_customer_account=id_customer_account, id_customer=id_customer, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.customer_account import CustomerAccount 
			if response is None:
				response = CustomerAccount()
			method = getattr(CustomerAccount, '_' + 'CustomerAccount' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def wrap_response(customer_account_response):
		sanitized_customer_account_response = CommonUtil.sanitize_for_serialization(customer_account_response)
		customer_account = CommonUtil.deserialize(sanitized_customer_account_response, 'CustomerAccount', 'response')
		return customer_account
