# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.customer_api import CustomerApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class GetCustomerHandler(Base):

	def process(self, customer):

		configuration = Configuration()
		configuration.secret = customer.invocation_context.secret
		configuration.client_api_key = customer.invocation_context.api_key
		configuration.algorithm = customer.invocation_context.auth_scheme
		configuration.host = customer.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = CustomerApi(api_client=api_client)
	
			channel = customer.invocation_context.channel
			client_key = customer.invocation_context.client_key
			product = customer.invocation_context.product
			if customer.invocation_context.timestamp is not None: 
				timestamp = customer.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = customer.invocation_context.idempotent_request_key
			requestor_type = customer.invocation_context.requestor_type
			id_customer = customer.id_customer
			requestor = customer.invocation_context.requestor
			x_opay_headers = customer.invocation_context.x_opay_headers
			trace_id = customer.invocation_context.trace_id
	
			result = api_instance.get_customer(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_customer=id_customer, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.customer import Customer 
			if response is None:
				response = Customer()
			method = getattr(Customer, '_' + 'Customer' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def wrap_response(customer_response):
		sanitized_customer_response = CommonUtil.sanitize_for_serialization(customer_response)
		customer = CommonUtil.deserialize(sanitized_customer_response, 'Customer', 'response')
		return customer
