# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.customer_account_api import CustomerAccountApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class RetrieveCustomerAccountsHandler(Base):

	def process(self, customer_accounts):

		configuration = Configuration()
		configuration.secret = customer_accounts.invocation_context.secret
		configuration.client_api_key = customer_accounts.invocation_context.api_key
		configuration.algorithm = customer_accounts.invocation_context.auth_scheme
		configuration.host = customer_accounts.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = CustomerAccountApi(api_client=api_client)
	
			channel = customer_accounts.invocation_context.channel
			client_key = customer_accounts.invocation_context.client_key
			product = customer_accounts.invocation_context.product
			if customer_accounts.invocation_context.timestamp is not None: 
				timestamp = customer_accounts.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = customer_accounts.invocation_context.idempotent_request_key
			requestor_type = customer_accounts.invocation_context.requestor_type
			id_customer = customer_accounts.id_customer
			requestor = customer_accounts.invocation_context.requestor
			x_opay_headers = customer_accounts.invocation_context.x_opay_headers
			trace_id = customer_accounts.invocation_context.trace_id
			page_size = customer_accounts.page_size
			account_number = customer_accounts.account_number
			status = customer_accounts.status
	
			result = api_instance.retrieve_customer_accounts(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_customer=id_customer, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id, page_size=page_size, account_number=account_number, status=status)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.customer_accounts import CustomerAccounts 
			if response is None:
				response = CustomerAccounts()
			method = getattr(CustomerAccounts, '_' + 'CustomerAccounts' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def wrap_response(customer_accounts_response):
		sanitized_customer_accounts_response = CommonUtil.sanitize_for_serialization(customer_accounts_response)
		customer_accounts = CommonUtil.deserialize(sanitized_customer_accounts_response, 'CustomerAccounts', 'response')
		return customer_accounts
