# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.customer_api import CustomerApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class RetrieveCustomersPageHandler(Base):

	def process(self, customers):

		configuration = Configuration()
		configuration.secret = customers.invocation_context.secret
		configuration.client_api_key = customers.invocation_context.api_key
		configuration.algorithm = customers.invocation_context.auth_scheme
		configuration.host = customers.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = CustomerApi(api_client=api_client)
	
			channel = customers.invocation_context.channel
			client_key = customers.invocation_context.client_key
			product = customers.invocation_context.product
			if customers.invocation_context.timestamp is not None: 
				timestamp = customers.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = customers.invocation_context.idempotent_request_key
			requestor_type = customers.invocation_context.requestor_type
			query_id = customers.query_id
			requestor = customers.invocation_context.requestor
			x_opay_headers = customers.invocation_context.x_opay_headers
			trace_id = customers.invocation_context.trace_id
			page_size = customers.page_size
			from_index = customers.from_index
			to_index = customers.to_index
	
			result = api_instance.retrieve_customers_page(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, query_id=query_id, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id, page_size=page_size, from_index=from_index, to_index=to_index)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.customers import Customers 
			if response is None:
				response = Customers()
			method = getattr(Customers, '_' + 'Customers' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def wrap_response(customers_response):
		sanitized_customers_response = CommonUtil.sanitize_for_serialization(customers_response)
		customers = CommonUtil.deserialize(sanitized_customers_response, 'Customers', 'response')
		return customers
