# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.funding_account_api import FundingAccountApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class RetrieveFundingAccountsHandler(Base):

	def process(self, funding_accounts):

		configuration = Configuration()
		configuration.secret = funding_accounts.invocation_context.secret
		configuration.client_api_key = funding_accounts.invocation_context.api_key
		configuration.algorithm = funding_accounts.invocation_context.auth_scheme
		configuration.host = funding_accounts.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = FundingAccountApi(api_client=api_client)
	
			channel = funding_accounts.invocation_context.channel
			client_key = funding_accounts.invocation_context.client_key
			product = funding_accounts.invocation_context.product
			if funding_accounts.invocation_context.timestamp is not None: 
				timestamp = funding_accounts.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = funding_accounts.invocation_context.idempotent_request_key
			requestor_type = funding_accounts.invocation_context.requestor_type
			id_customer = funding_accounts.id_customer
			requestor = funding_accounts.invocation_context.requestor
			x_opay_headers = funding_accounts.invocation_context.x_opay_headers
			trace_id = funding_accounts.invocation_context.trace_id
			page_size = funding_accounts.page_size
			account_number = funding_accounts.account_number
			status = funding_accounts.status
			aba_routing_number = funding_accounts.aba_routing_number
			account_type = funding_accounts.account_type
			account_subtype = funding_accounts.account_subtype
	
			result = api_instance.retrieve_funding_accounts(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, id_customer=id_customer, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id, page_size=page_size, account_number=account_number, status=status, aba_routing_number=aba_routing_number, account_type=account_type, account_subtype=account_subtype)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.funding_accounts import FundingAccounts 
			if response is None:
				response = FundingAccounts()
			method = getattr(FundingAccounts, '_' + 'FundingAccounts' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def wrap_response(funding_accounts_response):
		sanitized_funding_accounts_response = CommonUtil.sanitize_for_serialization(funding_accounts_response)
		funding_accounts = CommonUtil.deserialize(sanitized_funding_accounts_response, 'FundingAccounts', 'response')
		return funding_accounts
