# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.payment_setup_api import PaymentSetupApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class RetrievePaymentSetupsHandler(Base):

	def process(self, payment_setups):

		configuration = Configuration()
		configuration.secret = payment_setups.invocation_context.secret
		configuration.client_api_key = payment_setups.invocation_context.api_key
		configuration.algorithm = payment_setups.invocation_context.auth_scheme
		configuration.host = payment_setups.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = PaymentSetupApi(api_client=api_client)
	
			channel = payment_setups.invocation_context.channel
			client_key = payment_setups.invocation_context.client_key
			product = payment_setups.invocation_context.product
			if payment_setups.invocation_context.timestamp is not None: 
				timestamp = payment_setups.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = payment_setups.invocation_context.idempotent_request_key
			requestor_type = payment_setups.invocation_context.requestor_type
			requestor = payment_setups.invocation_context.requestor
			x_opay_headers = payment_setups.invocation_context.x_opay_headers
			trace_id = payment_setups.invocation_context.trace_id
			id_customer = payment_setups.id_customer
			id_funding_account = payment_setups.id_funding_account
			id_customer_account = payment_setups.id_customer_account
			confirmation_number = payment_setups.confirmation_number
			status = payment_setups.status
			payment_setup_schedule_type = payment_setups.payment_setup_schedule_type
			from_date = payment_setups.from_date
			to_date = payment_setups.to_date
			page_size = payment_setups.page_size
			query_id = payment_setups.query_id
			from_index = payment_setups.from_index
			to_index = payment_setups.to_index
	
			result = api_instance.retrieve_payment_setups(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id, id_customer=id_customer, id_funding_account=id_funding_account, id_customer_account=id_customer_account, confirmation_number=confirmation_number, status=status, payment_setup_schedule_type=payment_setup_schedule_type, from_date=from_date, to_date=to_date, page_size=page_size, query_id=query_id, from_index=from_index, to_index=to_index)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.payment_setups import PaymentSetups 
			if response is None:
				response = PaymentSetups()
			method = getattr(PaymentSetups, '_' + 'PaymentSetups' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def wrap_response(payment_setups_response):
		sanitized_payment_setups_response = CommonUtil.sanitize_for_serialization(payment_setups_response)
		payment_setups = CommonUtil.deserialize(sanitized_payment_setups_response, 'PaymentSetups', 'response')
		return payment_setups
