# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.payment_api import PaymentApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class RetrievePaymentsHandler(Base):

	def process(self, payments):

		configuration = Configuration()
		configuration.secret = payments.invocation_context.secret
		configuration.client_api_key = payments.invocation_context.api_key
		configuration.algorithm = payments.invocation_context.auth_scheme
		configuration.host = payments.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = PaymentApi(api_client=api_client)
	
			channel = payments.invocation_context.channel
			client_key = payments.invocation_context.client_key
			product = payments.invocation_context.product
			if payments.invocation_context.timestamp is not None: 
				timestamp = payments.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = payments.invocation_context.idempotent_request_key
			requestor_type = payments.invocation_context.requestor_type
			requestor = payments.invocation_context.requestor
			x_opay_headers = payments.invocation_context.x_opay_headers
			trace_id = payments.invocation_context.trace_id
			id_customer = payments.id_customer
			id_funding_account = payments.id_funding_account
			id_customer_account = payments.id_customer_account
			confirmation_number = payments.confirmation_number
			status = payments.status
			payment_schedule_type = payments.payment_schedule_type
			payment_method = payments.payment_method
			from_date = payments.from_date
			to_date = payments.to_date
			page_size = payments.page_size
			query_id = payments.query_id
			from_index = payments.from_index
			to_index = payments.to_index
	
			result = api_instance.retrieve_payments(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id, id_customer=id_customer, id_funding_account=id_funding_account, id_customer_account=id_customer_account, confirmation_number=confirmation_number, status=status, payment_schedule_type=payment_schedule_type, payment_method=payment_method, from_date=from_date, to_date=to_date, page_size=page_size, query_id=query_id, from_index=from_index, to_index=to_index)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.payments import Payments 
			if response is None:
				response = Payments()
			method = getattr(Payments, '_' + 'Payments' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def wrap_response(payments_response):
		sanitized_payments_response = CommonUtil.sanitize_for_serialization(payments_response)
		payments = CommonUtil.deserialize(sanitized_payments_response, 'Payments', 'response')
		return payments
