# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.refund_api import RefundApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class RetrieveRefundsHandler(Base):

	def process(self, refunds):

		configuration = Configuration()
		configuration.secret = refunds.invocation_context.secret
		configuration.client_api_key = refunds.invocation_context.api_key
		configuration.algorithm = refunds.invocation_context.auth_scheme
		configuration.host = refunds.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = RefundApi(api_client=api_client)
	
			channel = refunds.invocation_context.channel
			client_key = refunds.invocation_context.client_key
			product = refunds.invocation_context.product
			if refunds.invocation_context.timestamp is not None: 
				timestamp = refunds.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = refunds.invocation_context.idempotent_request_key
			requestor_type = refunds.invocation_context.requestor_type
			requestor = refunds.invocation_context.requestor
			x_opay_headers = refunds.invocation_context.x_opay_headers
			trace_id = refunds.invocation_context.trace_id
			id_customer = refunds.id_customer
			confirmation_number = refunds.confirmation_number
			status = refunds.status
			from_date = refunds.from_date
			to_date = refunds.to_date
			page_size = refunds.page_size
			query_id = refunds.query_id
			from_index = refunds.from_index
			to_index = refunds.to_index
	
			result = api_instance.retrieve_refunds(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id, id_customer=id_customer, confirmation_number=confirmation_number, status=status, from_date=from_date, to_date=to_date, page_size=page_size, query_id=query_id, from_index=from_index, to_index=to_index)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.refunds import Refunds 
			if response is None:
				response = Refunds()
			method = getattr(Refunds, '_' + 'Refunds' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def wrap_response(refunds_response):
		sanitized_refunds_response = CommonUtil.sanitize_for_serialization(refunds_response)
		refunds = CommonUtil.deserialize(sanitized_refunds_response, 'Refunds', 'response')
		return refunds
