# coding: utf-8

"""
    ############################################################################################
    ## This class subject to change without prior notice, Please dont use this class directly.##
    ############################################################################################
"""


import six
from orbipay_paymentsapi_client.configuration import Configuration
from orbipay_paymentsapi_client.common_util import CommonUtil
from orbipay_paymentsapi_client.wrappers.base import Base
from orbipay_paymentsapi_client.api_client import ApiClient
from orbipay_paymentsapi_client.api.payment_api import PaymentApi
from orbipay_paymentsapi_client.rest import ApiException
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException


class SimplePaymentHandler(Base):

	def process(self, simple_payment):

		configuration = Configuration()
		configuration.secret = simple_payment.invocation_context.secret
		configuration.client_api_key = simple_payment.invocation_context.api_key
		configuration.algorithm = simple_payment.invocation_context.auth_scheme
		configuration.host = simple_payment.api_end_point

		try:
			api_client = ApiClient(configuration=configuration)
			api_instance = PaymentApi(api_client=api_client)
	
			channel = simple_payment.invocation_context.channel
			client_key = simple_payment.invocation_context.client_key
			product = simple_payment.invocation_context.product
			if simple_payment.invocation_context.timestamp is not None: 
				timestamp = simple_payment.invocation_context.timestamp
			else:
				timestamp = CommonUtil.time_stamp()
			idempotent_request_key = simple_payment.invocation_context.idempotent_request_key
			requestor_type = simple_payment.invocation_context.requestor_type
			simple_payment_request = self.populate_body(simple_payment)
			requestor = simple_payment.invocation_context.requestor
			x_opay_headers = simple_payment.invocation_context.x_opay_headers
			trace_id = simple_payment.invocation_context.trace_id
			save_funding_account = simple_payment.save_funding_account
	
			result = api_instance.simple_payment(channel=channel, client_key=client_key, product=product, timestamp=timestamp, idempotent_request_key=idempotent_request_key, requestor_type=requestor_type, simple_payment_request=simple_payment_request, requestor=requestor, x_opay_headers=x_opay_headers, trace_id=trace_id, save_funding_account=save_funding_account)
			response = self.wrap_response(result['data'])
			from orbipay_paymentsapi_client.wrappers.simple_payment import SimplePayment 
			if response is None:
				response = SimplePayment()
			method = getattr(SimplePayment, '_' + 'SimplePayment' + '__init_object')
			if method is not None:
				response = method(response, invocation_context=CommonUtil.invocation_context(result['headers']), http_status_code=result['status'])
			return response
		except ApiException as e:
			six.raise_from(OrbipayApiException(api_exception=e), e)
		except Exception as e:
			six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)
			

	@staticmethod
	def populate_body(simple_payment):
		sanitized_simple_payment = CommonUtil.sanitize_for_serialization(simple_payment)
		simple_payment_request = CommonUtil.deserialize(sanitized_simple_payment, 'SimplePaymentRequest', 'request')
		return simple_payment_request

	@staticmethod
	def wrap_response(simple_payment_response):
		sanitized_simple_payment_response = CommonUtil.sanitize_for_serialization(simple_payment_response)
		simple_payment = CommonUtil.deserialize(sanitized_simple_payment_response, 'SimplePayment', 'response')
		return simple_payment
