# coding: utf-8

import json
from orbipay_paymentsapi_client.common_util import CommonUtil
import traceback
import sys
import base64


class OrbipayApiException(Exception):

    def __init__(self, http_status_code=None, reason=None, api_exception=None, args=None):

        self._http_status_code = None
        self._reason = None
        self._error_response = None
        self._invocation_context = None
        self._is_timed_out = False
        
        if api_exception:
            self._http_status_code = api_exception.status
            self._reason = api_exception.reason
            if api_exception.body:
                self.error_response = api_exception.body
            self.invocation_context = api_exception.headers
            if api_exception.is_timed_out:
                self._is_timed_out = api_exception.is_timed_out            
        else:
            self._http_status_code = http_status_code
            self._reason = reason
            self._args = args

    @property
    def is_timed_out(self):
        return self._is_timed_out
    
    @property
    def http_status_code(self):
        return self._http_status_code

    @property
    def reason(self):
        return self._reason

    @property
    def invocation_context(self):
        return self._invocation_context
          
    @property
    def error_response(self):
        return self._error_response

    @error_response.setter
    def error_response(self, response_body):
        if CommonUtil.is_json(response_body):
            self._error_response = CommonUtil.deserialize(json.loads(response_body), 'ErrorResponse', 'error_response')

    @invocation_context.setter
    def invocation_context(self, response_headers):
        if response_headers is not None:
            self._invocation_context = CommonUtil.invocation_context(response_headers)
