# coding: utf-8

import six
import pprint
from orbipay_paymentsapi_client.wrappers.invocation_context import InvocationContext
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException
from orbipay_paymentsapi_client import logger_util

class Base(object):

    swagger_types = {
        'api_end_point': 'str',
        'live_mode': 'str',
        'invocation_context': 'InvocationContext',
        'http_status_code': 'http_status_code'
    }

    attribute_map = {
        'api_end_point': 'api_end_point',
        'live_mode': 'live_mode',
        'invocation_context': 'invocation_context',
        'http_status_code': 'http_status_code'
    }

    def __init__(self, api_end_point=None, live_mode=None, invocation_context=None, http_status_code=None):
        self._api_end_point = None
        self._live_mode = None
        self._invocation_context = None
        self._http_status_code = None
        
        self._api_end_point = api_end_point
        self._live_mode = live_mode
        self._invocation_context = invocation_context
        self._http_status_code = http_status_code

    @property
    def api_end_point(self):
        return self._api_end_point

    @property
    def live_mode(self):
        return self._live_mode

    @property    
    def invocation_context(self):
        return self._invocation_context

    @property
    def http_status_code(self):
        return self._http_status_code
        
    def _set_api_end_point(self, api_end_point=None):
        if self._live_mode == "true":
            self._api_end_point = "https://api.orbipay.com/payments/v1"        
        elif self._live_mode == "false" and api_end_point is None:
            self._api_end_point = "https://sbapi.orbipay.com/payments/v1"
        elif self._live_mode == "false" and api_end_point is not None:
            self._api_end_point = api_end_point
        else:
            logger_util.log_error("Invalid value for live_mode,  value Received in request : " + str(self._live_mode) + ", valid values for live_mode: true, false")
            raise OrbipayApiException(
                reason="Invalid value for live_mode,  value Received in request : " + str(self._live_mode) + ", valid values for live_mode: true, false"
            )

    def _with_context(self, invocation_context, live_mode=None, api_end_point=None):
        self._live_mode = live_mode
        self._set_api_end_point(api_end_point)

        if self._invocation_context is None:
            self._invocation_context = InvocationContext()
        kwargs = {}
        logger_util.log_info("idempotent_request_key :" + str(invocation_context.idempotent_request_key))
        if invocation_context.product is not None:
            kwargs["product"] = invocation_context.product
        if invocation_context.secret is not None:
            kwargs["secret"] = invocation_context.secret
        if invocation_context.idempotent_request_key is not None:
            kwargs["idempotent_request_key"] = invocation_context.idempotent_request_key
        if invocation_context.x_opay_headers is not None:
            kwargs["x_opay_headers"] = invocation_context.x_opay_headers
        if invocation_context.api_key is not None:
            kwargs["api_key"] = invocation_context.api_key
        if invocation_context.auth_scheme is not None:
            kwargs["auth_scheme"] = invocation_context.auth_scheme
        if invocation_context.trace_id is not None:
            kwargs["trace_id"] = invocation_context.trace_id
        method = getattr(InvocationContext, '_' + "InvocationContext" + '__init_object')
        if method is not None:
            self._invocation_context = method(self._invocation_context, **kwargs)    
    
    def vide_channel(self, channel):
        if self._invocation_context is None: 
            self._invocation_context = InvocationContext()
        kwargs = {}
        if channel is not None:
            kwargs["channel"] = channel
        method = getattr(InvocationContext, '_' + "InvocationContext" + '__init_object')
        if method is not None:
            self._invocation_context = method(self._invocation_context, **kwargs)    
        return self
    
    def requested_by(self, requestor, requestor_type):
        if self._invocation_context is None: 
            self._invocation_context = InvocationContext()
        kwargs = {}
        if requestor is not None:
            kwargs["requestor"] = requestor
        if requestor_type is not None:
            kwargs["requestor_type"] = requestor_type
        method = getattr(InvocationContext, '_' + "InvocationContext" + '__init_object')
        if method is not None:
            self._invocation_context = method(self._invocation_context, **kwargs)    
        return self
    
    def for_client(self, client_key):
        if self._invocation_context is None: 
            self._invocation_context = InvocationContext()
        kwargs = {}
        if client_key is not None:
            kwargs["client_key"] = client_key
        method = getattr(InvocationContext, '_' + "InvocationContext" + '__init_object')
        if method is not None:
            self._invocation_context = method(self._invocation_context, **kwargs)    
        return self

    @staticmethod
    def __init_object(obj=None, api_end_point=None, live_mode=None, invocation_context=None, http_status_code=None):
        if obj is None:
            obj = Base()

        if api_end_point is not None:
            obj._api_end_point = api_end_point
        if live_mode is not None:
            obj._live_mode = live_mode
        if invocation_context is not None:
            obj._invocation_context = invocation_context
        if http_status_code is not None:
            obj._http_status_code = http_status_code
        return obj

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(Base.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Base, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

