# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class Customer(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'first_name': 'str',	
        'last_name': 'str',	
        'middle_name': 'str',	
        'gender': 'str',	
        'date_of_birth': 'str',	
        'ssn': 'str',	
        'locale': 'str',	
        'email': 'str',	
        'registered_email': 'str',	
        'home_phone': 'str',	
        'work_phone': 'str',	
        'mobile_phone': 'str',	
        'address': 'Address',	
        'custom_fields': 'dict(str, str)',	
        'customer_reference': 'str',	
        'id': 'str',	
        'status': 'str',	
        'url': 'str',	
        'comments': 'str',	
        'customer_accounts': 'CustomerAccounts',	
        'funding_accounts': 'FundingAccounts',	
        'payments': 'Payments',	
        'payment_setups': 'PaymentSetups',	
        'audit_info': 'AuditInfo',	
        'id_customer': 'str'	
    }    

    attribute_map = {
        'first_name': 'first_name', 
        'last_name': 'last_name', 
        'middle_name': 'middle_name', 
        'gender': 'gender', 
        'date_of_birth': 'date_of_birth', 
        'ssn': 'ssn', 
        'locale': 'locale', 
        'email': 'email', 
        'registered_email': 'registered_email', 
        'home_phone': 'home_phone', 
        'work_phone': 'work_phone', 
        'mobile_phone': 'mobile_phone', 
        'address': 'address', 
        'custom_fields': 'custom_fields', 
        'customer_reference': 'customer_reference', 
        'id': 'id', 
        'status': 'status', 
        'url': 'url', 
        'comments': 'comments', 
        'customer_accounts': 'customer_accounts', 
        'funding_accounts': 'funding_accounts', 
        'payments': 'payments', 
        'payment_setups': 'payment_setups', 
        'audit_info': 'audit_info', 
        'id_customer': 'id_customer' 
    }
    
    def __init__(self, id_customer=None):
        super(Customer, self).__init__()
        
        self._first_name = None
        self._last_name = None
        self._middle_name = None
        self._gender = None
        self._date_of_birth = None
        self._ssn = None
        self._locale = None
        self._email = None
        self._registered_email = None
        self._home_phone = None
        self._work_phone = None
        self._mobile_phone = None
        self._address = None
        self._custom_fields = None
        self._customer_reference = None
        self._id = None
        self._status = None
        self._url = None
        self._comments = None
        self._customer_accounts = None
        self._funding_accounts = None
        self._payments = None
        self._payment_setups = None
        self._audit_info = None
        self._id_customer = None
        
        self._id_customer = id_customer
        self._id = id_customer

    @property
    def first_name(self):
        """Gets the first_name of this Customer.  # noqa: E501
        The first name of the customer. Either first_name or last_name is required to create a customer.  # noqa: E501

        :return: The first_name of this Customer. 
        :rtype: str
        """
        return self._first_name

    @property
    def last_name(self):
        """Gets the last_name of this Customer.  # noqa: E501
        The last name of the customer. Either first_name or last_name is required to create a customer.  # noqa: E501

        :return: The last_name of this Customer. 
        :rtype: str
        """
        return self._last_name

    @property
    def middle_name(self):
        """Gets the middle_name of this Customer.  # noqa: E501
        The middle name of the customer.  # noqa: E501

        :return: The middle_name of this Customer. 
        :rtype: str
        """
        return self._middle_name

    @property
    def gender(self):
        """Gets the gender of this Customer.  # noqa: E501
        The gender of the customer.  # noqa: E501

        :return: The gender of this Customer. 
        :rtype: str
        """
        return self._gender

    @property
    def date_of_birth(self):
        """Gets the date_of_birth of this Customer.  # noqa: E501
        The date of birth of the customer. It should be in the full-date format as per ISO8601, namely, YYYY-MM-DD.  # noqa: E501

        :return: The date_of_birth of this Customer. 
        :rtype: str
        """
        return self._date_of_birth

    @property
    def ssn(self):
        """Gets the ssn of this Customer.  # noqa: E501
        The SSN of the customer if the account holder is an individual or the tax ID, if the customer is a business.  # noqa: E501

        :return: The ssn of this Customer. 
        :rtype: str
        """
        return self._ssn

    @property
    def locale(self):
        """Gets the locale of this Customer.  # noqa: E501
        The language in which the customer wants the alerts and notifications from EBPP. This is to be specified in the format,<br><i>< <b>ISO-639-1 language code</b>  >_< <b>ISO ALPHA-2 Country Code</b> ></i>.<br> For e.g., <b>en_US</b> indicates the language preference as US English, which is also the default value.  # noqa: E501

        :return: The locale of this Customer. 
        :rtype: str
        """
        return self._locale

    @property
    def email(self):
        """Gets the email of this Customer.  # noqa: E501
        The email address of the customer to which notifications from EBPP will be sent. This will default to the registered_email while creating a customer.  # noqa: E501

        :return: The email of this Customer. 
        :rtype: str
        """
        return self._email

    @property
    def registered_email(self):
        """Gets the registered_email of this Customer.  # noqa: E501
        The email address of the customer registered with the client. This may be used to uniquely identify the customer if the client is set up accordingly in EBPP.  # noqa: E501

        :return: The registered_email of this Customer. 
        :rtype: str
        """
        return self._registered_email

    @property
    def home_phone(self):
        """Gets the home_phone of this Customer.  # noqa: E501
        The home phone number of the customer.  # noqa: E501

        :return: The home_phone of this Customer. 
        :rtype: str
        """
        return self._home_phone

    @property
    def work_phone(self):
        """Gets the work_phone of this Customer.  # noqa: E501
        The work phone number of the customer.  # noqa: E501

        :return: The work_phone of this Customer. 
        :rtype: str
        """
        return self._work_phone

    @property
    def mobile_phone(self):
        """Gets the mobile_phone of this Customer.  # noqa: E501
        The mobile phone number of the customer.  # noqa: E501

        :return: The mobile_phone of this Customer. 
        :rtype: str
        """
        return self._mobile_phone

    @property
    def address(self):
        """Gets the address of this Customer.  # noqa: E501

        :return: The address of this Customer. 
        :rtype: Address
        """
        return self._address

    @property
    def custom_fields(self):
        """Gets the custom_fields of this Customer.  # noqa: E501
        The additional information or meta-information that EBPP can accept, maintain and transmit back to the client. The custom fields need to be configured with EBPP before they can be accepted. EBPP would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields. Only the custom fields enabled for the client, in EBPP, would be accepted in the requests.  # noqa: E501

        :return: The custom_fields of this Customer. 
        :rtype: dict(str, str)
        """
        return self._custom_fields

    @property
    def customer_reference(self):
        """Gets the customer_reference of this Customer.  # noqa: E501
        The unique identifier in the client system for the customer.  # noqa: E501

        :return: The customer_reference of this Customer. 
        :rtype: str
        """
        return self._customer_reference

    @property
    def id(self):
        """Gets the id of this Customer.  # noqa: E501
        The unique identifier assigned by EBPP to the customer.  # noqa: E501

        :return: The id of this Customer. 
        :rtype: str
        """
        return self._id

    @property
    def status(self):
        """Gets the status of this Customer.  # noqa: E501
        The status of the customer in EBPP.   # noqa: E501

        :return: The status of this Customer. 
        :rtype: str
        """
        return self._status

    @property
    def url(self):
        """Gets the url of this Customer.  # noqa: E501
        This URL fetches the Customer details.  # noqa: E501

        :return: The url of this Customer. 
        :rtype: str
        """
        return self._url

    @property
    def comments(self):
        """Gets the comments of this Customer.  # noqa: E501
        Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (EBPP) is not responsible for the security of any sensitive information that may be passed as part of the memo.  # noqa: E501

        :return: The comments of this Customer. 
        :rtype: str
        """
        return self._comments

    @property
    def customer_accounts(self):
        """Gets the customer_accounts of this Customer.  # noqa: E501

        :return: The customer_accounts of this Customer. 
        :rtype: CustomerAccounts
        """
        return self._customer_accounts

    @property
    def funding_accounts(self):
        """Gets the funding_accounts of this Customer.  # noqa: E501

        :return: The funding_accounts of this Customer. 
        :rtype: FundingAccounts
        """
        return self._funding_accounts

    @property
    def payments(self):
        """Gets the payments of this Customer.  # noqa: E501

        :return: The payments of this Customer. 
        :rtype: Payments
        """
        return self._payments

    @property
    def payment_setups(self):
        """Gets the payment_setups of this Customer.  # noqa: E501

        :return: The payment_setups of this Customer. 
        :rtype: PaymentSetups
        """
        return self._payment_setups

    @property
    def audit_info(self):
        """Gets the audit_info of this Customer.  # noqa: E501

        :return: The audit_info of this Customer. 
        :rtype: AuditInfo
        """
        return self._audit_info

    @property
    def id_customer(self):
        """Gets the id_customer of this Customer.  # noqa: E501
        The unique identifier assigned by EBPP to the customer.  # noqa: E501

        :return: The id_customer of this Customer. 
        :rtype: str
        """
        return self._id_customer

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        super(Customer, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        super(Customer, self).vide_channel(channel)
        return self
    def with_details(self, first_name=None, last_name=None, middle_name=None, date_of_birth=None, ssn=None, gender=None, registered_email=None):
        """
        :param first_name: The first_name is of type str.
        :param last_name: The last_name is of type str.
        :param middle_name: The middle_name is of type str.
        :param date_of_birth: The date_of_birth is of type str.
        :param ssn: The ssn is of type str.
        :param gender: The gender is of type str.
        :param registered_email: The registered_email is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._first_name = first_name
        self._last_name = last_name
        self._middle_name = middle_name
        self._date_of_birth = date_of_birth
        self._ssn = ssn
        self._gender = gender
        self._registered_email = registered_email
        return self
    def with_reference(self, customer_reference=None):
        """
        :param customer_reference: The customer_reference is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._customer_reference = customer_reference
        return self
    def with_status(self, status=None):
        """
        :param status: The status is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._status = status
        return self
    def with_contact_info(self, email=None, home_phone=None, work_phone=None, mobile_phone=None):
        """
        :param email: The email is of type str.
        :param home_phone: The home_phone is of type str.
        :param work_phone: The work_phone is of type str.
        :param mobile_phone: The mobile_phone is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._email = email
        self._home_phone = home_phone
        self._work_phone = work_phone
        self._mobile_phone = mobile_phone
        return self
    def with_locale(self, locale=None):
        """
        :param locale: The locale is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._locale = locale
        return self
    def with_accounts(self, customer_accounts=None):
        """
        :param customer_accounts: The customer_accounts is of type CustomerAccounts.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._customer_accounts = customer_accounts
        return self
    def with_address(self, address=None):
        """
        :param address: The address is of type Address.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._address = address
        return self
    def with_custom_fields(self, custom_fields=None):
        """
        :param custom_fields: The custom_fields is of type dict(str, str).
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._custom_fields = custom_fields
        return self
    def with_memo(self, comments=None):
        """
        :param comments: The comments is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        self._comments = comments
        return self
    ## support methods
    def created_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        super(Customer, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def retrieved_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        super(Customer, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def updated_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        super(Customer, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    def create(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        try:
            super(Customer, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.create_customer_handler import CreateCustomerHandler
            return CreateCustomerHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def get(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        try:
            super(Customer, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.get_customer_handler import GetCustomerHandler
            return GetCustomerHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def update(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of Customer. 
        :rtype: Customer
        """
        try:
            super(Customer, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.update_customer_handler import UpdateCustomerHandler
            return UpdateCustomerHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, first_name=None, last_name=None, middle_name=None, gender=None, date_of_birth=None, ssn=None, locale=None, email=None, registered_email=None, home_phone=None, work_phone=None, mobile_phone=None, address=None, custom_fields=None, customer_reference=None, id=None, status=None, url=None, comments=None, customer_accounts=None, funding_accounts=None, payments=None, payment_setups=None, audit_info=None, id_customer=None):
        if obj is None:
            obj = Customer()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if first_name is not None:
            obj._first_name = first_name
        if last_name is not None:
            obj._last_name = last_name
        if middle_name is not None:
            obj._middle_name = middle_name
        if gender is not None:
            obj._gender = gender
        if date_of_birth is not None:
            obj._date_of_birth = date_of_birth
        if ssn is not None:
            obj._ssn = ssn
        if locale is not None:
            obj._locale = locale
        if email is not None:
            obj._email = email
        if registered_email is not None:
            obj._registered_email = registered_email
        if home_phone is not None:
            obj._home_phone = home_phone
        if work_phone is not None:
            obj._work_phone = work_phone
        if mobile_phone is not None:
            obj._mobile_phone = mobile_phone
        if address is not None:
            obj._address = address
        if custom_fields is not None:
            obj._custom_fields = custom_fields
        if customer_reference is not None:
            obj._customer_reference = customer_reference
        if id is not None:
            obj._id = id
        if status is not None:
            obj._status = status
        if url is not None:
            obj._url = url
        if comments is not None:
            obj._comments = comments
        if customer_accounts is not None:
            obj._customer_accounts = customer_accounts
        if funding_accounts is not None:
            obj._funding_accounts = funding_accounts
        if payments is not None:
            obj._payments = payments
        if payment_setups is not None:
            obj._payment_setups = payment_setups
        if audit_info is not None:
            obj._audit_info = audit_info
        if id_customer is not None:
            obj._id_customer = id_customer
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(Customer, self).to_dict()
        for attr, _ in six.iteritems(Customer.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Customer, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
