# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class CustomerAccount(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'account_holder_name': 'str',	
        'nickname': 'str',	
        'address': 'Address',	
        'customer_account_reference': 'str',	
        'account_number': 'str',	
        'current_balance': 'str',	
        'current_statement_balance': 'str',	
        'minimum_payment_due': 'str',	
        'past_amount_due': 'str',	
        'payment_due_date': 'str',	
        'statement_date': 'str',	
        'id': 'str',	
        'url': 'str',	
        'status': 'str',	
        'custom_fields': 'dict(str, str)',	
        'comments': 'str',	
        'audit_info': 'AuditInfo',	
        'id_customer_account': 'str',	
        'id_customer': 'str'	
    }    

    attribute_map = {
        'account_holder_name': 'account_holder_name', 
        'nickname': 'nickname', 
        'address': 'address', 
        'customer_account_reference': 'customer_account_reference', 
        'account_number': 'account_number', 
        'current_balance': 'current_balance', 
        'current_statement_balance': 'current_statement_balance', 
        'minimum_payment_due': 'minimum_payment_due', 
        'past_amount_due': 'past_amount_due', 
        'payment_due_date': 'payment_due_date', 
        'statement_date': 'statement_date', 
        'id': 'id', 
        'url': 'url', 
        'status': 'status', 
        'custom_fields': 'custom_fields', 
        'comments': 'comments', 
        'audit_info': 'audit_info', 
        'id_customer_account': 'id_customer_account', 
        'id_customer': 'id_customer' 
    }
    
    def __init__(self, id_customer_account=None):
        super(CustomerAccount, self).__init__()
        
        self._account_holder_name = None
        self._nickname = None
        self._address = None
        self._customer_account_reference = None
        self._account_number = None
        self._current_balance = None
        self._current_statement_balance = None
        self._minimum_payment_due = None
        self._past_amount_due = None
        self._payment_due_date = None
        self._statement_date = None
        self._id = None
        self._url = None
        self._status = None
        self._custom_fields = None
        self._comments = None
        self._audit_info = None
        self._id_customer_account = None
        self._id_customer = None
        
        self._id_customer_account = id_customer_account
        self._id = id_customer_account

    @property
    def account_holder_name(self):
        """Gets the account_holder_name of this CustomerAccount.  # noqa: E501
        The name as specified on the account.  # noqa: E501

        :return: The account_holder_name of this CustomerAccount. 
        :rtype: str
        """
        return self._account_holder_name

    @property
    def nickname(self):
        """Gets the nickname of this CustomerAccount.  # noqa: E501
        The nickname by which a customer might want to identify the account.  # noqa: E501

        :return: The nickname of this CustomerAccount. 
        :rtype: str
        """
        return self._nickname

    @property
    def address(self):
        """Gets the address of this CustomerAccount.  # noqa: E501

        :return: The address of this CustomerAccount. 
        :rtype: Address
        """
        return self._address

    @property
    def customer_account_reference(self):
        """Gets the customer_account_reference of this CustomerAccount.  # noqa: E501
        The unique identifier in the client system for the customer account.  # noqa: E501

        :return: The customer_account_reference of this CustomerAccount. 
        :rtype: str
        """
        return self._customer_account_reference

    @property
    def account_number(self):
        """Gets the account_number of this CustomerAccount.  # noqa: E501
        The number or reference that a customer uses to identify the customer's account.  # noqa: E501

        :return: The account_number of this CustomerAccount. 
        :rtype: str
        """
        return self._account_number

    @property
    def current_balance(self):
        """Gets the current_balance of this CustomerAccount.  # noqa: E501
        The current balance on the customer's account with the biller.  # noqa: E501

        :return: The current_balance of this CustomerAccount. 
        :rtype: str
        """
        return self._current_balance

    @property
    def current_statement_balance(self):
        """Gets the current_statement_balance of this CustomerAccount.  # noqa: E501
        The balance on the customer’s account with the biller as reported on the last statement.  # noqa: E501

        :return: The current_statement_balance of this CustomerAccount. 
        :rtype: str
        """
        return self._current_statement_balance

    @property
    def minimum_payment_due(self):
        """Gets the minimum_payment_due of this CustomerAccount.  # noqa: E501
        The minimum payment amount, as specified by the biller, that needs to be made against the customer’s account.  # noqa: E501

        :return: The minimum_payment_due of this CustomerAccount. 
        :rtype: str
        """
        return self._minimum_payment_due

    @property
    def past_amount_due(self):
        """Gets the past_amount_due of this CustomerAccount.  # noqa: E501
        The amount that is due from the past bills/statements on the customer’s account with the biller.  # noqa: E501

        :return: The past_amount_due of this CustomerAccount. 
        :rtype: str
        """
        return self._past_amount_due

    @property
    def payment_due_date(self):
        """Gets the payment_due_date of this CustomerAccount.  # noqa: E501
        The date by which the customer needs to clear the dues against the customer's account with the biller. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The payment_due_date of this CustomerAccount. 
        :rtype: str
        """
        return self._payment_due_date

    @property
    def statement_date(self):
        """Gets the statement_date of this CustomerAccount.  # noqa: E501
        The date on which the biller generates the statement for the customer's account. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The statement_date of this CustomerAccount. 
        :rtype: str
        """
        return self._statement_date

    @property
    def id(self):
        """Gets the id of this CustomerAccount.  # noqa: E501
        Id is a unique identifier assigned to the customer account in EBPP.  # noqa: E501

        :return: The id of this CustomerAccount. 
        :rtype: str
        """
        return self._id

    @property
    def url(self):
        """Gets the url of this CustomerAccount.  # noqa: E501
        This URL fetches the Account details.  # noqa: E501

        :return: The url of this CustomerAccount. 
        :rtype: str
        """
        return self._url

    @property
    def status(self):
        """Gets the status of this CustomerAccount.  # noqa: E501
        The status of the customer's account in EBPP.   # noqa: E501

        :return: The status of this CustomerAccount. 
        :rtype: str
        """
        return self._status

    @property
    def custom_fields(self):
        """Gets the custom_fields of this CustomerAccount.  # noqa: E501
        The additional information or meta-information that EBPP can accept, maintain and transmit back to the client. The custom fields need to be configured with EBPP before they can be accepted. EBPP would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields. Only the custom fields enabled for the client, in EBPP, would be accepted in the requests.  # noqa: E501

        :return: The custom_fields of this CustomerAccount. 
        :rtype: dict(str, str)
        """
        return self._custom_fields

    @property
    def comments(self):
        """Gets the comments of this CustomerAccount.  # noqa: E501
        Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (EBPP) is not responsible for the security of any sensitive information that may be passed as part of the memo.  # noqa: E501

        :return: The comments of this CustomerAccount. 
        :rtype: str
        """
        return self._comments

    @property
    def audit_info(self):
        """Gets the audit_info of this CustomerAccount.  # noqa: E501

        :return: The audit_info of this CustomerAccount. 
        :rtype: AuditInfo
        """
        return self._audit_info

    @property
    def id_customer_account(self):
        """Gets the id_customer_account of this CustomerAccount.  # noqa: E501
        The unique identifier assigned by EBPP to the Customer Account.  # noqa: E501

        :return: The id_customer_account of this CustomerAccount. 
        :rtype: str
        """
        return self._id_customer_account

    @property
    def id_customer(self):
        """Gets the id_customer of this CustomerAccount.  # noqa: E501
        The unique identifier assigned by EBPP to the customer.  # noqa: E501

        :return: The id_customer of this CustomerAccount. 
        :rtype: str
        """
        return self._id_customer

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        super(CustomerAccount, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        super(CustomerAccount, self).vide_channel(channel)
        return self
    def with_details(self, account_holder_name=None, nickname=None, account_number=None, minimum_payment_due=None, past_amount_due=None, payment_due_date=None, statement_date=None):
        """
        :param account_holder_name: The account_holder_name is of type str.
        :param nickname: The nickname is of type str.
        :param account_number: The account_number is of type str.
        :param minimum_payment_due: The minimum_payment_due is of type str.
        :param past_amount_due: The past_amount_due is of type str.
        :param payment_due_date: The payment_due_date is of type str.
        :param statement_date: The statement_date is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        self._account_holder_name = account_holder_name
        self._nickname = nickname
        self._account_number = account_number
        self._minimum_payment_due = minimum_payment_due
        self._past_amount_due = past_amount_due
        self._payment_due_date = payment_due_date
        self._statement_date = statement_date
        return self
    def with_balances(self, current_balance=None, current_statement_balance=None):
        """
        :param current_balance: The current_balance is of type str.
        :param current_statement_balance: The current_statement_balance is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        self._current_balance = current_balance
        self._current_statement_balance = current_statement_balance
        return self
    def with_reference(self, customer_account_reference=None):
        """
        :param customer_account_reference: The customer_account_reference is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        self._customer_account_reference = customer_account_reference
        return self
    def with_status(self, status=None):
        """
        :param status: The status is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        self._status = status
        return self
    def with_address(self, address=None):
        """
        :param address: The address is of type Address.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        self._address = address
        return self
    def with_custom_fields(self, custom_fields=None):
        """
        :param custom_fields: The custom_fields is of type dict(str, str).
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        self._custom_fields = custom_fields
        return self
    def with_memo(self, comments=None):
        """
        :param comments: The comments is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        self._comments = comments
        return self
    def for_customer(self, id_customer=None):
        """
        :param id_customer: The id_customer is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        self._id_customer = id_customer
        return self
    ## support methods
    def retrieved_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        super(CustomerAccount, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def renewed_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        super(CustomerAccount, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    def get(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        try:
            super(CustomerAccount, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.get_customer_account_handler import GetCustomerAccountHandler
            return GetCustomerAccountHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def renew(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of CustomerAccount. 
        :rtype: CustomerAccount
        """
        try:
            super(CustomerAccount, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.renew_customer_account_handler import RenewCustomerAccountHandler
            return RenewCustomerAccountHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, account_holder_name=None, nickname=None, address=None, customer_account_reference=None, account_number=None, current_balance=None, current_statement_balance=None, minimum_payment_due=None, past_amount_due=None, payment_due_date=None, statement_date=None, id=None, url=None, status=None, custom_fields=None, comments=None, audit_info=None, id_customer_account=None, id_customer=None):
        if obj is None:
            obj = CustomerAccount()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if account_holder_name is not None:
            obj._account_holder_name = account_holder_name
        if nickname is not None:
            obj._nickname = nickname
        if address is not None:
            obj._address = address
        if customer_account_reference is not None:
            obj._customer_account_reference = customer_account_reference
        if account_number is not None:
            obj._account_number = account_number
        if current_balance is not None:
            obj._current_balance = current_balance
        if current_statement_balance is not None:
            obj._current_statement_balance = current_statement_balance
        if minimum_payment_due is not None:
            obj._minimum_payment_due = minimum_payment_due
        if past_amount_due is not None:
            obj._past_amount_due = past_amount_due
        if payment_due_date is not None:
            obj._payment_due_date = payment_due_date
        if statement_date is not None:
            obj._statement_date = statement_date
        if id is not None:
            obj._id = id
        if url is not None:
            obj._url = url
        if status is not None:
            obj._status = status
        if custom_fields is not None:
            obj._custom_fields = custom_fields
        if comments is not None:
            obj._comments = comments
        if audit_info is not None:
            obj._audit_info = audit_info
        if id_customer_account is not None:
            obj._id_customer_account = id_customer_account
        if id_customer is not None:
            obj._id_customer = id_customer
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(CustomerAccount, self).to_dict()
        for attr, _ in six.iteritems(CustomerAccount.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CustomerAccount, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
