# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class FundingAccount(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'account_holder_name': 'str',	
        'nickname': 'str',	
        'address': 'Address',	
        'account_number': 'str',	
        'aba_routing_number': 'str',	
        'issuer_name': 'str',	
        'currency_code3d': 'str',	
        'expiry_date': 'str',	
        'account_holder_type': 'str',	
        'custom_fields': 'dict(str, str)',	
        'account_subtype': 'str',	
        'id': 'str',	
        'url': 'str',	
        'ach_eligible_flag': 'str',	
        'atm_eligible_flag': 'str',	
        'card_cvv_number': 'str',	
        'account_type': 'str',	
        'status': 'str',	
        'device_manufacturer_id': 'str',	
        'display_text': 'str',	
        'comments': 'str',	
        'deleted': 'str',	
        'audit_info': 'AuditInfo',	
        'id_customer': 'str',	
        'id_funding_account': 'str'	
    }    

    attribute_map = {
        'account_holder_name': 'account_holder_name', 
        'nickname': 'nickname', 
        'address': 'address', 
        'account_number': 'account_number', 
        'aba_routing_number': 'aba_routing_number', 
        'issuer_name': 'issuer_name', 
        'currency_code3d': 'currency_code3d', 
        'expiry_date': 'expiry_date', 
        'account_holder_type': 'account_holder_type', 
        'custom_fields': 'custom_fields', 
        'account_subtype': 'account_subtype', 
        'id': 'id', 
        'url': 'url', 
        'ach_eligible_flag': 'ach_eligible_flag', 
        'atm_eligible_flag': 'atm_eligible_flag', 
        'card_cvv_number': 'card_cvv_number', 
        'account_type': 'account_type', 
        'status': 'status', 
        'device_manufacturer_id': 'device_manufacturer_id', 
        'display_text': 'display_text', 
        'comments': 'comments', 
        'deleted': 'deleted', 
        'audit_info': 'audit_info', 
        'id_customer': 'id_customer', 
        'id_funding_account': 'id_funding_account' 
    }
    
    def __init__(self, id_funding_account=None):
        super(FundingAccount, self).__init__()
        
        self._account_holder_name = None
        self._nickname = None
        self._address = None
        self._account_number = None
        self._aba_routing_number = None
        self._issuer_name = None
        self._currency_code3d = None
        self._expiry_date = None
        self._account_holder_type = None
        self._custom_fields = None
        self._account_subtype = None
        self._id = None
        self._url = None
        self._ach_eligible_flag = None
        self._atm_eligible_flag = None
        self._card_cvv_number = None
        self._account_type = None
        self._status = None
        self._device_manufacturer_id = None
        self._display_text = None
        self._comments = None
        self._deleted = None
        self._audit_info = None
        self._id_customer = None
        self._id_funding_account = None
        
        self._id_funding_account = id_funding_account
        self._id = id_funding_account

    @property
    def account_holder_name(self):
        """Gets the account_holder_name of this FundingAccount.  # noqa: E501
        The name as specified on the account.  # noqa: E501

        :return: The account_holder_name of this FundingAccount. 
        :rtype: str
        """
        return self._account_holder_name

    @property
    def nickname(self):
        """Gets the nickname of this FundingAccount.  # noqa: E501
        The nickname by which a customer might want to identify the account.  # noqa: E501

        :return: The nickname of this FundingAccount. 
        :rtype: str
        """
        return self._nickname

    @property
    def address(self):
        """Gets the address of this FundingAccount.  # noqa: E501

        :return: The address of this FundingAccount. 
        :rtype: Address
        """
        return self._address

    @property
    def account_number(self):
        """Gets the account_number of this FundingAccount.  # noqa: E501
        The number or reference that a customer uses to identify the funding account. In case of Apple Pay, this is the device-specific account number of the card.  # noqa: E501

        :return: The account_number of this FundingAccount. 
        :rtype: str
        """
        return self._account_number

    @property
    def aba_routing_number(self):
        """Gets the aba_routing_number of this FundingAccount.  # noqa: E501
        The ABA/Routing number for the bank account.  # noqa: E501

        :return: The aba_routing_number of this FundingAccount. 
        :rtype: str
        """
        return self._aba_routing_number

    @property
    def issuer_name(self):
        """Gets the issuer_name of this FundingAccount.  # noqa: E501
        The name of the authority that has issued the account. This is relevant only in the case of bank accounts.  # noqa: E501

        :return: The issuer_name of this FundingAccount. 
        :rtype: str
        """
        return self._issuer_name

    @property
    def currency_code3d(self):
        """Gets the currency_code3d of this FundingAccount.  # noqa: E501
        The three-letter currency code in the ISO4217 format, in uppercase, for the card account. It must be a currency supported by Orbipay. This defaults to USD.  # noqa: E501

        :return: The currency_code3d of this FundingAccount. 
        :rtype: str
        """
        return self._currency_code3d

    @property
    def expiry_date(self):
        """Gets the expiry_date of this FundingAccount.  # noqa: E501
        The expiry date for the card. This is to be in the format MM/YY.  # noqa: E501

        :return: The expiry_date of this FundingAccount. 
        :rtype: str
        """
        return self._expiry_date

    @property
    def account_holder_type(self):
        """Gets the account_holder_type of this FundingAccount.  # noqa: E501
        The type of ownership for the funding account. This is applicable only in the case of bank accounts and it is defaults to personal.   # noqa: E501

        :return: The account_holder_type of this FundingAccount. 
        :rtype: str
        """
        return self._account_holder_type

    @property
    def custom_fields(self):
        """Gets the custom_fields of this FundingAccount.  # noqa: E501
        The additional information or meta-information that Orbipay can accept, maintain and transmit back to the client. The custom fields need to be configured with Orbipay before they can be accepted. Orbipay would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields. Only the custom fields enabled for the client, in Orbipay, would be accepted in the requests.  # noqa: E501

        :return: The custom_fields of this FundingAccount. 
        :rtype: dict(str, str)
        """
        return self._custom_fields

    @property
    def account_subtype(self):
        """Gets the account_subtype of this FundingAccount.  # noqa: E501
        The sub type of the funding account. This is derived by Orbipay in the case of card funding accounts. It needs to be passed when adding or editing a DDA/Bank/ApplePay account. For e.g., a savings account, a checking account.  # noqa: E501

        :return: The account_subtype of this FundingAccount. 
        :rtype: str
        """
        return self._account_subtype

    @property
    def id(self):
        """Gets the id of this FundingAccount.  # noqa: E501
        Id is a unique identifier assigned to the funding account in Orbipay. A new Id is generated for the funding account when the account number is updated.  # noqa: E501

        :return: The id of this FundingAccount. 
        :rtype: str
        """
        return self._id

    @property
    def url(self):
        """Gets the url of this FundingAccount.  # noqa: E501
        This URL fetches the funding account details.  # noqa: E501

        :return: The url of this FundingAccount. 
        :rtype: str
        """
        return self._url

    @property
    def ach_eligible_flag(self):
        """Gets the ach_eligible_flag of this FundingAccount.  # noqa: E501
        Indicates if the funding account can be debited vide ACH, in case the funding account is a Demand Deposit Account (DDA).  # noqa: E501

        :return: The ach_eligible_flag of this FundingAccount. 
        :rtype: str
        """
        return self._ach_eligible_flag

    @property
    def atm_eligible_flag(self):
        """Gets the atm_eligible_flag of this FundingAccount.  # noqa: E501
        Indicates if the funding account can be debited vide the ATM Rail network, in case the funding account is a Debit Card.  # noqa: E501

        :return: The atm_eligible_flag of this FundingAccount. 
        :rtype: str
        """
        return self._atm_eligible_flag

    @property
    def card_cvv_number(self):
        """Gets the card_cvv_number of this FundingAccount.  # noqa: E501
        This is never returned by Orbipay. This needs to be passed to Orbipay when adding or editing a card funding account.  # noqa: E501

        :return: The card_cvv_number of this FundingAccount. 
        :rtype: str
        """
        return self._card_cvv_number

    @property
    def account_type(self):
        """Gets the account_type of this FundingAccount.  # noqa: E501
        The type of the funding account.  # noqa: E501

        :return: The account_type of this FundingAccount. 
        :rtype: str
        """
        return self._account_type

    @property
    def status(self):
        """Gets the status of this FundingAccount.  # noqa: E501
        The status of the funding account in Orbipay.  # noqa: E501

        :return: The status of this FundingAccount. 
        :rtype: str
        """
        return self._status

    @property
    def device_manufacturer_id(self):
        """Gets the device_manufacturer_id of this FundingAccount.  # noqa: E501
        This is applicable only for Apple Pay transactions. Hex-encoded device manufacturer identifier.  # noqa: E501

        :return: The device_manufacturer_id of this FundingAccount. 
        :rtype: str
        """
        return self._device_manufacturer_id

    @property
    def display_text(self):
        """Gets the display_text of this FundingAccount.  # noqa: E501
        UI representation of the account.  # noqa: E501

        :return: The display_text of this FundingAccount. 
        :rtype: str
        """
        return self._display_text

    @property
    def comments(self):
        """Gets the comments of this FundingAccount.  # noqa: E501
        Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (Orbipay) is not responsible for the security of any sensitive information that may be passed as part of the memo.  # noqa: E501

        :return: The comments of this FundingAccount. 
        :rtype: str
        """
        return self._comments

    @property
    def deleted(self):
        """Gets the deleted of this FundingAccount.  # noqa: E501
        The status of funding account which says whether the funding account is deleted or not  # noqa: E501

        :return: The deleted of this FundingAccount. 
        :rtype: str
        """
        return self._deleted

    @property
    def audit_info(self):
        """Gets the audit_info of this FundingAccount.  # noqa: E501

        :return: The audit_info of this FundingAccount. 
        :rtype: AuditInfo
        """
        return self._audit_info

    @property
    def id_customer(self):
        """Gets the id_customer of this FundingAccount.  # noqa: E501
        The unique identifier assigned by EBPP to the customer.  # noqa: E501

        :return: The id_customer of this FundingAccount. 
        :rtype: str
        """
        return self._id_customer

    @property
    def id_funding_account(self):
        """Gets the id_funding_account of this FundingAccount.  # noqa: E501
        The unique identifier assigned by EBPP to the Funding Account.  # noqa: E501

        :return: The id_funding_account of this FundingAccount. 
        :rtype: str
        """
        return self._id_funding_account

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        super(FundingAccount, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        super(FundingAccount, self).vide_channel(channel)
        return self
    def with_details(self, account_holder_name=None, nickname=None, account_number=None, account_subtype=None, aba_routing_number=None, card_cvv_number=None, expiry_date=None, account_holder_type=None):
        """
        :param account_holder_name: The account_holder_name is of type str.
        :param nickname: The nickname is of type str.
        :param account_number: The account_number is of type str.
        :param account_subtype: The account_subtype is of type str.
        :param aba_routing_number: The aba_routing_number is of type str.
        :param card_cvv_number: The card_cvv_number is of type str.
        :param expiry_date: The expiry_date is of type str.
        :param account_holder_type: The account_holder_type is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._account_holder_name = account_holder_name
        self._nickname = nickname
        self._account_number = account_number
        self._account_subtype = account_subtype
        self._aba_routing_number = aba_routing_number
        self._card_cvv_number = card_cvv_number
        self._expiry_date = expiry_date
        self._account_holder_type = account_holder_type
        return self
    def with_recorded_details(self, account_number=None, issuer_name=None):
        """
        :param account_number: The account_number is of type str.
        :param issuer_name: The issuer_name is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._account_number = account_number
        self._issuer_name = issuer_name
        return self
    def with_status(self, status=None):
        """
        :param status: The status is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._status = status
        return self
    def with_address(self, address=None):
        """
        :param address: The address is of type Address.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._address = address
        return self
    def with_custom_fields(self, custom_fields=None):
        """
        :param custom_fields: The custom_fields is of type dict(str, str).
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._custom_fields = custom_fields
        return self
    def with_memo(self, comments=None):
        """
        :param comments: The comments is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._comments = comments
        return self
    def for_customer(self, id_customer=None):
        """
        :param id_customer: The id_customer is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._id_customer = id_customer
        return self
    def with_device_manufacturer_id(self, device_manufacturer_id=None):
        """
        :param device_manufacturer_id: The device_manufacturer_id is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._device_manufacturer_id = device_manufacturer_id
        return self
    def with_display_text(self, display_text=None):
        """
        :param display_text: The display_text is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        self._display_text = display_text
        return self
    ## support methods
    def created_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        super(FundingAccount, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def deleted_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        super(FundingAccount, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def retrieved_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        super(FundingAccount, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def replaced_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        super(FundingAccount, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def updated_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        super(FundingAccount, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    def create(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        try:
            super(FundingAccount, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.add_funding_account_handler import AddFundingAccountHandler
            return AddFundingAccountHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def delete(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        try:
            super(FundingAccount, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.delete_funding_account_handler import DeleteFundingAccountHandler
            return DeleteFundingAccountHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def get(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        try:
            super(FundingAccount, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.get_funding_account_handler import GetFundingAccountHandler
            return GetFundingAccountHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def replace(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        try:
            super(FundingAccount, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.replace_funding_account_handler import ReplaceFundingAccountHandler
            return ReplaceFundingAccountHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def update(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of FundingAccount. 
        :rtype: FundingAccount
        """
        try:
            super(FundingAccount, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.update_funding_account_handler import UpdateFundingAccountHandler
            return UpdateFundingAccountHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, account_holder_name=None, nickname=None, address=None, account_number=None, aba_routing_number=None, issuer_name=None, currency_code3d=None, expiry_date=None, account_holder_type=None, custom_fields=None, account_subtype=None, id=None, url=None, ach_eligible_flag=None, atm_eligible_flag=None, card_cvv_number=None, account_type=None, status=None, device_manufacturer_id=None, display_text=None, comments=None, deleted=None, audit_info=None, id_customer=None, id_funding_account=None):
        if obj is None:
            obj = FundingAccount()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if account_holder_name is not None:
            obj._account_holder_name = account_holder_name
        if nickname is not None:
            obj._nickname = nickname
        if address is not None:
            obj._address = address
        if account_number is not None:
            obj._account_number = account_number
        if aba_routing_number is not None:
            obj._aba_routing_number = aba_routing_number
        if issuer_name is not None:
            obj._issuer_name = issuer_name
        if currency_code3d is not None:
            obj._currency_code3d = currency_code3d
        if expiry_date is not None:
            obj._expiry_date = expiry_date
        if account_holder_type is not None:
            obj._account_holder_type = account_holder_type
        if custom_fields is not None:
            obj._custom_fields = custom_fields
        if account_subtype is not None:
            obj._account_subtype = account_subtype
        if id is not None:
            obj._id = id
        if url is not None:
            obj._url = url
        if ach_eligible_flag is not None:
            obj._ach_eligible_flag = ach_eligible_flag
        if atm_eligible_flag is not None:
            obj._atm_eligible_flag = atm_eligible_flag
        if card_cvv_number is not None:
            obj._card_cvv_number = card_cvv_number
        if account_type is not None:
            obj._account_type = account_type
        if status is not None:
            obj._status = status
        if device_manufacturer_id is not None:
            obj._device_manufacturer_id = device_manufacturer_id
        if display_text is not None:
            obj._display_text = display_text
        if comments is not None:
            obj._comments = comments
        if deleted is not None:
            obj._deleted = deleted
        if audit_info is not None:
            obj._audit_info = audit_info
        if id_customer is not None:
            obj._id_customer = id_customer
        if id_funding_account is not None:
            obj._id_funding_account = id_funding_account
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(FundingAccount, self).to_dict()
        for attr, _ in six.iteritems(FundingAccount.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(FundingAccount, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
