# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class PaymentAuthorization(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'payment_auth_reference': 'str',	
        'id': 'str',	
        'url': 'str',	
        'auth_amount': 'str',	
        'status': 'str',	
        'payment_method': 'str',	
        'currency_code3d': 'str',	
        'card_cvv_number': 'str',	
        'entry_time': 'str',	
        'captured_time': 'str',	
        'comments': 'str',	
        'custom_fields': 'dict(str, str)',	
        'customer': 'Customer',	
        'funding_account': 'FundingAccount',	
        'customer_account': 'CustomerAccount',	
        'payment_network_response': 'PaymentNetworkResponse',	
        'audit_info': 'AuditInfo',	
        'id_customer': 'str',	
        'id_authorization': 'str'	
    }    

    attribute_map = {
        'payment_auth_reference': 'payment_auth_reference', 
        'id': 'id', 
        'url': 'url', 
        'auth_amount': 'auth_amount', 
        'status': 'status', 
        'payment_method': 'payment_method', 
        'currency_code3d': 'currency_code3d', 
        'card_cvv_number': 'card_cvv_number', 
        'entry_time': 'entry_time', 
        'captured_time': 'captured_time', 
        'comments': 'comments', 
        'custom_fields': 'custom_fields', 
        'customer': 'customer', 
        'funding_account': 'funding_account', 
        'customer_account': 'customer_account', 
        'payment_network_response': 'payment_network_response', 
        'audit_info': 'audit_info', 
        'id_customer': 'id_customer', 
        'id_authorization': 'id_authorization' 
    }
    
    def __init__(self, id_authorization=None):
        super(PaymentAuthorization, self).__init__()
        
        self._payment_auth_reference = None
        self._id = None
        self._url = None
        self._auth_amount = None
        self._status = None
        self._payment_method = None
        self._currency_code3d = None
        self._card_cvv_number = None
        self._entry_time = None
        self._captured_time = None
        self._comments = None
        self._custom_fields = None
        self._customer = None
        self._funding_account = None
        self._customer_account = None
        self._payment_network_response = None
        self._audit_info = None
        self._id_customer = None
        self._id_authorization = None
        
        self._id_authorization = id_authorization
        self._id = id_authorization

    @property
    def payment_auth_reference(self):
        """Gets the payment_auth_reference of this PaymentAuthorization.  # noqa: E501
        The unique identifier in the client system for the payment.  # noqa: E501

        :return: The payment_auth_reference of this PaymentAuthorization. 
        :rtype: str
        """
        return self._payment_auth_reference

    @property
    def id(self):
        """Gets the id of this PaymentAuthorization.  # noqa: E501
        The unique identifier assigned by EBPP to the payment authorization.  # noqa: E501

        :return: The id of this PaymentAuthorization. 
        :rtype: str
        """
        return self._id

    @property
    def url(self):
        """Gets the url of this PaymentAuthorization.  # noqa: E501
        This URL fetches the details of payment authorization.  # noqa: E501

        :return: The url of this PaymentAuthorization. 
        :rtype: str
        """
        return self._url

    @property
    def auth_amount(self):
        """Gets the auth_amount of this PaymentAuthorization.  # noqa: E501
        The amount being authorized.  # noqa: E501

        :return: The auth_amount of this PaymentAuthorization. 
        :rtype: str
        """
        return self._auth_amount

    @property
    def status(self):
        """Gets the status of this PaymentAuthorization.  # noqa: E501
        The status of the payment.  # noqa: E501

        :return: The status of this PaymentAuthorization. 
        :rtype: str
        """
        return self._status

    @property
    def payment_method(self):
        """Gets the payment_method of this PaymentAuthorization.  # noqa: E501
        The fulfillment method for the payment authorization.  # noqa: E501

        :return: The payment_method of this PaymentAuthorization. 
        :rtype: str
        """
        return self._payment_method

    @property
    def currency_code3d(self):
        """Gets the currency_code3d of this PaymentAuthorization.  # noqa: E501
        The three-letter currency code in the ISO4217 format, in uppercase, for the card account. It must be a currency supported by EBPP. This defaults to USD.  # noqa: E501

        :return: The currency_code3d of this PaymentAuthorization. 
        :rtype: str
        """
        return self._currency_code3d

    @property
    def card_cvv_number(self):
        """Gets the card_cvv_number of this PaymentAuthorization.  # noqa: E501
        This is never returned by EBPP. This needs to be passed to EBPP when making or editing a payment with a card funding account.  # noqa: E501

        :return: The card_cvv_number of this PaymentAuthorization. 
        :rtype: str
        """
        return self._card_cvv_number

    @property
    def entry_time(self):
        """Gets the entry_time of this PaymentAuthorization.  # noqa: E501
        The date on which the payment authorization created in UTC timestamp, with ISO8601 format. For e.g., 2019-08-13T09:21:34.359+0000.  # noqa: E501

        :return: The entry_time of this PaymentAuthorization. 
        :rtype: str
        """
        return self._entry_time

    @property
    def captured_time(self):
        """Gets the captured_time of this PaymentAuthorization.  # noqa: E501
        The date on which the payment authorization is captured in UTC timestamp, with ISO8601 format. For e.g., 2019-08-13T09:21:34.359+0000..  # noqa: E501

        :return: The captured_time of this PaymentAuthorization. 
        :rtype: str
        """
        return self._captured_time

    @property
    def comments(self):
        """Gets the comments of this PaymentAuthorization.  # noqa: E501
        Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (EBPP) is not responsible for the security of any sensitive information that may be passed as part of the memo.  # noqa: E501

        :return: The comments of this PaymentAuthorization. 
        :rtype: str
        """
        return self._comments

    @property
    def custom_fields(self):
        """Gets the custom_fields of this PaymentAuthorization.  # noqa: E501
        The additional information or meta-information that EBPP can accept, maintain and transmit back to the client. The custom fields need to be configured with EBPP before they can be accepted. EBPP would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields. Only the custom fields enabled for the client, in EBPP, would be accepted in the requests.  # noqa: E501

        :return: The custom_fields of this PaymentAuthorization. 
        :rtype: dict(str, str)
        """
        return self._custom_fields

    @property
    def customer(self):
        """Gets the customer of this PaymentAuthorization.  # noqa: E501

        :return: The customer of this PaymentAuthorization. 
        :rtype: Customer
        """
        return self._customer

    @property
    def funding_account(self):
        """Gets the funding_account of this PaymentAuthorization.  # noqa: E501

        :return: The funding_account of this PaymentAuthorization. 
        :rtype: FundingAccount
        """
        return self._funding_account

    @property
    def customer_account(self):
        """Gets the customer_account of this PaymentAuthorization.  # noqa: E501

        :return: The customer_account of this PaymentAuthorization. 
        :rtype: CustomerAccount
        """
        return self._customer_account

    @property
    def payment_network_response(self):
        """Gets the payment_network_response of this PaymentAuthorization.  # noqa: E501

        :return: The payment_network_response of this PaymentAuthorization. 
        :rtype: PaymentNetworkResponse
        """
        return self._payment_network_response

    @property
    def audit_info(self):
        """Gets the audit_info of this PaymentAuthorization.  # noqa: E501

        :return: The audit_info of this PaymentAuthorization. 
        :rtype: AuditInfo
        """
        return self._audit_info

    @property
    def id_customer(self):
        """Gets the id_customer of this PaymentAuthorization.  # noqa: E501
        The unique identifier assigned by EBPP to the customer.  # noqa: E501

        :return: The id_customer of this PaymentAuthorization. 
        :rtype: str
        """
        return self._id_customer

    @property
    def id_authorization(self):
        """Gets the id_authorization of this PaymentAuthorization.  # noqa: E501
        The unique identifier assigned by EBPP to the payment authorization.  # noqa: E501

        :return: The id_authorization of this PaymentAuthorization. 
        :rtype: str
        """
        return self._id_authorization

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        super(PaymentAuthorization, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        super(PaymentAuthorization, self).vide_channel(channel)
        return self
    def with_details(self, auth_amount=None, card_cvv_number=None):
        """
        :param auth_amount: The auth_amount is of type str.
        :param card_cvv_number: The card_cvv_number is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        self._auth_amount = auth_amount
        self._card_cvv_number = card_cvv_number
        return self
    def for_customer(self, id_customer=None):
        """
        :param id_customer: The id_customer is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        self._id_customer = id_customer
        from orbipay_paymentsapi_client.wrappers.customer import Customer
        method = getattr(Customer, '_' + 'Customer' + '__init_object')
        if method is not None:
            self._customer = method(None, id=id_customer)
        return self
    def from_account(self, funding_account=None):
        """
        :param funding_account: The funding_account is of type FundingAccount.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        self._funding_account = funding_account
        return self
    def to_account(self, customer_account=None):
        """
        :param customer_account: The customer_account is of type CustomerAccount.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        self._customer_account = customer_account
        return self
    def with_reference(self, payment_auth_reference=None):
        """
        :param payment_auth_reference: The payment_auth_reference is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        self._payment_auth_reference = payment_auth_reference
        return self
    def with_custom_fields(self, custom_fields=None):
        """
        :param custom_fields: The custom_fields is of type dict(str, str).
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        self._custom_fields = custom_fields
        return self
    def with_memo(self, comments=None):
        """
        :param comments: The comments is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        self._comments = comments
        return self
    ## support methods
    def cancelled_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        super(PaymentAuthorization, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def created_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        super(PaymentAuthorization, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    def cancel(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        try:
            super(PaymentAuthorization, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.cancel_payment_authorization_handler import CancelPaymentAuthorizationHandler
            return CancelPaymentAuthorizationHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def create(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of PaymentAuthorization. 
        :rtype: PaymentAuthorization
        """
        try:
            super(PaymentAuthorization, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.create_payment_authorization_handler import CreatePaymentAuthorizationHandler
            return CreatePaymentAuthorizationHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, payment_auth_reference=None, id=None, url=None, auth_amount=None, status=None, payment_method=None, currency_code3d=None, card_cvv_number=None, entry_time=None, captured_time=None, comments=None, custom_fields=None, customer=None, funding_account=None, customer_account=None, payment_network_response=None, audit_info=None, id_customer=None, id_authorization=None):
        if obj is None:
            obj = PaymentAuthorization()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if payment_auth_reference is not None:
            obj._payment_auth_reference = payment_auth_reference
        if id is not None:
            obj._id = id
        if url is not None:
            obj._url = url
        if auth_amount is not None:
            obj._auth_amount = auth_amount
        if status is not None:
            obj._status = status
        if payment_method is not None:
            obj._payment_method = payment_method
        if currency_code3d is not None:
            obj._currency_code3d = currency_code3d
        if card_cvv_number is not None:
            obj._card_cvv_number = card_cvv_number
        if entry_time is not None:
            obj._entry_time = entry_time
        if captured_time is not None:
            obj._captured_time = captured_time
        if comments is not None:
            obj._comments = comments
        if custom_fields is not None:
            obj._custom_fields = custom_fields
        if customer is not None:
            obj._customer = customer
        if funding_account is not None:
            obj._funding_account = funding_account
        if customer_account is not None:
            obj._customer_account = customer_account
        if payment_network_response is not None:
            obj._payment_network_response = payment_network_response
        if audit_info is not None:
            obj._audit_info = audit_info
        if id_customer is not None:
            obj._id_customer = id_customer
        if id_authorization is not None:
            obj._id_authorization = id_authorization
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(PaymentAuthorization, self).to_dict()
        for attr, _ in six.iteritems(PaymentAuthorization.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PaymentAuthorization, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
