# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class PaymentSchedule(object):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'payment_recurring_type': 'str',	
        'payment_recurring_count': 'str',	
        'payment_amount_type': 'str',	
        'payment_start_date': 'str',	
        'payment_end_date': 'str',	
        'payment_limit_amount': 'str',	
        'payment_plan_id': 'str',	
        'payment_defer_days': 'str'	
    }    

    attribute_map = {
        'payment_recurring_type': 'payment_recurring_type', 
        'payment_recurring_count': 'payment_recurring_count', 
        'payment_amount_type': 'payment_amount_type', 
        'payment_start_date': 'payment_start_date', 
        'payment_end_date': 'payment_end_date', 
        'payment_limit_amount': 'payment_limit_amount', 
        'payment_plan_id': 'payment_plan_id', 
        'payment_defer_days': 'payment_defer_days' 
    }
    
    def __init__(self, payment_recurring_type=None, payment_recurring_count=None, payment_amount_type=None, payment_start_date=None, payment_end_date=None, payment_limit_amount=None, payment_plan_id=None, payment_defer_days=None):
        super(PaymentSchedule, self).__init__()
        
        self._payment_recurring_type = None
        self._payment_recurring_count = None
        self._payment_amount_type = None
        self._payment_start_date = None
        self._payment_end_date = None
        self._payment_limit_amount = None
        self._payment_plan_id = None
        self._payment_defer_days = None
        
        self._payment_recurring_type = payment_recurring_type
        self._payment_recurring_count = payment_recurring_count
        self._payment_amount_type = payment_amount_type
        self._payment_start_date = payment_start_date
        self._payment_end_date = payment_end_date
        self._payment_limit_amount = payment_limit_amount
        self._payment_plan_id = payment_plan_id
        self._payment_defer_days = payment_defer_days

    @property
    def payment_recurring_type(self):
        """Gets the payment_recurring_type of this PaymentSchedule.  # noqa: E501
        The frequency of the payments created as part of a payment setup.  # noqa: E501

        :return: The payment_recurring_type of this PaymentSchedule. 
        :rtype: str
        """
        return self._payment_recurring_type

    @property
    def payment_recurring_count(self):
        """Gets the payment_recurring_count of this PaymentSchedule.  # noqa: E501
        The maximum number of payments that can be created as part of a recurring payment setup.  # noqa: E501

        :return: The payment_recurring_count of this PaymentSchedule. 
        :rtype: str
        """
        return self._payment_recurring_count

    @property
    def payment_amount_type(self):
        """Gets the payment_amount_type of this PaymentSchedule.  # noqa: E501
        The amount type that is used to set the amount for payments created as part of a recurring payment or autopay setup.   # noqa: E501

        :return: The payment_amount_type of this PaymentSchedule. 
        :rtype: str
        """
        return self._payment_amount_type

    @property
    def payment_start_date(self):
        """Gets the payment_start_date of this PaymentSchedule.  # noqa: E501
        The date on which the first payment will be drafted as part of a recurring payment setup. This is to be specified in the ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The payment_start_date of this PaymentSchedule. 
        :rtype: str
        """
        return self._payment_start_date

    @property
    def payment_end_date(self):
        """Gets the payment_end_date of this PaymentSchedule.  # noqa: E501
        The date by which the last payment will be drafted as part of a recurring payment setup. This is to be specified in the ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The payment_end_date of this PaymentSchedule. 
        :rtype: str
        """
        return self._payment_end_date

    @property
    def payment_limit_amount(self):
        """Gets the payment_limit_amount of this PaymentSchedule.  # noqa: E501
        The upper limit for the payment amount in the case of recurring or autopay payments.  # noqa: E501

        :return: The payment_limit_amount of this PaymentSchedule. 
        :rtype: str
        """
        return self._payment_limit_amount

    @property
    def payment_plan_id(self):
        """Gets the payment_plan_id of this PaymentSchedule.  # noqa: E501
        The reference to the payment plan, offered by the biller, that is used to set up the recurring payment. The list of payment plans offered is published as part of the biller set-up.  # noqa: E501

        :return: The payment_plan_id of this PaymentSchedule. 
        :rtype: str
        """
        return self._payment_plan_id

    @property
    def payment_defer_days(self):
        """Gets the payment_defer_days of this PaymentSchedule.  # noqa: E501
        The number of days from the bill due date that the customer wishes to delay the autopay payment by.  # noqa: E501

        :return: The payment_defer_days of this PaymentSchedule. 
        :rtype: str
        """
        return self._payment_defer_days

    def with_payment_limit_amount(self, payment_limit_amount=None):
        """
        :param payment_limit_amount: The payment_limit_amount is of type str.
        
        :return: The Instance of PaymentSchedule. 
        :rtype: PaymentSchedule
        """
        self._payment_limit_amount = payment_limit_amount
        return self
    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, payment_recurring_type=None, payment_recurring_count=None, payment_amount_type=None, payment_start_date=None, payment_end_date=None, payment_limit_amount=None, payment_plan_id=None, payment_defer_days=None):
        if obj is None:
            obj = PaymentSchedule()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if payment_recurring_type is not None:
            obj._payment_recurring_type = payment_recurring_type
        if payment_recurring_count is not None:
            obj._payment_recurring_count = payment_recurring_count
        if payment_amount_type is not None:
            obj._payment_amount_type = payment_amount_type
        if payment_start_date is not None:
            obj._payment_start_date = payment_start_date
        if payment_end_date is not None:
            obj._payment_end_date = payment_end_date
        if payment_limit_amount is not None:
            obj._payment_limit_amount = payment_limit_amount
        if payment_plan_id is not None:
            obj._payment_plan_id = payment_plan_id
        if payment_defer_days is not None:
            obj._payment_defer_days = payment_defer_days
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}
        for attr, _ in six.iteritems(PaymentSchedule.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PaymentSchedule, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
