# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class PaymentSetup(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'id': 'str',	
        'url': 'str',	
        'fee': 'Fee',	
        'status': 'str',	
        'payment_method': 'str',	
        'payment_setup_schedule_type': 'str',	
        'confirmation_number': 'str',	
        'custom_fields': 'dict(str, str)',	
        'currency_code3d': 'str',	
        'amount': 'str',	
        'card_cvv_number': 'str',	
        'next_payment_date': 'str',	
        'payment_setup_reference': 'str',	
        'payment_schedule': 'PaymentSchedule',	
        'payment_setup_entry_date': 'str',	
        'comments': 'str',	
        'deleted': 'str',	
        'customer': 'Customer',	
        'funding_account': 'FundingAccount',	
        'customer_account': 'CustomerAccount',	
        'payment_network_response': 'PaymentNetworkResponse',	
        'audit_info': 'AuditInfo',	
        'id_customer': 'str',	
        'id_payment_setup': 'str'	
    }    

    attribute_map = {
        'id': 'id', 
        'url': 'url', 
        'fee': 'fee', 
        'status': 'status', 
        'payment_method': 'payment_method', 
        'payment_setup_schedule_type': 'payment_setup_schedule_type', 
        'confirmation_number': 'confirmation_number', 
        'custom_fields': 'custom_fields', 
        'currency_code3d': 'currency_code3d', 
        'amount': 'amount', 
        'card_cvv_number': 'card_cvv_number', 
        'next_payment_date': 'next_payment_date', 
        'payment_setup_reference': 'payment_setup_reference', 
        'payment_schedule': 'payment_schedule', 
        'payment_setup_entry_date': 'payment_setup_entry_date', 
        'comments': 'comments', 
        'deleted': 'deleted', 
        'customer': 'customer', 
        'funding_account': 'funding_account', 
        'customer_account': 'customer_account', 
        'payment_network_response': 'payment_network_response', 
        'audit_info': 'audit_info', 
        'id_customer': 'id_customer', 
        'id_payment_setup': 'id_payment_setup' 
    }
    
    def __init__(self, id_payment_setup=None):
        super(PaymentSetup, self).__init__()
        
        self._id = None
        self._url = None
        self._fee = None
        self._status = None
        self._payment_method = None
        self._payment_setup_schedule_type = None
        self._confirmation_number = None
        self._custom_fields = None
        self._currency_code3d = None
        self._amount = None
        self._card_cvv_number = None
        self._next_payment_date = None
        self._payment_setup_reference = None
        self._payment_schedule = None
        self._payment_setup_entry_date = None
        self._comments = None
        self._deleted = None
        self._customer = None
        self._funding_account = None
        self._customer_account = None
        self._payment_network_response = None
        self._audit_info = None
        self._id_customer = None
        self._id_payment_setup = None
        
        self._id_payment_setup = id_payment_setup
        self._id = id_payment_setup

    @property
    def id(self):
        """Gets the id of this PaymentSetup.  # noqa: E501
        The unique identifier assigned by Orbipay to the payment setup.  # noqa: E501

        :return: The id of this PaymentSetup. 
        :rtype: str
        """
        return self._id

    @property
    def url(self):
        """Gets the url of this PaymentSetup.  # noqa: E501
        This URL fetches the details of payment setup.  # noqa: E501

        :return: The url of this PaymentSetup. 
        :rtype: str
        """
        return self._url

    @property
    def fee(self):
        """Gets the fee of this PaymentSetup.  # noqa: E501

        :return: The fee of this PaymentSetup. 
        :rtype: Fee
        """
        return self._fee

    @property
    def status(self):
        """Gets the status of this PaymentSetup.  # noqa: E501
        The status of the payment.  # noqa: E501

        :return: The status of this PaymentSetup. 
        :rtype: str
        """
        return self._status

    @property
    def payment_method(self):
        """Gets the payment_method of this PaymentSetup.  # noqa: E501
        The fulfillment method for the payment.  # noqa: E501

        :return: The payment_method of this PaymentSetup. 
        :rtype: str
        """
        return self._payment_method

    @property
    def payment_setup_schedule_type(self):
        """Gets the payment_setup_schedule_type of this PaymentSetup.  # noqa: E501
        The schedule type for the payment setup.  # noqa: E501

        :return: The payment_setup_schedule_type of this PaymentSetup. 
        :rtype: str
        """
        return self._payment_setup_schedule_type

    @property
    def confirmation_number(self):
        """Gets the confirmation_number of this PaymentSetup.  # noqa: E501
        The confirmation number or reference provided to the customer for the payment setup.  # noqa: E501

        :return: The confirmation_number of this PaymentSetup. 
        :rtype: str
        """
        return self._confirmation_number

    @property
    def custom_fields(self):
        """Gets the custom_fields of this PaymentSetup.  # noqa: E501
        The additional information or meta-information that Orbipay can accept, maintain and transmit back to the client. The custom fields need to be configured with Orbipay before they can be accepted. Orbipay would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields. Only the custom fields enabled for the client, in Orbipay, would be accepted in the requests.  # noqa: E501

        :return: The custom_fields of this PaymentSetup. 
        :rtype: dict(str, str)
        """
        return self._custom_fields

    @property
    def currency_code3d(self):
        """Gets the currency_code3d of this PaymentSetup.  # noqa: E501
        The three-letter currency code in the ISO4217 format, in uppercase, for the card account. It must be a currency supported by Orbipay. This defaults to USD.  # noqa: E501

        :return: The currency_code3d of this PaymentSetup. 
        :rtype: str
        """
        return self._currency_code3d

    @property
    def amount(self):
        """Gets the amount of this PaymentSetup.  # noqa: E501
        The amount being paid. In case the payment_amount_type is 'discounted_amount', 'penalty_amount' or 'other', the amount needs to be specified. In other cases, this is derived from the payment_amount_type.  # noqa: E501

        :return: The amount of this PaymentSetup. 
        :rtype: str
        """
        return self._amount

    @property
    def card_cvv_number(self):
        """Gets the card_cvv_number of this PaymentSetup.  # noqa: E501
        This is never returned by Orbipay. This needs to be passed to Orbipay when making or editing a payment setup with a card funding account.  # noqa: E501

        :return: The card_cvv_number of this PaymentSetup. 
        :rtype: str
        """
        return self._card_cvv_number

    @property
    def next_payment_date(self):
        """Gets the next_payment_date of this PaymentSetup.  # noqa: E501
        The date on which the payment is scheduled to be made. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The next_payment_date of this PaymentSetup. 
        :rtype: str
        """
        return self._next_payment_date

    @property
    def payment_setup_reference(self):
        """Gets the payment_setup_reference of this PaymentSetup.  # noqa: E501
        The unique identifier in the client system for the payment setup.  # noqa: E501

        :return: The payment_setup_reference of this PaymentSetup. 
        :rtype: str
        """
        return self._payment_setup_reference

    @property
    def payment_schedule(self):
        """Gets the payment_schedule of this PaymentSetup.  # noqa: E501

        :return: The payment_schedule of this PaymentSetup. 
        :rtype: PaymentSchedule
        """
        return self._payment_schedule

    @property
    def payment_setup_entry_date(self):
        """Gets the payment_setup_entry_date of this PaymentSetup.  # noqa: E501
        The date on which the payment setup is captured in Orbipay. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The payment_setup_entry_date of this PaymentSetup. 
        :rtype: str
        """
        return self._payment_setup_entry_date

    @property
    def comments(self):
        """Gets the comments of this PaymentSetup.  # noqa: E501
        Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (Orbipay) is not responsible for the security of any sensitive information that may be passed as part of the memo.  # noqa: E501

        :return: The comments of this PaymentSetup. 
        :rtype: str
        """
        return self._comments

    @property
    def deleted(self):
        """Gets the deleted of this PaymentSetup.  # noqa: E501
        The status of payment which says whether the payment is deleted or not  # noqa: E501

        :return: The deleted of this PaymentSetup. 
        :rtype: str
        """
        return self._deleted

    @property
    def customer(self):
        """Gets the customer of this PaymentSetup.  # noqa: E501

        :return: The customer of this PaymentSetup. 
        :rtype: Customer
        """
        return self._customer

    @property
    def funding_account(self):
        """Gets the funding_account of this PaymentSetup.  # noqa: E501

        :return: The funding_account of this PaymentSetup. 
        :rtype: FundingAccount
        """
        return self._funding_account

    @property
    def customer_account(self):
        """Gets the customer_account of this PaymentSetup.  # noqa: E501

        :return: The customer_account of this PaymentSetup. 
        :rtype: CustomerAccount
        """
        return self._customer_account

    @property
    def payment_network_response(self):
        """Gets the payment_network_response of this PaymentSetup.  # noqa: E501

        :return: The payment_network_response of this PaymentSetup. 
        :rtype: PaymentNetworkResponse
        """
        return self._payment_network_response

    @property
    def audit_info(self):
        """Gets the audit_info of this PaymentSetup.  # noqa: E501

        :return: The audit_info of this PaymentSetup. 
        :rtype: AuditInfo
        """
        return self._audit_info

    @property
    def id_customer(self):
        """Gets the id_customer of this PaymentSetup.  # noqa: E501
        The unique identifier assigned by EBPP to the customer.  # noqa: E501

        :return: The id_customer of this PaymentSetup. 
        :rtype: str
        """
        return self._id_customer

    @property
    def id_payment_setup(self):
        """Gets the id_payment_setup of this PaymentSetup.  # noqa: E501
        The unique identifier assigned by EBPP to the payment setup.  # noqa: E501

        :return: The id_payment_setup of this PaymentSetup. 
        :rtype: str
        """
        return self._id_payment_setup

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        super(PaymentSetup, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        super(PaymentSetup, self).vide_channel(channel)
        return self
    def with_details(self, amount=None, card_cvv_number=None, payment_setup_schedule_type=None):
        """
        :param amount: The amount is of type str.
        :param card_cvv_number: The card_cvv_number is of type str.
        :param payment_setup_schedule_type: The payment_setup_schedule_type is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._amount = amount
        self._card_cvv_number = card_cvv_number
        self._payment_setup_schedule_type = payment_setup_schedule_type
        return self
    def from_account(self, funding_account=None):
        """
        :param funding_account: The funding_account is of type FundingAccount.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._funding_account = funding_account
        return self
    def to_account(self, customer_account=None):
        """
        :param customer_account: The customer_account is of type CustomerAccount.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._customer_account = customer_account
        return self
    def with_reference(self, payment_setup_reference=None):
        """
        :param payment_setup_reference: The payment_setup_reference is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._payment_setup_reference = payment_setup_reference
        return self
    def with_payment_schedule(self, payment_schedule=None):
        """
        :param payment_schedule: The payment_schedule is of type PaymentSchedule.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._payment_schedule = payment_schedule
        return self
    def with_status(self, status=None):
        """
        :param status: The status is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._status = status
        return self
    def with_custom_fields(self, custom_fields=None):
        """
        :param custom_fields: The custom_fields is of type dict(str, str).
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._custom_fields = custom_fields
        return self
    def with_memo(self, comments=None):
        """
        :param comments: The comments is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._comments = comments
        return self
    def with_fee(self, fee=None):
        """
        :param fee: The fee is of type Fee.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._fee = fee
        return self
    def for_customer(self, id_customer=None):
        """
        :param id_customer: The id_customer is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        self._id_customer = id_customer
        from orbipay_paymentsapi_client.wrappers.customer import Customer
        method = getattr(Customer, '_' + 'Customer' + '__init_object')
        if method is not None:
            self._customer = method(None, id=id_customer)
        return self
    ## support methods
    def created_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        super(PaymentSetup, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def deleted_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        super(PaymentSetup, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def retrieved_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        super(PaymentSetup, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def updated_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        super(PaymentSetup, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    def create(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        try:
            super(PaymentSetup, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.create_recurring_payment_setup_handler import CreateRecurringPaymentSetupHandler
            return CreateRecurringPaymentSetupHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def delete(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        try:
            super(PaymentSetup, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.delete_payment_setup_handler import DeletePaymentSetupHandler
            return DeletePaymentSetupHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def get(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        try:
            super(PaymentSetup, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.get_payment_setup_handler import GetPaymentSetupHandler
            return GetPaymentSetupHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def update(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of PaymentSetup. 
        :rtype: PaymentSetup
        """
        try:
            super(PaymentSetup, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.update_payment_setup_handler import UpdatePaymentSetupHandler
            return UpdatePaymentSetupHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, id=None, url=None, fee=None, status=None, payment_method=None, payment_setup_schedule_type=None, confirmation_number=None, custom_fields=None, currency_code3d=None, amount=None, card_cvv_number=None, next_payment_date=None, payment_setup_reference=None, payment_schedule=None, payment_setup_entry_date=None, comments=None, deleted=None, customer=None, funding_account=None, customer_account=None, payment_network_response=None, audit_info=None, id_customer=None, id_payment_setup=None):
        if obj is None:
            obj = PaymentSetup()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if id is not None:
            obj._id = id
        if url is not None:
            obj._url = url
        if fee is not None:
            obj._fee = fee
        if status is not None:
            obj._status = status
        if payment_method is not None:
            obj._payment_method = payment_method
        if payment_setup_schedule_type is not None:
            obj._payment_setup_schedule_type = payment_setup_schedule_type
        if confirmation_number is not None:
            obj._confirmation_number = confirmation_number
        if custom_fields is not None:
            obj._custom_fields = custom_fields
        if currency_code3d is not None:
            obj._currency_code3d = currency_code3d
        if amount is not None:
            obj._amount = amount
        if card_cvv_number is not None:
            obj._card_cvv_number = card_cvv_number
        if next_payment_date is not None:
            obj._next_payment_date = next_payment_date
        if payment_setup_reference is not None:
            obj._payment_setup_reference = payment_setup_reference
        if payment_schedule is not None:
            obj._payment_schedule = payment_schedule
        if payment_setup_entry_date is not None:
            obj._payment_setup_entry_date = payment_setup_entry_date
        if comments is not None:
            obj._comments = comments
        if deleted is not None:
            obj._deleted = deleted
        if customer is not None:
            obj._customer = customer
        if funding_account is not None:
            obj._funding_account = funding_account
        if customer_account is not None:
            obj._customer_account = customer_account
        if payment_network_response is not None:
            obj._payment_network_response = payment_network_response
        if audit_info is not None:
            obj._audit_info = audit_info
        if id_customer is not None:
            obj._id_customer = id_customer
        if id_payment_setup is not None:
            obj._id_payment_setup = id_payment_setup
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(PaymentSetup, self).to_dict()
        for attr, _ in six.iteritems(PaymentSetup.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PaymentSetup, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
