# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class PaymentSetups(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'list': 'list[PaymentSetup]',	
        'total_results_count': 'str',	
        'has_more_results': 'str',	
        'url': 'str',	
        'from_index': 'str',	
        'to_index': 'str',	
        'query_id': 'str',	
        'id_customer': 'str',	
        'id_funding_account': 'str',	
        'id_customer_account': 'str',	
        'confirmation_number': 'str',	
        'status': 'list[str]',	
        'payment_setup_schedule_type': 'list[str]',	
        'from_date': 'str',	
        'to_date': 'str',	
        'page_size': 'str'	
    }    

    attribute_map = {
        'list': 'list', 
        'total_results_count': 'total_results_count', 
        'has_more_results': 'has_more_results', 
        'url': 'url', 
        'from_index': 'from_index', 
        'to_index': 'to_index', 
        'query_id': 'query_id', 
        'id_customer': 'id_customer', 
        'id_funding_account': 'id_funding_account', 
        'id_customer_account': 'id_customer_account', 
        'confirmation_number': 'confirmation_number', 
        'status': 'status', 
        'payment_setup_schedule_type': 'payment_setup_schedule_type', 
        'from_date': 'from_date', 
        'to_date': 'to_date', 
        'page_size': 'page_size' 
    }
    
    def __init__(self):
        super(PaymentSetups, self).__init__()
        self._list = None
        self._total_results_count = None
        self._has_more_results = None
        self._url = None
        self._from_index = None
        self._to_index = None
        self._query_id = None
        self._id_customer = None
        self._id_funding_account = None
        self._id_customer_account = None
        self._confirmation_number = None
        self._status = None
        self._payment_setup_schedule_type = None
        self._from_date = None
        self._to_date = None
        self._page_size = None

    @property
    def list(self):
        """Gets the list of this PaymentSetups.  # noqa: E501

        :return: The list of this PaymentSetups. 
        :rtype: list[PaymentSetup]
        """
        return self._list

    @property
    def total_results_count(self):
        """Gets the total_results_count of this PaymentSetups.  # noqa: E501
        The total number of payment setup available matching the search criteria.  # noqa: E501

        :return: The total_results_count of this PaymentSetups. 
        :rtype: str
        """
        return self._total_results_count

    @property
    def has_more_results(self):
        """Gets the has_more_results of this PaymentSetups.  # noqa: E501
        true if there are more results or pages  # noqa: E501

        :return: The has_more_results of this PaymentSetups. 
        :rtype: str
        """
        return self._has_more_results

    @property
    def url(self):
        """Gets the url of this PaymentSetups.  # noqa: E501
        This URL fetches the next or previous set of payment setups, based on the presence of after_object or before_object parameter.  # noqa: E501

        :return: The url of this PaymentSetups. 
        :rtype: str
        """
        return self._url

    @property
    def from_index(self):
        """Gets the from_index of this PaymentSetups.  # noqa: E501
        The id of the object before which the previous set of objects are to be retrieved.  # noqa: E501

        :return: The from_index of this PaymentSetups. 
        :rtype: str
        """
        return self._from_index

    @property
    def to_index(self):
        """Gets the to_index of this PaymentSetups.  # noqa: E501
        The id of the object after which the next set of objects are to be retrieved.  # noqa: E501

        :return: The to_index of this PaymentSetups. 
        :rtype: str
        """
        return self._to_index

    @property
    def query_id(self):
        """Gets the query_id of this PaymentSetups.  # noqa: E501
        The query id of the object.  # noqa: E501

        :return: The query_id of this PaymentSetups. 
        :rtype: str
        """
        return self._query_id

    @property
    def id_customer(self):
        """Gets the id_customer of this PaymentSetups.  # noqa: E501
        The unique identifier assigned by EBPP to the customer.  # noqa: E501

        :return: The id_customer of this PaymentSetups. 
        :rtype: str
        """
        return self._id_customer

    @property
    def id_funding_account(self):
        """Gets the id_funding_account of this PaymentSetups.  # noqa: E501
        The unique identifier assigned by EBPP to the funding account  # noqa: E501

        :return: The id_funding_account of this PaymentSetups. 
        :rtype: str
        """
        return self._id_funding_account

    @property
    def id_customer_account(self):
        """Gets the id_customer_account of this PaymentSetups.  # noqa: E501
        The unique identifier assigned by EBPP to the customer account  # noqa: E501

        :return: The id_customer_account of this PaymentSetups. 
        :rtype: str
        """
        return self._id_customer_account

    @property
    def confirmation_number(self):
        """Gets the confirmation_number of this PaymentSetups.  # noqa: E501
        The confirmation number or reference provided to the customer for the successful payment.  # noqa: E501

        :return: The confirmation_number of this PaymentSetups. 
        :rtype: str
        """
        return self._confirmation_number

    @property
    def status(self):
        """Gets the status of this PaymentSetups.  # noqa: E501
        The status of the payment. This can take multiple values in the format key=value1&key=value2....  # noqa: E501

        :return: The status of this PaymentSetups. 
        :rtype: list[str]
        """
        return self._status

    @property
    def payment_setup_schedule_type(self):
        """Gets the payment_setup_schedule_type of this PaymentSetups.  # noqa: E501
        The schedule type for the payment setup. This can take multiple values in the format key=value1&key=value2....  # noqa: E501

        :return: The payment_setup_schedule_type of this PaymentSetups. 
        :rtype: list[str]
        """
        return self._payment_setup_schedule_type

    @property
    def from_date(self):
        """Gets the from_date of this PaymentSetups.  # noqa: E501
        The date from which recurring payments setup, matching the criteria specified, need to be retrieved.  # noqa: E501

        :return: The from_date of this PaymentSetups. 
        :rtype: str
        """
        return self._from_date

    @property
    def to_date(self):
        """Gets the to_date of this PaymentSetups.  # noqa: E501
        The date upto which recurring payments setup, matching the criteria specified, need to be retrieved.  # noqa: E501

        :return: The to_date of this PaymentSetups. 
        :rtype: str
        """
        return self._to_date

    @property
    def page_size(self):
        """Gets the page_size of this PaymentSetups.  # noqa: E501
        The maximum number of objects returned in the query.  # noqa: E501

        :return: The page_size of this PaymentSetups. 
        :rtype: str
        """
        return self._page_size

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of PaymentSetups. 
        :rtype: PaymentSetups
        """
        super(PaymentSetups, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of PaymentSetups. 
        :rtype: PaymentSetups
        """
        super(PaymentSetups, self).vide_channel(channel)
        return self
    def find_by(self, filter_parameters=None):
        if(filter_parameters):

            if(len(filter_parameters['id_funding_account'])>0):
                self._id_funding_account = filter_parameters['id_funding_account'][0]

            if(len(filter_parameters['id_customer_account'])>0):
                self._id_customer_account = filter_parameters['id_customer_account'][0]

            if(len(filter_parameters['confirmation_number'])>0):
                self._confirmation_number = filter_parameters['confirmation_number'][0]
            self._payment_setup_schedule_type = filter_parameters['payment_setup_schedule_type']


            if(len(filter_parameters['from_date'])>0):
                self._from_date = filter_parameters['from_date'][0]

            if(len(filter_parameters['to_date'])>0):
                self._to_date = filter_parameters['to_date'][0]
            self._status = filter_parameters['status']


            if(len(filter_parameters['page_size'])>0):
                self._page_size = filter_parameters['page_size'][0]

            if(len(filter_parameters['query_id'])>0):
                self._query_id = filter_parameters['query_id'][0]

            if(len(filter_parameters['from_index'])>0):
                self._from_index = filter_parameters['from_index'][0]

            if(len(filter_parameters['to_index'])>0):
                self._to_index = filter_parameters['to_index'][0]
        return self
    def for_customer(self, id_customer=None):
        """
        :param id_customer: The id_customer is of type str.
        
        :return: The Instance of PaymentSetups. 
        :rtype: PaymentSetups
        """
        self._id_customer = id_customer
        return self
    ## support methods
    def retrieved_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of PaymentSetups. 
        :rtype: PaymentSetups
        """
        super(PaymentSetups, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    def get(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of PaymentSetups. 
        :rtype: PaymentSetups
        """
        try:
            super(PaymentSetups, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.retrieve_payment_setups_handler import RetrievePaymentSetupsHandler
            return RetrievePaymentSetupsHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, list=None, total_results_count=None, has_more_results=None, url=None, from_index=None, to_index=None, query_id=None, id_customer=None, id_funding_account=None, id_customer_account=None, confirmation_number=None, status=None, payment_setup_schedule_type=None, from_date=None, to_date=None, page_size=None):
        if obj is None:
            obj = PaymentSetups()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if list is not None:
            obj._list = list
        if total_results_count is not None:
            obj._total_results_count = total_results_count
        if has_more_results is not None:
            obj._has_more_results = has_more_results
        if url is not None:
            obj._url = url
        if from_index is not None:
            obj._from_index = from_index
        if to_index is not None:
            obj._to_index = to_index
        if query_id is not None:
            obj._query_id = query_id
        if id_customer is not None:
            obj._id_customer = id_customer
        if id_funding_account is not None:
            obj._id_funding_account = id_funding_account
        if id_customer_account is not None:
            obj._id_customer_account = id_customer_account
        if confirmation_number is not None:
            obj._confirmation_number = confirmation_number
        if status is not None:
            obj._status = status
        if payment_setup_schedule_type is not None:
            obj._payment_setup_schedule_type = payment_setup_schedule_type
        if from_date is not None:
            obj._from_date = from_date
        if to_date is not None:
            obj._to_date = to_date
        if page_size is not None:
            obj._page_size = page_size
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(PaymentSetups, self).to_dict()
        for attr, _ in six.iteritems(PaymentSetups.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PaymentSetups, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
