# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class RecordedPayment(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'id': 'str',	
        'url': 'str',	
        'fee': 'Fee',	
        'status': 'str',	
        'payment_method': 'str',	
        'token': 'str',	
        'confirmation_number': 'str',	
        'custom_fields': 'dict(str, str)',	
        'currency_code3d': 'str',	
        'amount': 'str',	
        'payment_date': 'str',	
        'payment_schedule_type': 'str',	
        'payment_amount_type': 'str',	
        'payment_reference': 'str',	
        'payment_entry_date': 'str',	
        'payment_return_date': 'str',	
        'comments': 'str',	
        'return_code': 'str',	
        'customer': 'Customer',	
        'funding_account': 'FundingAccount',	
        'customer_account': 'CustomerAccount',	
        'audit_info': 'AuditInfo',	
        'id_payment': 'str'	
    }    

    attribute_map = {
        'id': 'id', 
        'url': 'url', 
        'fee': 'fee', 
        'status': 'status', 
        'payment_method': 'payment_method', 
        'token': 'token', 
        'confirmation_number': 'confirmation_number', 
        'custom_fields': 'custom_fields', 
        'currency_code3d': 'currency_code3d', 
        'amount': 'amount', 
        'payment_date': 'payment_date', 
        'payment_schedule_type': 'payment_schedule_type', 
        'payment_amount_type': 'payment_amount_type', 
        'payment_reference': 'payment_reference', 
        'payment_entry_date': 'payment_entry_date', 
        'payment_return_date': 'payment_return_date', 
        'comments': 'comments', 
        'return_code': 'return_code', 
        'customer': 'customer', 
        'funding_account': 'funding_account', 
        'customer_account': 'customer_account', 
        'audit_info': 'audit_info', 
        'id_payment': 'id_payment' 
    }
    
    def __init__(self, id_payment=None):
        super(RecordedPayment, self).__init__()
        
        self._id = None
        self._url = None
        self._fee = None
        self._status = None
        self._payment_method = None
        self._token = None
        self._confirmation_number = None
        self._custom_fields = None
        self._currency_code3d = None
        self._amount = None
        self._payment_date = None
        self._payment_schedule_type = None
        self._payment_amount_type = None
        self._payment_reference = None
        self._payment_entry_date = None
        self._payment_return_date = None
        self._comments = None
        self._return_code = None
        self._customer = None
        self._funding_account = None
        self._customer_account = None
        self._audit_info = None
        self._id_payment = None
        
        self._id_payment = id_payment
        self._id = id_payment

    @property
    def id(self):
        """Gets the id of this RecordedPayment.  # noqa: E501
        The unique identifier assigned by EBPP to the payment.  # noqa: E501

        :return: The id of this RecordedPayment. 
        :rtype: str
        """
        return self._id

    @property
    def url(self):
        """Gets the url of this RecordedPayment.  # noqa: E501
        This URL fetches the details of payment.  # noqa: E501

        :return: The url of this RecordedPayment. 
        :rtype: str
        """
        return self._url

    @property
    def fee(self):
        """Gets the fee of this RecordedPayment.  # noqa: E501

        :return: The fee of this RecordedPayment. 
        :rtype: Fee
        """
        return self._fee

    @property
    def status(self):
        """Gets the status of this RecordedPayment.  # noqa: E501
        The status of the payment.  # noqa: E501

        :return: The status of this RecordedPayment. 
        :rtype: str
        """
        return self._status

    @property
    def payment_method(self):
        """Gets the payment_method of this RecordedPayment.  # noqa: E501
        The non fulfillment method for the payment.  # noqa: E501

        :return: The payment_method of this RecordedPayment. 
        :rtype: str
        """
        return self._payment_method

    @property
    def token(self):
        """Gets the token of this RecordedPayment.  # noqa: E501
        The token is used to uniquely identify the payment, such as the POS token.  # noqa: E501

        :return: The token of this RecordedPayment. 
        :rtype: str
        """
        return self._token

    @property
    def confirmation_number(self):
        """Gets the confirmation_number of this RecordedPayment.  # noqa: E501
        The confirmation number or reference provided to the customer for the payment.  # noqa: E501

        :return: The confirmation_number of this RecordedPayment. 
        :rtype: str
        """
        return self._confirmation_number

    @property
    def custom_fields(self):
        """Gets the custom_fields of this RecordedPayment.  # noqa: E501
        The additional information or meta-information that EBPP can accept, maintain and transmit back to the client. The custom fields need to be configured with EBPP before they can be accepted. EBPP would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields.  # noqa: E501

        :return: The custom_fields of this RecordedPayment. 
        :rtype: dict(str, str)
        """
        return self._custom_fields

    @property
    def currency_code3d(self):
        """Gets the currency_code3d of this RecordedPayment.  # noqa: E501
        The three-letter currency code in the ISO4217 format, in uppercase, for the card account. It must be a currency supported by EBPP. This defaults to USD.  # noqa: E501

        :return: The currency_code3d of this RecordedPayment. 
        :rtype: str
        """
        return self._currency_code3d

    @property
    def amount(self):
        """Gets the amount of this RecordedPayment.  # noqa: E501
        The amount being paid. In case the payment_amount_type is 'other', the amount needs to be specified. In other cases, this is derived from the payment_amount_type.  # noqa: E501

        :return: The amount of this RecordedPayment. 
        :rtype: str
        """
        return self._amount

    @property
    def payment_date(self):
        """Gets the payment_date of this RecordedPayment.  # noqa: E501
        The date on which the payment is scheduled to be made. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The payment_date of this RecordedPayment. 
        :rtype: str
        """
        return self._payment_date

    @property
    def payment_schedule_type(self):
        """Gets the payment_schedule_type of this RecordedPayment.  # noqa: E501
        The schedule type for the payment. This defaults to one_time_payment.  # noqa: E501

        :return: The payment_schedule_type of this RecordedPayment. 
        :rtype: str
        """
        return self._payment_schedule_type

    @property
    def payment_amount_type(self):
        """Gets the payment_amount_type of this RecordedPayment.  # noqa: E501
        The type of amount, as per the bill/statement, that is being paid. The payment amount is derived from the amount type selected for the payment. This defaults to other.  # noqa: E501

        :return: The payment_amount_type of this RecordedPayment. 
        :rtype: str
        """
        return self._payment_amount_type

    @property
    def payment_reference(self):
        """Gets the payment_reference of this RecordedPayment.  # noqa: E501
        The unique identifier in the client system for the payment.  # noqa: E501

        :return: The payment_reference of this RecordedPayment. 
        :rtype: str
        """
        return self._payment_reference

    @property
    def payment_entry_date(self):
        """Gets the payment_entry_date of this RecordedPayment.  # noqa: E501
        The date on which the payment is captured in EBPP. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The payment_entry_date of this RecordedPayment. 
        :rtype: str
        """
        return self._payment_entry_date

    @property
    def payment_return_date(self):
        """Gets the payment_return_date of this RecordedPayment.  # noqa: E501
        The date on which the ACH returns is received by EBPP.  # noqa: E501

        :return: The payment_return_date of this RecordedPayment. 
        :rtype: str
        """
        return self._payment_return_date

    @property
    def comments(self):
        """Gets the comments of this RecordedPayment.  # noqa: E501
        Comments that can be used to recollect the operation performed on the resource object.  # noqa: E501

        :return: The comments of this RecordedPayment. 
        :rtype: str
        """
        return self._comments

    @property
    def return_code(self):
        """Gets the return_code of this RecordedPayment.  # noqa: E501
        The ACH return code or the chargeback code received from Visa/Mastercard, in the event of returned payments.  # noqa: E501

        :return: The return_code of this RecordedPayment. 
        :rtype: str
        """
        return self._return_code

    @property
    def customer(self):
        """Gets the customer of this RecordedPayment.  # noqa: E501

        :return: The customer of this RecordedPayment. 
        :rtype: Customer
        """
        return self._customer

    @property
    def funding_account(self):
        """Gets the funding_account of this RecordedPayment.  # noqa: E501

        :return: The funding_account of this RecordedPayment. 
        :rtype: FundingAccount
        """
        return self._funding_account

    @property
    def customer_account(self):
        """Gets the customer_account of this RecordedPayment.  # noqa: E501

        :return: The customer_account of this RecordedPayment. 
        :rtype: CustomerAccount
        """
        return self._customer_account

    @property
    def audit_info(self):
        """Gets the audit_info of this RecordedPayment.  # noqa: E501

        :return: The audit_info of this RecordedPayment. 
        :rtype: AuditInfo
        """
        return self._audit_info

    @property
    def id_payment(self):
        """Gets the id_payment of this RecordedPayment.  # noqa: E501
        The unique identifier assigned by EBPP to the payment.  # noqa: E501

        :return: The id_payment of this RecordedPayment. 
        :rtype: str
        """
        return self._id_payment

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        super(RecordedPayment, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        super(RecordedPayment, self).vide_channel(channel)
        return self
    def with_details(self, payment_amount_type=None, amount=None, payment_date=None, payment_method=None):
        """
        :param payment_amount_type: The payment_amount_type is of type str.
        :param amount: The amount is of type str.
        :param payment_date: The payment_date is of type str.
        :param payment_method: The payment_method is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._payment_amount_type = payment_amount_type
        self._amount = amount
        self._payment_date = payment_date
        self._payment_method = payment_method
        return self
    def from_account(self, funding_account=None):
        """
        :param funding_account: The funding_account is of type FundingAccount.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._funding_account = funding_account
        return self
    def with_token(self, token=None):
        """
        :param token: The token is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._token = token
        return self
    def to_account(self, customer_account=None):
        """
        :param customer_account: The customer_account is of type CustomerAccount.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._customer_account = customer_account
        return self
    def with_reference(self, payment_reference=None):
        """
        :param payment_reference: The payment_reference is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._payment_reference = payment_reference
        return self
    def with_custom_fields(self, custom_fields=None):
        """
        :param custom_fields: The custom_fields is of type dict(str, str).
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._custom_fields = custom_fields
        return self
    def with_memo(self, comments=None):
        """
        :param comments: The comments is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._comments = comments
        return self
    def with_fee(self, fee=None):
        """
        :param fee: The fee is of type Fee.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._fee = fee
        return self
    def with_customer(self, customer=None):
        """
        :param customer: The customer is of type Customer.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        self._customer = customer
        return self
    ## support methods
    def deleted_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        super(RecordedPayment, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def created_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        super(RecordedPayment, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    def delete(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        try:
            super(RecordedPayment, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.delete_record_payment_handler import DeleteRecordPaymentHandler
            return DeleteRecordPaymentHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def create(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of RecordedPayment. 
        :rtype: RecordedPayment
        """
        try:
            super(RecordedPayment, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.record_payment_handler import RecordPaymentHandler
            return RecordPaymentHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, id=None, url=None, fee=None, status=None, payment_method=None, token=None, confirmation_number=None, custom_fields=None, currency_code3d=None, amount=None, payment_date=None, payment_schedule_type=None, payment_amount_type=None, payment_reference=None, payment_entry_date=None, payment_return_date=None, comments=None, return_code=None, customer=None, funding_account=None, customer_account=None, audit_info=None, id_payment=None):
        if obj is None:
            obj = RecordedPayment()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if id is not None:
            obj._id = id
        if url is not None:
            obj._url = url
        if fee is not None:
            obj._fee = fee
        if status is not None:
            obj._status = status
        if payment_method is not None:
            obj._payment_method = payment_method
        if token is not None:
            obj._token = token
        if confirmation_number is not None:
            obj._confirmation_number = confirmation_number
        if custom_fields is not None:
            obj._custom_fields = custom_fields
        if currency_code3d is not None:
            obj._currency_code3d = currency_code3d
        if amount is not None:
            obj._amount = amount
        if payment_date is not None:
            obj._payment_date = payment_date
        if payment_schedule_type is not None:
            obj._payment_schedule_type = payment_schedule_type
        if payment_amount_type is not None:
            obj._payment_amount_type = payment_amount_type
        if payment_reference is not None:
            obj._payment_reference = payment_reference
        if payment_entry_date is not None:
            obj._payment_entry_date = payment_entry_date
        if payment_return_date is not None:
            obj._payment_return_date = payment_return_date
        if comments is not None:
            obj._comments = comments
        if return_code is not None:
            obj._return_code = return_code
        if customer is not None:
            obj._customer = customer
        if funding_account is not None:
            obj._funding_account = funding_account
        if customer_account is not None:
            obj._customer_account = customer_account
        if audit_info is not None:
            obj._audit_info = audit_info
        if id_payment is not None:
            obj._id_payment = id_payment
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(RecordedPayment, self).to_dict()
        for attr, _ in six.iteritems(RecordedPayment.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RecordedPayment, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
