# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class RecordedPayments(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'list': 'list[RecordedPayment]',	
        'total_results_count': 'str',	
        'has_more_results': 'str',	
        'url': 'str',	
        'from_index': 'str',	
        'to_index': 'str',	
        'query_id': 'str'	
    }    

    attribute_map = {
        'list': 'list', 
        'total_results_count': 'total_results_count', 
        'has_more_results': 'has_more_results', 
        'url': 'url', 
        'from_index': 'from_index', 
        'to_index': 'to_index', 
        'query_id': 'query_id' 
    }
    
    def __init__(self):
        super(RecordedPayments, self).__init__()
        self._list = None
        self._total_results_count = None
        self._has_more_results = None
        self._url = None
        self._from_index = None
        self._to_index = None
        self._query_id = None

    @property
    def list(self):
        """Gets the list of this RecordedPayments.  # noqa: E501

        :return: The list of this RecordedPayments. 
        :rtype: list[RecordedPayment]
        """
        return self._list

    @property
    def total_results_count(self):
        """Gets the total_results_count of this RecordedPayments.  # noqa: E501
        total number of payments available with matching criteria  # noqa: E501

        :return: The total_results_count of this RecordedPayments. 
        :rtype: str
        """
        return self._total_results_count

    @property
    def has_more_results(self):
        """Gets the has_more_results of this RecordedPayments.  # noqa: E501
        true if there are more results or pages  # noqa: E501

        :return: The has_more_results of this RecordedPayments. 
        :rtype: str
        """
        return self._has_more_results

    @property
    def url(self):
        """Gets the url of this RecordedPayments.  # noqa: E501
        This URL fetches the next or previous set of payments, based on the presence of after_object or before_object parameter.  # noqa: E501

        :return: The url of this RecordedPayments. 
        :rtype: str
        """
        return self._url

    @property
    def from_index(self):
        """Gets the from_index of this RecordedPayments.  # noqa: E501
        The id of the object before which the previous set of objects are to be retrieved.  # noqa: E501

        :return: The from_index of this RecordedPayments. 
        :rtype: str
        """
        return self._from_index

    @property
    def to_index(self):
        """Gets the to_index of this RecordedPayments.  # noqa: E501
        The id of the object after which the next set of objects are to be retrieved.  # noqa: E501

        :return: The to_index of this RecordedPayments. 
        :rtype: str
        """
        return self._to_index

    @property
    def query_id(self):
        """Gets the query_id of this RecordedPayments.  # noqa: E501
        The query id of the object.  # noqa: E501

        :return: The query_id of this RecordedPayments. 
        :rtype: str
        """
        return self._query_id

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of RecordedPayments. 
        :rtype: RecordedPayments
        """
        super(RecordedPayments, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of RecordedPayments. 
        :rtype: RecordedPayments
        """
        super(RecordedPayments, self).vide_channel(channel)
        return self
    def find_by(self, filter_parameters=None):
        if(filter_parameters):

            if(len(filter_parameters['from_index'])>0):
                self._from_index = filter_parameters['from_index'][0]

            if(len(filter_parameters['to_index'])>0):
                self._to_index = filter_parameters['to_index'][0]

            if(len(filter_parameters['query_id'])>0):
                self._query_id = filter_parameters['query_id'][0]
        return self
    def for_customer(self, ):
        """
        
        :return: The Instance of RecordedPayments. 
        :rtype: RecordedPayments
        """
        return self
    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, list=None, total_results_count=None, has_more_results=None, url=None, from_index=None, to_index=None, query_id=None):
        if obj is None:
            obj = RecordedPayments()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if list is not None:
            obj._list = list
        if total_results_count is not None:
            obj._total_results_count = total_results_count
        if has_more_results is not None:
            obj._has_more_results = has_more_results
        if url is not None:
            obj._url = url
        if from_index is not None:
            obj._from_index = from_index
        if to_index is not None:
            obj._to_index = to_index
        if query_id is not None:
            obj._query_id = query_id
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(RecordedPayments, self).to_dict()
        for attr, _ in six.iteritems(RecordedPayments.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RecordedPayments, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
