# coding: utf-8

from orbipay_paymentsapi_client.wrappers.base import Base
import six
import pprint
from orbipay_paymentsapi_client.orbipay_api_exception import OrbipayApiException

class FinalClass(type):
    def __new__(cls, name, bases, classdict):
        for b in bases:
            if isinstance(b, FinalClass):
                raise TypeError("type '{0}' is not an acceptable base type".format(b.__name__))
        return type.__new__(cls, name, bases, dict(classdict))
        

class Refund(Base):

    __metaclass__ = FinalClass
    
    swagger_types = {
        'refund_reference': 'str',	
        'id': 'str',	
        'url': 'str',	
        'refund_type': 'str',	
        'refund_amount': 'str',	
        'fee_refund': 'FeeRefund',	
        'status': 'str',	
        'confirmation_number': 'str',	
        'refund_date': 'str',	
        'refund_entry_date': 'str',	
        'expected_refund_settlement_date': 'str',	
        'custom_fields': 'dict(str, str)',	
        'comments': 'str',	
        'source_payment': 'SourcePayment',	
        'payment_network_response': 'PaymentNetworkResponse',	
        'audit_info': 'AuditInfo',	
        'id_payment': 'str',	
        'id_customer': 'str',	
        'id_refund': 'str'	
    }    

    attribute_map = {
        'refund_reference': 'refund_reference', 
        'id': 'id', 
        'url': 'url', 
        'refund_type': 'refund_type', 
        'refund_amount': 'refund_amount', 
        'fee_refund': 'fee_refund', 
        'status': 'status', 
        'confirmation_number': 'confirmation_number', 
        'refund_date': 'refund_date', 
        'refund_entry_date': 'refund_entry_date', 
        'expected_refund_settlement_date': 'expected_refund_settlement_date', 
        'custom_fields': 'custom_fields', 
        'comments': 'comments', 
        'source_payment': 'source_payment', 
        'payment_network_response': 'payment_network_response', 
        'audit_info': 'audit_info', 
        'id_payment': 'id_payment', 
        'id_customer': 'id_customer', 
        'id_refund': 'id_refund' 
    }
    
    def __init__(self, id_refund=None):
        super(Refund, self).__init__()
        
        self._refund_reference = None
        self._id = None
        self._url = None
        self._refund_type = None
        self._refund_amount = None
        self._fee_refund = None
        self._status = None
        self._confirmation_number = None
        self._refund_date = None
        self._refund_entry_date = None
        self._expected_refund_settlement_date = None
        self._custom_fields = None
        self._comments = None
        self._source_payment = None
        self._payment_network_response = None
        self._audit_info = None
        self._id_payment = None
        self._id_customer = None
        self._id_refund = None
        
        self._id_refund = id_refund

    @property
    def refund_reference(self):
        """Gets the refund_reference of this Refund.  # noqa: E501
        The unique identifier in the client system for the refund.  # noqa: E501

        :return: The refund_reference of this Refund. 
        :rtype: str
        """
        return self._refund_reference

    @property
    def id(self):
        """Gets the id of this Refund.  # noqa: E501
        The unique identifier assigned by EBPP to the refund.  # noqa: E501

        :return: The id of this Refund. 
        :rtype: str
        """
        return self._id

    @property
    def url(self):
        """Gets the url of this Refund.  # noqa: E501
        This URL fetches the details of refund.  # noqa: E501

        :return: The url of this Refund. 
        :rtype: str
        """
        return self._url

    @property
    def refund_type(self):
        """Gets the refund_type of this Refund.  # noqa: E501
        The portion of the payment amount that is to be or has been refunded. The original payment can be refunded fully including the amount and the fee components, or partially, for just the amount or the fee component. Refund can also be initiated for an arbitrary amount limited by the original payment amount.  # noqa: E501

        :return: The refund_type of this Refund. 
        :rtype: str
        """
        return self._refund_type

    @property
    def refund_amount(self):
        """Gets the refund_amount of this Refund.  # noqa: E501
        The refund amount upto 2 decimal places.  # noqa: E501

        :return: The refund_amount of this Refund. 
        :rtype: str
        """
        return self._refund_amount

    @property
    def fee_refund(self):
        """Gets the fee_refund of this Refund.  # noqa: E501

        :return: The fee_refund of this Refund. 
        :rtype: FeeRefund
        """
        return self._fee_refund

    @property
    def status(self):
        """Gets the status of this Refund.  # noqa: E501
        The status of the refund.  # noqa: E501

        :return: The status of this Refund. 
        :rtype: str
        """
        return self._status

    @property
    def confirmation_number(self):
        """Gets the confirmation_number of this Refund.  # noqa: E501
        The confirmation number or reference provided to the customer for the refund.  # noqa: E501

        :return: The confirmation_number of this Refund. 
        :rtype: str
        """
        return self._confirmation_number

    @property
    def refund_date(self):
        """Gets the refund_date of this Refund.  # noqa: E501
        The date on which the refund is scheduled to be made. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The refund_date of this Refund. 
        :rtype: str
        """
        return self._refund_date

    @property
    def refund_entry_date(self):
        """Gets the refund_entry_date of this Refund.  # noqa: E501
        The date on which the refund captured in EBPP. It must be in ISO8601 full-date format, namely, YYYY-MM-DD.  # noqa: E501

        :return: The refund_entry_date of this Refund. 
        :rtype: str
        """
        return self._refund_entry_date

    @property
    def expected_refund_settlement_date(self):
        """Gets the expected_refund_settlement_date of this Refund.  # noqa: E501
        The date by which the credits would be posted to the original funding account. ISO8601 full-date format, YYYY-MM-DD  # noqa: E501

        :return: The expected_refund_settlement_date of this Refund. 
        :rtype: str
        """
        return self._expected_refund_settlement_date

    @property
    def custom_fields(self):
        """Gets the custom_fields of this Refund.  # noqa: E501
        The additional information or meta-information that EBPP can accept, maintain and transmit back to the client. The custom fields need to be configured with EBPP before they can be accepted. EBPP would reject custom fields that are not pre-configured. Please contact <a href = \"mailto: support@billerpayments.com\">support@billerpayments.com</a> for more information on configuring and using custom fields.  # noqa: E501

        :return: The custom_fields of this Refund. 
        :rtype: dict(str, str)
        """
        return self._custom_fields

    @property
    def comments(self):
        """Gets the comments of this Refund.  # noqa: E501
        Comments that can be used to recollect the operation performed on the resource object. API clients need to ensure that no sensitive information is passed in the memo. Alacriti (EBPP) is not responsible for the security of any sensitive information that may be passed as part of the memo.  # noqa: E501

        :return: The comments of this Refund. 
        :rtype: str
        """
        return self._comments

    @property
    def source_payment(self):
        """Gets the source_payment of this Refund.  # noqa: E501

        :return: The source_payment of this Refund. 
        :rtype: SourcePayment
        """
        return self._source_payment

    @property
    def payment_network_response(self):
        """Gets the payment_network_response of this Refund.  # noqa: E501

        :return: The payment_network_response of this Refund. 
        :rtype: PaymentNetworkResponse
        """
        return self._payment_network_response

    @property
    def audit_info(self):
        """Gets the audit_info of this Refund.  # noqa: E501

        :return: The audit_info of this Refund. 
        :rtype: AuditInfo
        """
        return self._audit_info

    @property
    def id_payment(self):
        """Gets the id_payment of this Refund.  # noqa: E501
        The unique identifier assigned by EBPP to the payment.  # noqa: E501

        :return: The id_payment of this Refund. 
        :rtype: str
        """
        return self._id_payment

    @property
    def id_customer(self):
        """Gets the id_customer of this Refund.  # noqa: E501
        The unique identifier assigned by EBPP to the customer.  # noqa: E501

        :return: The id_customer of this Refund. 
        :rtype: str
        """
        return self._id_customer

    @property
    def id_refund(self):
        """Gets the id_refund of this Refund.  # noqa: E501
        The unique identifier assigned by EBPP to the refund.  # noqa: E501

        :return: The id_refund of this Refund. 
        :rtype: str
        """
        return self._id_refund

    def for_client(self, client_key=None):
        """
        :param client_key: The client_key is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        super(Refund, self).for_client(client_key)
        return self
    def vide_channel(self, channel=None):
        """
        :param channel: The channel is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        super(Refund, self).vide_channel(channel)
        return self
    def with_details(self, refund_amount=None):
        """
        :param refund_amount: The refund_amount is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        self._refund_amount = refund_amount
        return self
    def with_reference(self, refund_reference=None):
        """
        :param refund_reference: The refund_reference is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        self._refund_reference = refund_reference
        return self
    def with_memo(self, comments=None):
        """
        :param comments: The comments is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        self._comments = comments
        return self
    def with_status(self, status=None):
        """
        :param status: The status is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        self._status = status
        return self
    def for_customer(self, id_customer=None):
        """
        :param id_customer: The id_customer is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        self._id_customer = id_customer
        return self
    def for_payment(self, id_payment=None):
        """
        :param id_payment: The id_payment is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        self._id_payment = id_payment
        return self
    def with_custom_fields(self, custom_fields=None):
        """
        :param custom_fields: The custom_fields is of type dict(str, str).
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        self._custom_fields = custom_fields
        return self
    def with_fee_refund(self, fee_refund=None):
        """
        :param fee_refund: The fee_refund is of type FeeRefund.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        self._fee_refund = fee_refund
        return self
    ## support methods
    def created_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        super(Refund, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    ## support methods
    def retrieved_by(self, requestor=None, requestor_type=None):
        """
        :param requestor: The requestor is of type str.
        :param requestor_type: The requestor_type is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        super(Refund, self).requested_by(requestor, requestor_type)
        return self
    ## support methods end
    def create(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        try:
            super(Refund, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.create_refund_handler import CreateRefundHandler
            return CreateRefundHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    def get(self, invocation_context=None, live_mode=None, api_end_point=None):
        """
        :param invocation_context: The invocation_context is of type str.
        :param live_mode: The live_mode is of type bool.
        :param api_end_point: The api_end_point is of type str.
        
        :return: The Instance of Refund. 
        :rtype: Refund
        """
        try:
            super(Refund, self)._with_context(invocation_context, live_mode, api_end_point)
            from orbipay_paymentsapi_client.handlers.get_refund_handler import GetRefundHandler
            return GetRefundHandler().process(self)
        except OrbipayApiException as e:
            raise e
        except Exception as e:
            six.raise_from(OrbipayApiException(reason=e.message if hasattr(e, 'message') else None, args=e.args, http_status_code= "0"), e)

    @staticmethod
    def __init_object(obj=None, invocation_context=None, live_mode=None, api_end_point=None, http_status_code=None, refund_reference=None, id=None, url=None, refund_type=None, refund_amount=None, fee_refund=None, status=None, confirmation_number=None, refund_date=None, refund_entry_date=None, expected_refund_settlement_date=None, custom_fields=None, comments=None, source_payment=None, payment_network_response=None, audit_info=None, id_payment=None, id_customer=None, id_refund=None):
        if obj is None:
            obj = Refund()

        method = getattr(Base, '_' + 'Base' + '__init_object')
        if method is not None:
            obj = method(obj, invocation_context=invocation_context, live_mode=live_mode, api_end_point=api_end_point, http_status_code=http_status_code)

        if refund_reference is not None:
            obj._refund_reference = refund_reference
        if id is not None:
            obj._id = id
        if url is not None:
            obj._url = url
        if refund_type is not None:
            obj._refund_type = refund_type
        if refund_amount is not None:
            obj._refund_amount = refund_amount
        if fee_refund is not None:
            obj._fee_refund = fee_refund
        if status is not None:
            obj._status = status
        if confirmation_number is not None:
            obj._confirmation_number = confirmation_number
        if refund_date is not None:
            obj._refund_date = refund_date
        if refund_entry_date is not None:
            obj._refund_entry_date = refund_entry_date
        if expected_refund_settlement_date is not None:
            obj._expected_refund_settlement_date = expected_refund_settlement_date
        if custom_fields is not None:
            obj._custom_fields = custom_fields
        if comments is not None:
            obj._comments = comments
        if source_payment is not None:
            obj._source_payment = source_payment
        if payment_network_response is not None:
            obj._payment_network_response = payment_network_response
        if audit_info is not None:
            obj._audit_info = audit_info
        if id_payment is not None:
            obj._id_payment = id_payment
        if id_customer is not None:
            obj._id_customer = id_customer
        if id_refund is not None:
            obj._id_refund = id_refund
        return obj
    
    def to_dict(self):
        """Returns the model properties as a dict"""
        result = super(Refund, self).to_dict()
        for attr, _ in six.iteritems(Refund.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Refund, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())
